<?php

namespace App;

use Illuminate\Database\Eloquent\Model;

/**
 * Class Annotation
 * @package App
 */
class Annotation extends Model
{

    protected $guarded = [];
    protected $hidden = ["created_at", "updated_at","pivot","gitlab_commit_description","vid","uid"];

    public function annotationvalue()
    {
        return $this->hasOne(AnnotationValue::class);
    }

    /**
     * Editors
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function editors(){
        return $this->belongsToMany(Editor::class);
    }

    /**
     * Annotators
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function annotators(){
        return $this->belongsToMany(Annotator::class);
    }

    /**
     * Preparations
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function preparations(){
        return $this->hasMany(Preparation::class);
    }

    /**
     * Documents
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function documents(){
        return $this->belongsToMany(Document::class);
    }

    /**
     * Corpora
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function corpus(){
        return $this->belongsTo(Corpus::class);
    }

    /**
     * The publications that belong to the Annotation.
     */
    public function publications()
    {
        return $this->hasMany(Publication::class);
    }

}
