<?php

namespace App;

/**
 * Class Corpus
 * @package App
 */
class Corpus extends Model
{

    protected $guarded = array();
    protected $hidden = ["created_at", "updated_at","pivot", "gitlab_commit_sha", "gitlab_commit_date", "gitlab_commit_description","guidelines_elasticsearch_index","elasticsearch_index","elasticsearch_id","vid","uid"];

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function corpusprojects(){
        return $this->belongsToMany(CorpusProject::class);
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function users() {
        return $this->belongsToMany(User::class)->withPivot('role_id');
    }

    /**
     * The publications that belong to the Corpus.
     */
    public function publications()
    {
        return $this->hasMany(Publication::class);
    }

    /**
     * The documents that belong to the Corpus.
     */
    public function documents()
    {
        return $this->hasMany(Document::class);
    }

    /**
     * Annotations
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function annotations(){
        return $this->hasMany(Annotation::class);
    }

    /**
     * CorpusFiles
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function corpusfiles(){
        return $this->hasMany(CorpusFile::class);
    }


    /**
     * External environments
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function externalenvironments(){
        return $this->hasMany(ExternalEnvironment::class);
    }

    /**
     * boardmessages
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function boardmessages() {
        return $this->hasMany(BoardMessage::class);
    }

    /**
     * The dois that belong to the corpus.
     */
    public function doi()
    {
        return $this->hasOne(Doi::class);
    }
}