<?php
/**
 * Created by PhpStorm.
 * User: rolfguescini
 * Date: 06.07.17
 * Time: 17:12
 */
namespace App\Custom;
use Illuminate\Http\Request;

interface ElasticsearchInterface {
    public function getPublishedCorpora();
    public function getCorpus($id,$full,$index);
    public function deleteCorpus($id);
    public function deleteIndexedObject($index,$params);
    public function setHasAnnotations($data, $corpusid, $corpusindex, $index, $deleted_id);
    public function setHasDocuments($data, $corpusid, $corpusindex, $index, $deleted_id);
    public function setInDocuments($data, $corpusid, $index);
    public function getDocument($id,$full,$index);
    public function getAnnotation($id,$full,$index);
    public function getAnnotationByNameAndCorpusId($name, $corpusId, $fields,$index);
    public function getAnnotationsByDocumentId($documentId,$index,$fields);
    public function getAnnotationGroups($matchdata,$index);
    public function getGuidelinesByCorpus($corpusId,$index);
    public function getGuidelinesByCorpusAndAnnotationId($corpusId,$annotationName,$index);
    public function getFormatsByCorpus($corpusId,$index);
    public function search($index, $field, $term);
    public function searchMain($searchData,$indexes,$source,$from,$size);
    public function searchMainWildCards($searchData,$indexes,$source,$from,$size);
    public function filterSearch($searchData, $fullTextQuery,  $index, $from, $size, $source, $hasWildCards);

    public function listAllPublished($searchData,$randomIndexes);
    public function searchCorpusIndex($searchData);
    public function corpusRangeSearch($searchData, $fullTextSearch, $fullTextFields, $filters, $highlights,$from, $size,$hasWildCards);
    public function corpusMixSearch($searchData, $fullTextQuery, $fulltextFields , $filters, $fields, $from, $size,$hasWildCards);
    public function documentDateRangeSearch($searchData, $fullTextSearch, $fullTextFields, $filters, $highlights, $from, $size, $hasWildCards);
    public function documentRangeSearch($searchData, $fullTextSearch, $fullTextFields, $filters, $highlights, $from, $size,$hasWildCards);
    public function documentMixSearch($searchData, $fullTextQuery, $fulltextFields, $filters, $fields, $from, $size,$hasWildCards);
    public function searchDocumentIndex($searchData);
    public function searchDocumentIndexWithParam(Request $request);
    public function getSearchTotal($searchData,$index);
    public function searchAnnotationIndex($searchData);
    public function getCorpusByDocument($searchData,$documentData,$index);
    public function getAnnotationByDocument($searchData,$documentData);
    public function getCorpusTitlesByDocument($searchData,$documentData);
    public function getCorpusByAnnotation($searchData);
    public function getDocumentByCorpus($searchData,$corpusData,$fields,$index);
    public function getDocumentsByAnnotationAndCorpusId($documentList,$corpusId,$index);
    public function getDocumentsByDocumentId($documentids,$index);
    public function getAnnotationsByCorpus($corpusId, $annotationsIds, $index, $fields);
    public function getAnnotationsByCorpusId($corpusId,$index,$fields);
    public function updatePublicationStatus($index,$ids);

    public function getAllPublishedCorpora();

    public function getLatestPublishedCorpora($indexes,$source,$from,$size);
    public function getLatestPublishedDocuments($indexes,$source,$from,$size);
    public function getLatestPublishedAnnotations($indexes,$source,$from,$size);

    public function getPublishedCorpus($id,$index);
    public function getAllPublishedDocumentsByCorpus($corpus_id,$document_index);
    Public function getAllPublishedAnnotationsByCorpus($corpus_id,$annotationIndex);
    public function getAllPublishedDocuments();
    public function getPublishedDocument($id,$index);
    public function getAllPublishedAnnotationsByDocument($documentId,$index);
    public function getAllPublishedAnnotations();
    public function getAllPublishedDocumentsByAnnotation($annotationId,$index);
    public function getPublishedAnnotation($id,$index);
    public function getSuggestions($searchData);

    public function getDocumentsByAnnotation($searchData,$annotationData);
    public function getCorporaByAnnotation($searchData,$annotationData,$index);
    public function getHitsCount($indexes,$query);

    public function createIndex($params);
    public function reIndex($params);
    public function postReindex($index, $postData);
    public function createMappedIndex($indexMappingPath, $index_id);
    public function createReindex($index, $matchQuery, $new_corpus_id);
    public function setDocumentAndAnnotationsToPublished($params);
    public function unsetDocumentAndAnnotationsToPublished($params);
    public function setCorpusToPublished($params);
    public function unsetCorpusToPublished($params);
    public function deleteIndex($params);
    public function truncateIndex($index);
    public function postToIndex($params);
    public function updateDocumentFieldsInAnnotation($new_annotation_index,$annotation_ids);
    public function updateInCorpora($index, $ids, $corpus_id);
    public function setMapping($params);
    public function getElasticIdByObjectId($index,$params);
    public function setWorkflowStatusByCorpusId($corpus_id);

    /**
     * Helpers
     */

    public function checkForKey($array, $key);
    public function removeKey($array, $key);
    public function doSearch($params);
    public function pingServer($wantsJSON);
    public function fixElasticsearchIds($corpusId,$corpusIndex,$documentIndex,$annotationIndex);
}