<?php
/**
 * Created by PhpStorm.
 * User: rolfguescini
 * Date: 06.12.17
 * Time: 10:04
 */

namespace App\Custom;


interface LaudatioUtilsInterface
{
    public function parseXMLToJson($xml, $options);

    public function setCorpusAttributes($json,$params);
    public function updateCorpusAttributes($params,$corpusId);
    public function updateCorpusFileAttributes($corpus);
    public function duplicateCorpus($oldCorpus, $data);
    public function corpusIsVersioned($corpusId);
    public function setDocumentAttributes($json,$corpusId,$uid,$fileName,$isDir);
    public function updateDocumentAttributes($params,$documentId);
    public function documentIsVersioned($documentId);
    public function setAnnotationAttributes($json,$corpusId,$uid,$fileName,$isDir);
    public function updateAnnotationAttributes($params,$annotationId);
    public function setPreparationAttributes($json,$annotationId,$corpusId,$isDir);
    public function annotationIsVersioned($annotationId);
    public function checkForElasticsearchIds($corpusId);

    public function associateDocumentsToCorpus($documents,$corpusId);
    public function associateAnnotationsToDocument($annotations,$documentId);
    public function associateAnnotationsToCorpus($annotations,$corpusId);
    public function associatePreparationsToAnnotation($preparations,$annotationId);

    public function getModelByType($id,$type);
    public function getModelByFileName($fileName, $type, $isDir, $corpusId);
    public function getModelByFileAndCorpus($fileName, $type, $isDir, $corpusId);
    public function getModelByTypeElasticserchIdAndIndex($id, $index, $type);
    public function getElasticSearchIdByCorpusId($corpusid);
    public function getElasticSearchIndexByCorpusId($corpusid);
    public function getDatabaseIdByCorpusId($corpusId,$current_corpus_index);
    public function getCorpusLogoByCorpusId($corpusid,$corpus_index);
    public function getDocumentGenreByCorpusId($corpusid,$index);
    public function getCorpusAndProjectPathByCorpusId($corpusid);
    public function getCorpusPathByCorpusId($corpusid,$corpus_index);
    public function getLatestCorpusTagByCorpusAndGitlabId($corpusId, $gitLabId);
    public function deleteModels($path);
    public function deleteModel($type,$id);
    public function updateDirectoryPaths($directory_path,$corpusId);
    public function setVersionMapping($object, $type);
    public function setCommitData($commitData,$corpusId);

    public function getDirectoryPath($paths,$fileName);
    public function getCorpusTokensByCorpusId($corpusid,$corpus_index);
    public function getCorpusGitLabIdByElasticsearchId($elasticsearchId,$corpus_index);
    public function getCorpusVersion($corpusId);
    public function getWorkFlowStatus($corpusId);

    public function buildCiteFormat($data);
    public function getDOIByCorpusDatabaseId($corpusDatabaseId);
    public function getDOIByCorpusElasticsearchId($elasticsearchId,$corpus_index);
    public function getDOIByCorpusId($corpusID,$corpus_index);
    //public function getLicenseByCorpus($data);
    public function getFormatFilesByCorpusId($corpusId);
    public function getDocumentRange($data,$documentResult);
    public function getCorpusNameByCorpusId($corpusid);
    public function getCorpusNameByObjectElasticsearchId($type,$objectId);

    public function getCurrentCorpusIndexByElasticsearchId($elasticSearchId);
    public function getCurrentCorpusIndexByDocumentElasticsearchIdAndIndex($elasticSearchId,$index);
    public function getCurrentDocumentIndexByElasticsearchId($elasticSearchId);
    public function getCurrentAnnotationIndexByElasticsearchId($elasticSearchId);
    public function getCurrentCorpusIndexByAnnotationElasticsearchIdAndIndex($elasticSearchId,$index);

    public function getCurrentCorpusIndexByAnnotationElasticsearchId($elasticSearchId);
    public function getCurrentCorpusIndexByDocumentElasticsearchId($elasticSearchId);

    public function getPublishedCorpusData($corpusresponses,$elasticService, $perPage ,$sortKriterium, $currentPage);

    public function getCurrentPublishedCorpusFromDatabase($gitlab_id);

    public function setCorpusLogoSymLink($flysystem,$path);
    public function copyCorpusFiles($oldCorpus,$new_corpus_id,$new_index);
    public function updateCorpusFiles($corpus);
    public function checkForDuplicateCorpusName($corpus_name, $corpus_name_path, $corpus_project);
    public function removeMergedDuplicates($data);
    public function removeMergedNA($data);

    public function determineAdminRole($admin_roles);
    public function determineUserAdminRole($user_roles);
    public function getRoleMappingByUser($user);
    public function getInvitedByCorpus($corpusId);
    public function getInvitedByCorpusProject($projectId);
    public function pickCorpusDescription($descriptions);

    public function transformElasticSearchIndexes($corpusIndex);
    public function checkIfElasticsearchIdExists($header, $elasticSearchId,$elasticsearchIndex);
    public function hasHigherCorpusVersion($gitRepoId, $version);
    public function getGitRepoId($id, $index, $type);
    public function hasHigherCorpusVersionByGitRepoId($indicesByCorpus,$gitrepoid);
    public function pickRandomCorpora();
    public function pickLatestCorpora();
    public function pickLatestDocuments();
    public function pickLatestAnnotations();

    /* CACHE */
    public function emptyCorpusCache($corpusId,$index);
    public function emptyDocumentCache();
    public function emptyAnnotationCache();
    public function emptyCorpusFileCache($path,$corpusId);
    public function emptyCorpusDataFileCache($path,$corpusId);
    public function emptyCorpusFileInfoCache($path,$corpusId);
    public function emptyFileCaches($corpusId);
    public function emptyFlySystemFileCache();

    public function emptyDocumentCacheByCorpusId($corpusId,$index);
    public function emptyDocumentCacheByDocumentIndex($documentIndex);
    public function emptyDocumentCacheByDocumentElasticsearchId($documentId, $documentIndex);
    public function emptyAnnotationCacheByCorpusId($corpusId,$index);
    public function emptyAnnotationCacheByAnnotationIndex($index);
    public function emptyAnnotationGroupCacheByAnnotationAndCorpusId($annotationId,$corpusId, $index);
    public function emptySearchCache();
    public function emptyCorpusProjectCache();
    public function stripQuotes($text);
    public function json_validate($string);
    public function json_validator($string);
    public function convert_date_format($strs);
    public function setMarkForDeletion($type,$id,$mark);
    public function strPosInArray($string, $array);
    public function importAnnisLinks();
}
