<?php
/**
 * Created by PhpStorm.
 * User: rolfguescini
 * Date: 12.10.18
 * Time: 08:21
 */

namespace App\Exceptions;


use App\Custom\LaudatioExceptionInterface;
use Exception;
use Log;


class GitInitException extends \RuntimeException implements LaudatioExceptionInterface
{
    protected $wantsJSON = false;

    public function __construct($message = null, $code = 0, \Exception $previous = null, $wantsJSON = false)
    {
        $this->wantsJSON = $wantsJSON;
        parent::__construct($message, $code, $previous);
    }


    /**
     * Report the exception.
     *
     * @return void
     */

    public function wantsJSONFormat(){
        return $this->wantsJSON;
    }


    /**
     * Report the exception.
     *
     * @return void
     */
    public function report()
    {
    }
    /**
     * Render the exception into an HTTP response.
     *
     * @param  \Illuminate\Http\Request
     * @return \Illuminate\Http\Response
     */
    public function render(Request $request, Exception $e)
    {
        Log::info("GItINitEXCEPTION: render: ".$e->getMessage());
        //dd(get_class($request));
        // If the request wants JSON (AJAX doesn't always want JSON)
        //if ($request->wantsJson()) {
        // Define the response
        $response = [
            'errors' => 'Sorry, something went wrong.'
        ];

        // If the app is in debug mode
        if (config('app.debug')) {
            // Add the exception class name, message and stack trace to response
            $response['exception'] = get_class($e); // Reflection might be better here
            $response['message'] = $e->getMessage();
            $response['trace'] = $e->getTrace();
        }

        // Default response of 400
        $status = 400;

        // If this exception is an instance of HttpException
        if ($this->isHttpException($e)) {
            // Grab the HTTP status code from the Exception
            $status = $e->getStatusCode();
        }

        // Return a JSON response with the response array and status code
        return response()->json($response, $status);
        //}

        // Default to the parent class' implementation of handler
        return parent::render($request, $e);
    }

    public function  getUserFacingMessage(){
        return "We are really sorry, but something unexpected happened. 
        Please refresh the page and try again. If the problem persists, please report it to laudatio-support (at) hu-berlin.de.";
    }

    public function  createUserFacingMessage($message) {
        /*
         * stdClass Object
                (
                    [path] => Array
                        (
                            [0] => has already been taken
                        )

                )
         */
        $notification = new \stdClass();
        $notification->Error =  array("The initiation of the Corpus repository 
        returned an error . Please refresh the page and try again.
        If the problem persists, please report it to laudatio-support (at) hu-berlin.de and report the following error: ".$message);
        return $notification;
    }

    public function logErrorMessage($message,$context){
        $notification = array(
            'exception' => 'GitInitException',
            'payload' => $message,
            'alert_type' => 'error',
        );
        $context['notification'] = $notification;
        Log::stack(['daily', 'errorlog'])->error("GIT INIT of corpus repository failed ", $context);
    }
}