<?php
/**
 * Created by PhpStorm.
 * User: rolfguescini
 * Date: 12.10.18
 * Time: 08:21
 */

namespace App\Exceptions;


use App\Custom\LaudatioExceptionInterface;
use Exception;
use Log;

class GitNetworkException extends \RuntimeException implements LaudatioExceptionInterface
{
    protected $wantsJSON = false;

    public function __construct($message = null, $code = 0, \Exception $previous = null, $wantsJSON = false)
    {
        $this->wantsJSON = $wantsJSON;
        parent::__construct($message, $code, $previous);
    }


    /**
     * Report the exception.
     *
     * @return void
     */

    public function wantsJSONFormat(){
        return $this->wantsJSON;
    }


    public function  getUserFacingMessage(){
        return "We are really sorry, but we could not connect with the GIT backend. 
        Please refresh the page and try again. If the problem persists, please report it to laudatio-support (at) hu-berlin.de.";
    }

    public function  createUserFacingMessage($message) {
        $notification = new \stdClass();
        $notification->Error =  array("The creation of the Corpus project was unsuccessful due to a network error . Please refresh the page and try again.
        If the problem persists, please report it to laudatio-support (at) hu-berlin.de and report the following error: ".$message);
        return $notification;
    }

    public function createUserFacingJSONMessage($message) {
        $notification = array(
            'error' => 'GitNetworkException',
            'payload' =>  array("The creation of the Corpus project was unsuccessful due to a network error . Please refresh the page and try again.
        If the problem persists, please report it to laudatio-support (at) hu-berlin.de and report the following error: ".$message),
            'alert_type' => 'error',
        );
        return $notification;
    }

    public function createUserFacingProcessJSONMessage($message) {
        $notification = array(
            'error' => 'GitNetworkException',
            'payload' =>  array("The processing of the uploaded files was unsuccessful due to a network error . Please refresh the page and try again.
        If the problem persists, please report it to laudatio-support (at) hu-berlin.de and report the following error: ".$message),
            'alert_type' => 'error',
        );
        return $notification;
    }

    public function logErrorMessage($exception,$context){
        $notification = array(
            'error' => 'GitNetworkException',
            'payload' => $exception,
            'alert_type' => 'error',
        );
        $context['notification'] = $notification;
        Log::stack(['daily', 'errorlog'])->error("There was an error creating a Corpus Project", $context);
    }

}