<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use Illuminate\Foundation\Auth\AuthenticatesUsers;
use Spatie\Permission\Traits\HasRoles;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Adldap\Laravel\Facades\Adldap;
use App\Custom\LDAPInterface;
use App\User;
use App\Role;
use Log;

class LoginController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles authenticating users for the application and
    | redirecting them to your home screen. The controller uses a trait
    | to conveniently provide its functionality to your applications.
    |
    */

    use AuthenticatesUsers;

    /**
     * Where to redirect users after login.
     *
     * @var string
     */
    protected $redirectTo = '/corpusprojects';
    protected $LDAPService;

    /**
     * LoginController constructor.
     * @param LDAPInterface $ldapService
     */
    public function __construct(LDAPInterface $ldapService)
    {
        $this->middleware('guest')->except('logout');
        $this->LDAPService = $ldapService;
    }


    public function username()
    {
        return config('ldap_auth.identifiers.database.username_column');
    }

    public function doLogin(Request $request){
        $credentials = $request->only($this->username(), 'password');
        $username = $credentials[$this->username()];
        $password = $credentials['password'];

        $base_dn = config('ldap.connections.default.settings.base_dn');
        $user_format = config('laudatio.user_format','uid=%s');
        $userdn = sprintf($user_format, $username).','.$base_dn;

        if(Adldap::auth()->attempt($userdn, $password, true)) {
            $user = User::where($this->username(), $username)->first();

            if (!$user) {

                //$ldapuser = Adldap::search()->where(config('laudatio.user_attribute'), '=', $username)->first();
                //Log::info("LDAPUSER?: ".print_r($ldapuser,1));
                // the user doesn't exist in the local database, so we have to create one

                $user = new User();
                $user->username = $username;

                // you can skip this if there are no extra attributes to read from the LDAP server
                // or you can move it below this if(!$user) block if you want to keep the user always
                // in sync with the LDAP server

                $sync_attrs = $this->retrieveSyncAttributes($username);

                foreach ($sync_attrs as $field => $value) {
                    $user->{$field} = $value !== null ? $value : '';
                    //$user_data[$field] = $value !== null ? $value : '';
                }
                $user->save();
                $user->roles()->attach(2);

            }//end no user


            $intended = \URL::previous();//$request->session()->get('url.intended',url('/'));
            $urlArray = explode("/",$intended);
            $redirect = '/'.join('/',array_slice($urlArray, 3));


            if($user && !$user->authenticated){
                if($redirect == "/") {
                    $redirect = "/frontpage";
                }
                else if($redirect == "/authenticateuser") {
                    $redirect = "/corpusprojects";
                }
                $response = array('success' => true, 'redirect' => "/registerform", 'userid' => $user->id, "not_authenticated" => true, "authenticated_redirect" => $redirect);
                return response()->json($response);
            }
            else {
                // by logging the user we create the session, so there is no need to login again (in the configured time).
                // pass false as second parameter if you want to force the session to expire when the user closes the browser.
                // have a look at the section 'session lifetime' in `config/session.php` for more options.
                //$this->guard()->login($user, true);
                Auth::guard('web')->login($user, true);

                if($redirect == "/authenticateuser") {
                    $redirect = "/corpusprojects";
                }

                $response = array('success' => true, 'redirect' => $redirect, "not_authenticated" => false, "authenticated_redirect" => $redirect);
                return response()->json($response);
            }



        }
        else{
            $response = array('success' => false, 'message' => 'Invalid login credentials');
            return response()->json($response);
        }
    }

    public function doExternalLogin(Request $request){
        $credentials = $request->only('external_username', 'external_password');
        $username = $credentials['external_username'];
        $password = $credentials['external_password'];

        $saltedShaPAss = $this->generateSaltedSha512($password,$username);
        $base_dn = config('laudatio.external_base_dn','ou=users,ou=Laudatio,ou=Services,ou=Computer- und Medienservice,o=Humboldt-Universitaet zu Berlin,c=de');

        $user_format = config('laudatio.user_format','uid=%s');

        $userdn = sprintf($user_format, $username).','.$base_dn;

        if(Adldap::auth()->attempt($userdn, $saltedShaPAss, true)) {
            $user = User::where($this->username(), $username)->first();
            if (!$user) {

                // the user doesn't exist in the local database, so we have to create one

                $user = new User();
                $user->username = $username;

                // you can skip this if there are no extra attributes to read from the LDAP server
                // or you can move it below this if(!$user) block if you want to keep the user always
                // in sync with the LDAP server

                $sync_attrs = $this->retrieveExternalSyncAttributes($username);

                foreach ($sync_attrs as $field => $value) {
                    $user->{$field} = $value !== null ? $value : '';
                    //$user_data[$field] = $value !== null ? $value : '';
                }

                $user->save();
                $user->roles()->attach(2);
            }//end no user


            $intended = \URL::previous();//$request->session()->get('url.intended',url('/'));
            $urlArray = explode("/",$intended);
            $redirect = '/'.join('/',array_slice($urlArray, 3));


            if($user && !$user->authenticated){
                if($redirect == "/") {
                    $redirect = "/frontpage";
                }
                else if($redirect == "/authenticateuser") {
                    $redirect = "/corpusprojects";
                }

                $response = array('success' => true, 'redirect' => "/registerform", 'userid' => $user->id, "not_authenticated" => true, "authenticated_redirect" => $redirect);
                return response()->json($response);
            }
            else if($user && $user->authenticated){
                // by logging the user we create the session, so there is no need to login again (in the configured time).
                // pass false as second parameter if you want to force the session to expire when the user closes the browser.
                // have a look at the section 'session lifetime' in `config/session.php` for more options.
                //$this->guard()->login($user, true);
                Auth::guard('web')->login($user, true);

                $redirect = '/corpusprojects';
                $intended = \URL::previous();//$request->session()->get('url.intended',url('/'));
                $urlArray = explode("/",$intended);
                $redirect = '/'.join('/',array_slice($urlArray, 3));
                Log::info("REDIRECT: ".print_r($redirect,1));
                if($redirect == "/authenticateuser") {
                    $redirect = "/corpusprojects";
                }
                Log::info("REDIRECT AFTER: ".print_r($redirect,1));
                $response = array('success' => true, 'redirect' => $redirect);
                return response()->json($response);
            }


        }
        else{
            Log::info("doExternalLogin: Invalid login: ".print_r($userdn,1));
            $response = array('success' => false, 'message' => 'Invalid login credentials');
            return response()->json($response);
        }
    }

    public function signin(Request $request){
        return view('/auth.signin');
    }

    protected function validateLogin(Request $request)
    {
        $this->validate($request, [
            $this->username() => 'required|string|regex:/^\w+$/',
            'password' => 'required|string',
        ]);
    }

    protected function retrieveSyncAttributes($username)
    {
        $ldapuser = Adldap::search()->where(config('laudatio.user_attribute','uid'), '=', $username)->first();
        if ( !$ldapuser ) {
            // log error
            return false;
        }
        // if you want to see the list of available attributes in your specific LDAP server:
        //var_dump($ldapuser->attributes); exit;

        // needed if any attribute is not directly accessible via a method call.
        // attributes in \Adldap\Models\User are protected, so we will need
        // to retrieve them using reflection.
        $ldapuser_attrs = null;

        $attrs = [];

        foreach (config('ldap_auth.sync_attributes') as $local_attr => $ldap_attr) {

            if ( $local_attr == 'username' ) {
                continue;
            }

            $method = 'get' . $ldap_attr;
            if (method_exists($ldapuser, $method)) {
                $attrs[$local_attr] = $ldapuser->$method();
                continue;
            }

            if ($ldapuser_attrs === null) {
                $ldapuser_attrs = self::accessProtected($ldapuser, 'attributes');
            }

            if (!isset($ldapuser_attrs[$ldap_attr])) {
                // an exception could be thrown
                $attrs[$local_attr] = null;
                continue;
            }

            if (!is_array($ldapuser_attrs[$ldap_attr])) {
                $attrs[$local_attr] = $ldapuser_attrs[$ldap_attr];
            }

            if (count($ldapuser_attrs[$ldap_attr]) == 0) {
                // an exception could be thrown
                $attrs[$local_attr] = null;
                continue;
            }
            // now it returns the first item, but it could return
            // a comma-separated string or any other thing that suits you better
            $attrs[$local_attr] = $ldapuser_attrs[$ldap_attr][0];
        }
        return $attrs;
    }

    protected function retrieveExternalSyncAttributes($username) {
        //connect to the tree
        $this->LDAPService->connectADLDAP();

        $searchattrs = [];

        $ldapuser = $this->LDAPService->searchTree(
            array(
                config('laudatio.user_attribute','uid') => $username,

            ),$searchattrs);

        if ( !$ldapuser ) {
            // log error
            return false;
        }

        foreach ($searchattrs as $searchattr) {
            $attrs[$local_attr] = $ldapuser->$method();
        }

        $attrs = [];

        foreach (config('ldap_auth.external_sync_attributes') as $local_attr => $ldap_attr){
            $attrs[$local_attr] = $ldapuser->pluck('attributes.'.$ldap_attr)[0][0];
        }

        return $attrs;
    }

    public function doLogout(Request $request) {
        Auth::logout();
        return Redirect::to(\URL::previous());
    }


    protected static function accessProtected ($obj, $prop)
    {
        $reflection = new \ReflectionClass($obj);
        $property = $reflection->getProperty($prop);
        $property->setAccessible(true);
        return $property->getValue($obj);
    }

    private function encodePassword($password)
    {
        return iconv('UTF-8', 'UTF-16LE', '"'.$password.'"');
    }

    private function generateSaltedSha512($password,$salt) {
        //$hash = hash('sha512', $salt.$password);
        $hash = base64_encode(hash('sha512', $salt.$password, true));
        return $hash;
    }
}
