<?php

namespace App\Http\Controllers;

use App\Exceptions\LaudatioElasticsearchException;
use App\ExternalEnvironment;
use Illuminate\Http\Request;
use App\Custom\ElasticsearchInterface;
use App\Custom\LaudatioUtilsInterface;
use Illuminate\Pagination\LengthAwarePaginator;
use Carbon\Carbon;
use JavaScript;
use Log;
use Cache;

/**
 * Class BrowseController
 * @package App\Http\Controllers
 */
class BrowseController extends Controller
{

    protected $ElasticService;
    protected $LaudatioUtilService;
    protected $ccBaseUri;

    /**
     * BrowseController constructor.
     * @param ElasticsearchInterface $Elasticservice
     * @param LaudatioUtilsInterface $laudatioUtils
     */
    public function __construct(ElasticsearchInterface $Elasticservice, LaudatioUtilsInterface $laudatioUtils)
    {
        $this->ElasticService = $Elasticservice;
        $this->LaudatioUtilService = $laudatioUtils;
        $this->ccBaseUri = config('laudatio.ccBaseuri');
    }

    /**
     * @param null $perPage
     * @param null $sortKriterium
     * @return mixed
     */
    public function index($perPage = null, $sortKriterium = null){
        $isLoggedIn = \Auth::check();
        $user = \Auth::user();
        $corpusresponses = array();


        try{
            $corpusresponses = $this->ElasticService->pingServer();
        }
        catch (LaudatioElasticsearchException $laudatioElasticsearchException) {
            $message = $laudatioElasticsearchException->createUserFacingMessage($laudatioElasticsearchException->getMessage());
            //Log the error to file
            $laudatioElasticsearchException->logErrorMessage($laudatioElasticsearchException->getMessage(),['file' => __FILE__, 'line' => __LINE__]);
            return back()->withError($message);
        }

        if(!isset($perPage)) {
            $perPage  = 4;
        }

        $cache_page = 0;
        if(null != $perPage) {
            $cache_page = $perPage;
        }

        $cache_sorted = 0;
        if(null != $sortKriterium) {
            $cache_sorted = $sortKriterium;
        }
        $currentPage = LengthAwarePaginator::resolveCurrentPage();


        if (Cache::store('redis')->has('list_latest_published_corpora_'.$cache_page."_".$cache_sorted."_".$currentPage)) {
            $responseArray = Cache::store('redis')->get('list_latest_published_corpora_'.$cache_page."_".$cache_sorted."_".$currentPage);
        }
        else {
            $corpusresponses = $this->ElasticService->getPublishedCorpora();

            $corpusdata = array();
            $documentcount = 0;
            $annotationcount = 0;

            $entries = null;
            $perPageArray = array();
            $sortedCollection = array();

            $responseArray = $this->LaudatioUtilService->getPublishedCorpusData($corpusresponses,$this->ElasticService, $perPage ,$sortKriterium, $currentPage);
            Cache::store('redis')->forever('list_latest_published_corpora_'.$cache_page."_".$cache_sorted."_".$currentPage, $responseArray);
        }


        return view('browse.index')
            ->with('isLoggedIn', $isLoggedIn)
            ->with('corpusdata',$responseArray['entries'])
            ->with('totalCount',$responseArray['totalcount'])
            ->with('perPageArray',$responseArray['perPageArray'])
            ->with('perPage',$responseArray['perPage'])
            ->with('ccBaseUri',$this->ccBaseUri)
            ->with('user',$user);
    }

    /**
     * @param $header
     * @param $id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function show($header,$id,$index){

        $data = array();
        $isLoggedIn = \Auth::check();
        $user = \Auth::user();
        $corpusElasticId = null;
        $citeData = array();
        $formatFiles = array();
        $corpusVersion = null;
        $workFlowStatus = null;
        $corpusName = null;
        $corpusPublicationLicense = null;
        $corpus_path = "";
        $corpusFiles = "";
        $corpusId = null;
        $versions = array();
        $allVersions = array();

        try{
            $corpusresponses = $this->ElasticService->pingServer();
        }
        catch (LaudatioElasticsearchException $laudatioElasticsearchException) {
            $message = $laudatioElasticsearchException->createUserFacingMessage($laudatioElasticsearchException->getMessage());
            //Log the error to file
            $laudatioElasticsearchException->logErrorMessage($laudatioElasticsearchException->getMessage(),['file' => __FILE__, 'line' => __LINE__]);
            return back()->withError($message);
        }

        $current_corpus_index =  config("laudatio.elasticsearch_corpus_index","corpora");

        $current_document_index = config("laudatio.elasticsearch_document_index","documents");

        $current_annotation_index =  config("laudatio.elasticsearch_annotation_index","annotations");

        $current_guideline_index = config("laudatio.elasticsearch_guideline_index","guidelines");
        $corpus_path = "";

        switch ($header){
            case "corpus":
                $corpusElasticId = $id;
                $apiData = $this->ElasticService->getCorpus($id,true,$current_corpus_index);
                $documentResult = array();

                if($apiData) {
                    $data = json_decode($apiData->getContent(), true);
                    $citeData['authors'] = array();
                    for($i=0;$i< count($data['result']['corpus_editor_forename']);$i++) {
                        array_push($citeData['authors'],$data['result']['corpus_editor_forename'][$i]." ".$data['result']['corpus_editor_surname'][$i]);
                    }

                    $corpusPublicationLicense = "";
                    $corpusId = is_array($data['result']['laudatio_corpus_id']) ? $data['result']['laudatio_corpus_id'][0]: $data['result']['laudatio_corpus_id'];
                    $corpus_path = $this->LaudatioUtilService->getCorpusAndProjectPathByCorpusId($corpusId);

                    $corpusName = $data['result']['corpus_title'][0];
                    if(count($data['result']['corpus_title']) > 0){
                        $citeData['title'] = $data['result']['corpus_title'][0];
                    }
                    $citeData['version'] = $data['result']['corpus_version'][count($data['result']['corpus_version']) -1];

                    if(count($data['result']['corpus_publication_publication_date']) > 0){
                        $citeData['publishing_year'] = date('Y',strtotime($data['result']['corpus_publication_publication_date'][0]));//Carbon::createFromFormat ('Y-m-d' , $data['result'][$
                    }
                    if(count($data['result']['corpus_publication_publisher']) > 0) {
                        $citeData['publishing_institution'] = $data['result']['corpus_publication_publisher'][0];
                    }
                    if(count($data['result']['corpus_publication_license']) > 0){
                        $citeData['corpus_publication_license'] = $data['result']['corpus_publication_license'][0];
                        $corpusPublicationLicense = $data['result']['corpus_publication_license'][0];
                    }
                    if(count($data['result']['corpus_encoding_project_homepage']) > 0) {
                        $citeData['corpus_encoding_project_homepage'] = $data['result']['corpus_encoding_project_homepage'][0];
                    }
                    $citeData['published_handle'] = "";

                    $doiIdentificator = $this->LaudatioUtilService->getDOIByCorpusElasticsearchId($id,$current_corpus_index);

                    $corpusFiles = $this->LaudatioUtilService->getFormatFilesByCorpusId($data['result']['laudatio_corpus_id']);

                    $corpusVersion =  '';
                    if (isset($data['result']['publication_version'])) {
                        $corpusVersion =  $data['result']['publication_version'];
                    }

                    $workFlowStatus = $data['result']['publication_status'];
                    $corpusPublicationLicense = $data['result']['corpus_publication_license'][0];


                    $formatSearchResult = $this->ElasticService->getFormatsByCorpus($corpusId,$current_corpus_index);
                    $formats = array();
                    if(isset($formatSearchResult['aggregations'])){
                        foreach ($formatSearchResult['aggregations']['formats']['buckets'] as $formatSearchResult) {
                            array_push($formats,$formatSearchResult['key']);
                        }
                    }

                    $data['result']['formats'] = $formats;



                    if(isset($data['result']['has_documents'])){
                        $documentResult = $this->ElasticService->getDocumentByCorpus(
                            $data['result']['has_documents'],
                            array($corpusId),
                            array("document_title","document_publication_publishing_date","document_publication_place","document_list_of_annotations_name","in_corpora","document_size_extent", "has_annotations"),
                            $current_document_index
                        );
                    }


                  //dd($documentResult);
                    $document_range = "";
                    if(count($documentResult) > 0) {
                        $document_range = $this->LaudatioUtilService->getDocumentRange($data,$documentResult);
                    }


                    $corpusDocuments = array();

                    $documentcount = 0;
                    for($d = 0; $d < count($documentResult); $d++) {
                            $documentArray = array();
                            $citem = $documentResult[$d];

                            $documentArray['document_id'] = $citem['_id'];
                            $documentArray['document_index'] = $current_document_index;
                            $documentArray["document_title"] = $citem["document_title"];
                            $documentArray["has_annotations"] = $citem["has_annotations"];
                            $documentArray["document_publication_publishing_date"] = $citem["document_publication_publishing_date"];
                            $documentArray["document_publication_place"] = $citem["document_publication_place"];
                            $documentArray["document_list_of_annotations_name"] = $citem["document_list_of_annotations_name"];
                            $size_extent = '0';

                            if(array_key_exists("document_size_extent",$citem)){
                                if(is_array($citem["document_size_extent"])){
                                    if(count($citem["document_size_extent"]) > 0 && is_numeric($citem["document_size_extent"][0])){
                                        $size_extent = $citem["document_size_extent"][0];
                                    }
                                }
                                else{
                                    if(is_numeric($citem["document_size_extent"])){
                                        $size_extent = $citem["document_size_extent"];
                                    }
                                }

                            }

                            $documentArray["document_size_extent"] = $size_extent;
                            array_push($corpusDocuments, $documentArray);
                            $documentcount++;
                    }
                    $data['result']['corpusdocuments'] = $corpusDocuments;
                    $data['result']['corpusdocumentcount'] = $documentcount;

                    $annotationMapping = array();
                    $annotationcount = 0;
                    $totalannotationcount = count($data['result']['annotation_id']);
                    $foundAnnotationCorpus = array();

                    $annotationData = array();

                    if(isset($data['result']['has_annotations'])){
                        $annotationData = $this->ElasticService->getAnnotationsByCorpus(
                            $corpusId,
                            $data['result']['has_annotations'],
                            $current_annotation_index,
                            array(
                                "preparation_encoding_annotation_group",
                                "preparation_annotation_id",
                                "preparation_title",
                                "in_documents",
                                "in_corpora",
                                "preparation_author_annotator_forename",
                                "preparation_author_annotator_surname",
                                "publication_status",
                                "laudatio_corpus_id"
                            ));
                    }


                    //dd($annotationData);
                    if(!empty($annotationData)){
                        $annotators = array();
                        foreach ($annotationData as $annotationDatum) {
                            if(array_key_exists('preparation_encoding_annotation_group', $annotationDatum)){
                                if(!array_key_exists($annotationDatum['_id'],$annotators)){
                                    $annotators[$annotationDatum['_id']] = array();
                                    if(array_key_exists('preparation_author_annotator_forename', $annotationDatum)){
                                        for($a=0; $a < count($annotationDatum['preparation_author_annotator_forename']); $a++){
                                            array_push($annotators[$annotationDatum['_id']], $annotationDatum['preparation_author_annotator_forename'][$a]." ".$annotationDatum['preparation_author_annotator_surname'][$a]);
                                        }
                                    }
                                }


                                $groups = array_unique($annotationDatum['preparation_encoding_annotation_group']);
                                foreach ($groups as $group){
                                    if(!array_key_exists($group,$annotationMapping)){
                                        $annotationMapping[$group] = array();
                                    }
                                    $dataArray = array();
                                    if(array_key_exists('in_documents', $annotationDatum)){
                                        $in_documents = array_unique(array_filter($annotationDatum['in_documents'],function($elm){return !is_array($elm);}));
                                        $dataArray['document_count'] = floatval(count($in_documents));
                                    }
                                    $dataArray['title'] = $annotationDatum['preparation_title'][0];
                                    $dataArray['preparation_annotation_id'] = $annotationDatum['_id'];
                                    $dataArray['annotation_index'] = $current_annotation_index;

                                    $dataArray['annotators'] = $annotators;
                                    array_push($annotationMapping[$group],$dataArray);
                                }
                                if(!in_array($annotationDatum['preparation_annotation_id'][0],$foundAnnotationCorpus)){
                                    $annotationcount++;
                                }
                                array_push($foundAnnotationCorpus,$annotationDatum['preparation_annotation_id'][0]);
                            }
                        }

                    }

                    $allAnnotationGroupResult = $this->ElasticService->getAnnotationGroups(
                        array(
                            "field" => "laudatio_corpus_id",
                            "value" => $corpusId
                        ),
                        $current_annotation_index
                    );

                    //dd($allAnnotationGroupResult);
                    $allAnnotationGroups = array();
                    if(isset($allAnnotationGroupResult['aggregations'])){
                        foreach($allAnnotationGroupResult['aggregations']['annotations']['buckets'] as $groupdata) {
                            array_push($allAnnotationGroups,$groupdata['key']);
                        }
                    }

                    $data['result']['workflow_status'] = $workFlowStatus;
                    $data['result']['corpus_version'] = $corpusVersion;
                    $data['result']['corpusPublicationLicense'] = $corpusPublicationLicense;
                    $data['result']['corpusAnnotationGroups'] = $annotationMapping;
                    $data['result']['allAnnotationGroups'] = $allAnnotationGroups;
                    $data['result']['corpusannotationcount'] = $annotationcount;
                    $data['result']['totalcorpusannotationcount'] = $totalannotationcount;
                    //$data['result']['document_genre'] = $this->LaudatioUtilService->getDocumentGenreByCorpusId($corpusId,$current_corpus_index);
                    $data['result']['document_range'] = $document_range;
                    $data['result']['corpus_encoding_project_description'] = $this->LaudatioUtilService->pickCorpusDescription($data['result']['corpus_encoding_project_description']);
                }//end if we have data


                $publishedcorpusresponses = $this->ElasticService->getPublishedCorpora();
                //dd($publishedcorpusresponses);
                if(!empty($publishedcorpusresponses['result'][0])) {
                    $currentrepoid = $this->LaudatioUtilService->getCorpusGitLabIdByElasticsearchId($id,$current_corpus_index);
                    //dd($publishedcorpusresponses);
                    foreach($publishedcorpusresponses['result'][0] as $publishedcorpusresponse) {
                        $publishedcorpus = $publishedcorpusresponse['_source'];
                       // dd($publishedcorpus);
                        if($publishedcorpus['laudatio_corpus_gitlab_id'] == $currentrepoid) {
                            $currentElasticsearchId  = $publishedcorpusresponse['_id'];
                            $versions[$currentElasticsearchId] = array();

                            $isActive = false;
                            if($id == $currentElasticsearchId) {
                                $isActive = true;
                            }

                            $publishedApiData = $this->ElasticService->getCorpus($currentElasticsearchId,true,$current_corpus_index);
                            $publishedApiDataDecoded = json_decode($publishedApiData->getContent(), true);

                            if(isset($publishedApiDataDecoded['result']['publication_version'])) {
                                $allVersions[$currentElasticsearchId] = $this->LaudatioUtilService->hasHigherCorpusVersion($publishedcorpus['laudatio_corpus_gitlab_id'],$publishedApiDataDecoded['result']['publication_version']);
                            }
                            $published_document_range = '';
                            if(count($documentResult) > 0) {
                                $published_document_range = $this->LaudatioUtilService->getDocumentRange($publishedApiDataDecoded,$documentResult);
                            }

                            $versions[$currentElasticsearchId] = array(
                                "documents" => count($publishedcorpus['has_documents']),
                                "annotations" => count($publishedcorpus['has_annotations']),
                                "name" => $publishedcorpus['corpus_title'][0],
                                "description"  => $this->LaudatioUtilService->pickCorpusDescription($publishedcorpus['corpus_encoding_project_description']),
                                'isActive' => $isActive,
                                'elasticsearchid' => $currentElasticsearchId,
                                'elasticsearch_index' => $current_corpus_index,
                                'version' => $publishedcorpus['publication_version'],
                                'document_range' => $published_document_range,
                                'tokens' => $publishedcorpus['corpus_size_value'],
                                'corpus_publication_publication_date' => $publishedApiDataDecoded['result']['corpus_publication_publication_date'][0]
                            );

                        }
                    }
                }


                break;
            case "document":

                $apiData = $this->ElasticService->getDocument($id,true,$current_document_index);
                $data = json_decode($apiData->getContent(), true);
                $corpusElasticId = is_array($data['result']['in_corpora']) ? $data['result']['in_corpora'][0]: $data['result']['in_corpora'];

                if($corpusElasticId){
                    $corpusId = $data['result']['laudatio_corpus_id'];
                    $corpus_path = $this->LaudatioUtilService->getCorpusAndProjectPathByCorpusId($corpusId);

                    $documentCorpusdata = $this->ElasticService->getCorpusByDocument(array(array('_id' => $corpusElasticId)),array($id),$current_corpus_index);
                   //dd($documentCorpusdata);
                    $data['result']['documentCorpusdata'] = $documentCorpusdata[$id][0]['_source'];

                    $citeData['authors'] = array();
                    for($i=0;$i< count($data['result']['documentCorpusdata']['corpus_editor_forename']);$i++) {
                        array_push($citeData['authors'],$data['result']['documentCorpusdata']['corpus_editor_forename'][$i]." ".$data['result']['documentCorpusdata']['corpus_editor_surname'][$i]);
                    }

                    $corpusName = $data['result']['documentCorpusdata']['corpus_title'][0];
                    if(isset($data['result']['documentCorpusdata']['publication_version'])) {
                        $corpusVersion =  $data['result']['documentCorpusdata']['publication_version'][0];
                    }

                    $workFlowStatus = $data['result']['documentCorpusdata']['publication_status'];
                    if(count($data['result']['documentCorpusdata']['corpus_publication_license']) > 0) {
                        $corpusPublicationLicense = $data['result']['documentCorpusdata']['corpus_publication_license'][0];
                    }

                    $citeData['title'] = $data['result']['documentCorpusdata']['corpus_title'][0];
                    $citeData['version'] = $data['result']['documentCorpusdata']['corpus_version'][count($data['result']['documentCorpusdata']['corpus_version']) -1];
                    if(count($data['result']['documentCorpusdata']['corpus_publication_publication_date']) > 0) {
                        $citeData['publishing_year'] = date('Y', strtotime($data['result']['documentCorpusdata']['corpus_publication_publication_date'][0]));//Carbon::createFromFormat ('Y-m-d' , $data['result']['documentCorpusdata']['corpus_publication_publication_date'][0])->format ('Y');
                    }
                    if(count($data['result']['documentCorpusdata']['corpus_publication_publisher']) > 0) {
                        $citeData['publishing_institution'] = $data['result']['documentCorpusdata']['corpus_publication_publisher'][0];
                    }

                    if(count($data['result']['documentCorpusdata']['corpus_encoding_project_homepage']) > 0) {
                        $citeData['corpus_encoding_project_homepage'] = $data['result']['documentCorpusdata']['corpus_encoding_project_homepage'][0];
                    }

                    $citeData['published_handle'] = "";

                    $doiIdentificator = $this->LaudatioUtilService->getDOIByCorpusId($corpusElasticId,$current_corpus_index);

                    $corpusFiles = $this->LaudatioUtilService->getFormatFilesByCorpusId($data['result']['laudatio_corpus_id']);

                    $annotationMapping = array();
                    $documentannotationcount = 0;
                    $totalannotationcount = count($data['result']['document_list_of_annotations_id']);
                    $foundAnnotationDocument = array();

                    $annotationData = $this->ElasticService->getAnnotationsByDocumentId($id,$current_annotation_index, array(
                        "preparation_encoding_annotation_group",
                        "preparation_annotation_id",
                        "_id",
                        "preparation_title",
                        "in_documents",
                        "in_corpora",
                        "preparation_author_annotator_forename",
                        "preparation_author_annotator_surname",
                        "generated_id"
                    ));
                    //dd($annotationData);

                    if(!$annotationData['error'] && count($annotationData['result']) > 0){
                        $annotators = array();
                        foreach ($annotationData['result'][0] as $annotationDatum) {
                            if(array_key_exists('preparation_encoding_annotation_group', $annotationDatum['_source'])){
                                if(!array_key_exists($annotationDatum['_id'],$annotators)){
                                    $annotators[$annotationDatum['_id']] = array();
                                    if(array_key_exists('preparation_author_annotator_forename', $annotationDatum['_source'])){
                                        for($a=0; $a < count($annotationDatum['_source']['preparation_author_annotator_forename']); $a++){
                                            array_push($annotators[$annotationDatum['_id']], $annotationDatum['_source']['preparation_author_annotator_forename'][$a]." ".$annotationDatum['_source']['preparation_author_annotator_surname'][$a]);
                                        }
                                    }
                                }


                                $groups = array_unique($annotationDatum['_source']['preparation_encoding_annotation_group']);
                                foreach ($groups as $group){
                                    if(!array_key_exists($group,$annotationMapping)){
                                        $annotationMapping[$group] = array();
                                    }
                                    $dataArray = array();
                                    if(array_key_exists('in_documents', $annotationDatum['_source'])){
                                        $in_documents = array_unique(array_filter($annotationDatum['_source']['in_documents'],function($elm){return !is_array($elm);}));
                                        $dataArray['document_count'] = floatval(count($in_documents));
                                    }
                                    $dataArray['title'] = $annotationDatum['_source']['preparation_title'][0];
                                    $dataArray['preparation_annotation_id'] = $annotationDatum['_id'];
                                    $dataArray['annotation_index'] = $annotationDatum['_index'];

                                    $dataArray['annotators'] = $annotators;
                                    array_push($annotationMapping[$group],$dataArray);
                                }
                                if(!in_array($annotationDatum['_source']['preparation_annotation_id'][0],$foundAnnotationDocument)){
                                    $documentannotationcount++;
                                }
                                array_push($foundAnnotationDocument,$annotationDatum['_source']['preparation_annotation_id'][0]);
                            }
                        }

                    }

                    $allAnnotationGroupResult = $this->ElasticService->getAnnotationGroups(
                        array(
                            "field" => "laudatio_corpus_id",
                            "value" => $corpusId
                        ),
                        $current_annotation_index
                    );

                    $allAnnotationGroups = array();
                    if(isset($allAnnotationGroupResult['aggregations'])){
                        foreach($allAnnotationGroupResult['aggregations']['annotations']['buckets'] as $groupdata) {
                            array_push($allAnnotationGroups,$groupdata['key']);
                        }
                    }

                    $data['result']['workflow_status'] = $workFlowStatus;
                    $data['result']['corpus_version'] = $corpusVersion;
                    $data['result']['allAnnotationGroups'] = $allAnnotationGroups;
                    $data['result']['documentannotationcount'] = $documentannotationcount;
                    $data['result']['annotationGroups'] = $annotationMapping;
                    $data['result']['totaldocumentannotationcount'] = $totalannotationcount;
                }
                break;
            case "annotation":

                $apiData = $this->ElasticService->getAnnotation($id,true,$current_annotation_index);
                $data = json_decode($apiData->getContent(), true);

                $corpusElasticId = is_array($data['result']['in_corpora']) ? $data['result']['in_corpora'][0]: $data['result']['in_corpora'];
                //$workFlowStatus = $this->LaudatioUtilService->getWorkFlowStatus($corpusId);
                //$corpusVersion = $this->LaudatioUtilService->getCorpusVersion($corpusId);

                if($corpusElasticId){

                    $annotationCorpusdata = $this->ElasticService->getCorporaByAnnotation(array(array('_id' => $corpusElasticId)),array($id),$current_corpus_index);
                    $data['result']['annotationCorpusdata'] = $annotationCorpusdata[$id][0]['_source'];
                    $corpusId = $data['result']['laudatio_corpus_id'];
                    $corpus_path = $this->LaudatioUtilService->getCorpusAndProjectPathByCorpusId($corpusId);

                    $citeData['authors'] = array();
                    for($i=0;$i< count($data['result']['annotationCorpusdata']['corpus_editor_forename']);$i++) {
                        array_push($citeData['authors'],$data['result']['annotationCorpusdata']['corpus_editor_forename'][$i]." ".$data['result']['annotationCorpusdata']['corpus_editor_surname'][$i]);
                    }

                    $corpusName = $data['result']['annotationCorpusdata']['corpus_title'][0];
                    $corpusVersion = "";
                    if(isset($data['result']['annotationCorpusdata']['publication_version'])) {
                        $corpusVersion = $data['result']['annotationCorpusdata']['publication_version'][0];
                    }
                    $workFlowStatus =  $data['result']['annotationCorpusdata']['publication_status'];
                    if(count($data['result']['annotationCorpusdata']['corpus_publication_license']) > 0) {
                        $corpusPublicationLicense = $data['result']['annotationCorpusdata']['corpus_publication_license'][0];
                    }
                    $data['result']['workflow_status'] = $workFlowStatus;
                    $data['result']['corpus_version'] = $corpusVersion;

                    $citeData['title'] = $data['result']['annotationCorpusdata']['corpus_title'][0];
                    $citeData['version'] = $data['result']['annotationCorpusdata']['corpus_version'][count($data['result']['annotationCorpusdata']['corpus_version']) -1];
                    if(count($data['result']['annotationCorpusdata']['corpus_publication_publication_date']) > 0) {
                        $citeData['publishing_year'] = date('Y', strtotime($data['result']['annotationCorpusdata']['corpus_publication_publication_date'][0]));//Carbon::createFromFormat ('Y-m-d' , $data['result']['annotationCorpusdata']['corpus_publication_publication_date'][0])->format ('Y');
                    }
                    if(count($data['result']['annotationCorpusdata']['corpus_publication_publisher']) > 0) {
                        $citeData['publishing_institution'] = $data['result']['annotationCorpusdata']['corpus_publication_publisher'][0];
                    }

                    if(count($data['result']['annotationCorpusdata']['corpus_encoding_project_homepage']) > 0) {
                        $citeData['corpus_encoding_project_homepage'] = $data['result']['annotationCorpusdata']['corpus_encoding_project_homepage'][0];
                    }
                    $citeData['published_handle'] = "";

                    $doiIdentificator = $this->LaudatioUtilService->getDOIByCorpusId($corpusElasticId,$current_corpus_index);

                    $corpusFiles = $this->LaudatioUtilService->getFormatFilesByCorpusId($data['result']['laudatio_corpus_id']);

                    $guidelines = $this->ElasticService->getGuidelinesByCorpusAndAnnotationId($corpusElasticId,$data['result']['preparation_annotation_id'][0],$current_guideline_index);
                    //dd($guidelines);
                    $formats = array();
                    $formatSearchResult = $this->ElasticService->getFormatsByCorpus($corpusElasticId,$current_guideline_index);

                    $guidelineArray = array();
                    //dd($formatSearchResult);
                    if(isset($formatSearchResult['aggregations'])){
                        foreach ($formatSearchResult['aggregations']['formats']['buckets'] as $formatSearchResult) {
                            //dd($formatSearchResult);
                            if(!array_key_exists($formatSearchResult['key'],$guidelineArray)){
                                $guidelineArray[$formatSearchResult['key']] = array('annotations' => array());
                            }

                            foreach ($guidelines['result']['hits']['hits'] as $guideline){
                                if(is_array($guideline['_source']['in_annotations'])) {
                                    foreach ($guideline['_source']['in_annotations'] as $annotationkey) {
                                        if ($data['result']['preparation_annotation_id'][0] == $annotationkey &&
                                            in_array($formatSearchResult['key'], $guideline['_source']['formats'])) {
                                            if (!array_key_exists($annotationkey, $guidelineArray[$formatSearchResult['key']]['annotations'])) {
                                                $guidelineArray[$formatSearchResult['key']]['annotations'][$annotationkey] = array();
                                            }

                                            if (!array_key_exists($guideline['_source']['id'], $guidelineArray[$formatSearchResult['key']]['annotations'][$annotationkey])) {
                                                $guidelineArray[$formatSearchResult['key']]['annotations'][$annotationkey][$guideline['_source']['id']] = $guideline['_source']['desc'];
                                            }
                                        }
                                    }
                                }
                                else{
                                    if($data['result']['preparation_annotation_id'][0] == $guideline['_source']['in_annotations'] &&
                                        $formatSearchResult['key'] == $guideline['_source']['formats']){
                                        if(!array_key_exists($guideline['_source']['in_annotations'], $guidelineArray[$formatSearchResult['key']]['annotations'])){
                                            $guidelineArray[$formatSearchResult['key']]['annotations'][$guideline['_source']['in_annotations']] = array();
                                        }

                                        if(!array_key_exists($guideline['_source']['id'], $guidelineArray[$formatSearchResult['key']]['annotations'][$guideline['_source']['in_annotations']])){
                                            $guidelineArray[$formatSearchResult['key']]['annotations'][$guideline['_source']['in_annotations']][$guideline['_source']['id']] = $guideline['_source']['desc'];
                                        }
                                    }
                                }


                            }

                            array_push($formats,$formatSearchResult['key']);

                        }
                    }
                    //dd($guidelineArray);
                    if(isset($data['result']['in_documents']) && count($data['result']['in_documents']) > 0){
                        $in_documents = array_unique(array_filter($data['result']['in_documents'],function($elm){return !is_array($elm);}));
                        $documentsByAnnotation = $this->ElasticService->getDocumentsByAnnotationAndCorpusId($in_documents,$data['result']['laudatio_corpus_id'],$current_document_index);
                        $data['result']['documents'] = $documentsByAnnotation['results'];
                        $data['result']['annotationdocumentcount'] = count($documentsByAnnotation['results']);
                    }

                    $data['result']['allformats'] = $formats;
                    $data['result']['guidelines'] = $guidelineArray;
                    $data['result']['ccBaseUri'] = $this->ccBaseUri;

                }
                break;
        }


        $corpusPathArray = explode("/",$corpus_path);
     //   $data['result']['project_directorypath'] = $corpusPathArray[0];
//        $data['result']['corpus_directorypath'] = $corpusPathArray[1];

        $corpusLogo = "";
        $corpusLogobyCorpusid = $this->LaudatioUtilService->getCorpusLogoByCorpusId($corpusElasticId, $current_corpus_index);
        if(null != $corpusLogobyCorpusid) {
            $corpusLogo = $corpusLogobyCorpusid;
        }
        $data['result']['corpus_logo'] = $corpusLogo;
        $data['result']['project_directorypath'] = $corpusPathArray[0];
        $data['result']['corpus_directorypath'] = $corpusPathArray[1];
        $user_name = "";
        $user_email = "";
        if($user){
            $user_name = $user->name;
            $user_email = $user->email;
        }

        $datacite_resolve_uri = config('laudatio.datacite_resolve_uri');
        $doiIdentificator = substr($doiIdentificator,4);


        $citeData['published_handle'] = $datacite_resolve_uri.$doiIdentificator;

        $externalEnvironmentsbyCorpus = ExternalEnvironment::where('corpus_id',$corpusId)->get();

        JavaScript::put([
            "header" => $header,
            "header_id" => $id,
            "corpus_elasticsearch_id" => $corpusElasticId,
            "corpusDOIIdentificator" => $doiIdentificator,
            "corpus_id" => $corpusId,
            "datacite_resolve_uri" => $datacite_resolve_uri,
            "corpus_logo" => $corpusLogo,
            "corpus_name" => $corpusName,
            "project_directorypath" => $corpusPathArray[0],
            "corpus_directorypath" => $corpusPathArray[1],
            "corpus_path" => $corpus_path,
            "corpus_data_files" => $corpusFiles,
            "workflow_status" => $workFlowStatus,
            "corpus_version" => $corpusVersion,
            "corpusPublicationLicense" => $corpusPublicationLicense,
            "header_data" => $data,
            "citedata" => $citeData,
            "externalenvironments" => $externalEnvironmentsbyCorpus,
            "user" => $user,
            "isLoggedIn" => $isLoggedIn,
            "ccBaseUri" => $this->ccBaseUri,
            "current_corpus_index" => $current_corpus_index,
            "current_document_index" => $current_document_index,
            "current_annotation_index" => $current_annotation_index,
            'auth_user_name' => $user_name,
            'auth_user_email' => $user_email,
            'published_versions' => $versions,
            'all_published_versions' => $allVersions
        ]);
        return view('browse.showHeaders')
            ->with('isLoggedIn', $isLoggedIn)
            ->with('workflowstatus', $workFlowStatus)
            ->with('user',$user);
    }

}
