<?php

namespace App\Http\Controllers;

use App\Exceptions\CorpusProjectAlreadyExistsException;
use App\Exceptions\GitNetworkException;
use Illuminate\Http\Request;
use App\CorpusProject;
use App\Corpus;
use App\User;
use App\Role;
use App\Invite;
use App\Custom\GitRepoInterface;
use App\Custom\GitLabInterface;
use App\Custom\LaudatioUtilsInterface;
use GrahamCampbell\Flysystem\FlysystemManager;
use Response;
use Log;


/**
 * Class CorpusProjectController
 * @package App\Http\Controllers
 */
class CorpusProjectController extends Controller
{
    protected $GitRepoService;
    protected $GitLabService;
    protected $LaudatioUtilService;


    /**
     * CorpusProjectController constructor.
     * @param GitRepoInterface $Gitservice
     * @param GitLabInterface $GitLabService
     * @param FlysystemManager $flysystem
     * @param LaudatioUtilsInterface $laudatioUtils
     */
    public function __construct(GitRepoInterface $Gitservice, GitLabInterface $GitLabService, FlysystemManager $flysystem, LaudatioUtilsInterface $laudatioUtils)
    {
        $this->GitRepoService = $Gitservice;
        $this->GitLabService = $GitLabService;
        $this->flysystem = $flysystem;
        $this->connection = $this->flysystem->getDefaultConnection();
        $this->LaudatioUtilService = $laudatioUtils;
        $this->basePath = config('laudatio.basePath');
        $this->middleware(['auth', 'isCorpusAdmin'])->except('index');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {

        $isLoggedIn = \Auth::check();
        $loggedInUser = \Auth::user();

        if($isLoggedIn && (!$loggedInUser->isSuperUser() && !$loggedInUser->isCorpusAdmin())) {
            return redirect()->route('frontpage');
        }

        $CorpusProjects = CorpusProject::latest()->get();

        $corpus_projects = array();
        $project_access_mapping = array();
        $corpus_access_mapping = array();

        $corpusAccessCount = 0;
        $corpusProjectCount = 0;
        $user_roles = array();
        $foundprojectroleuserids = array();
        $foundcorpusroleuserids = array();
        if($loggedInUser) {
            foreach ($CorpusProjects as $corpusProject) {
                $admin_roles = array();
                $user_roles = array();
                if (!array_key_exists($corpusProject->id, $corpus_projects)) {
                    $corpus_projects[$corpusProject->id] = array();
                    $corpus_projects[$corpusProject->id]['user_roles'] = array();
                    $corpus_projects[$corpusProject->id]['corpora'] = array();
                    $foundprojectroleuserids[$corpusProject->id] = array();
                }

                if (!array_key_exists($corpusProject->id, $project_access_mapping)) {
                    $project_access_mapping[$corpusProject->id] = array();
                }

                $corpus_projects[$corpusProject->id]['name'] = $corpusProject->name;
                $corpus_projects[$corpusProject->id]['description'] = $corpusProject->description;
                $corpus_projects[$corpusProject->id]['directory_path'] = $corpusProject->directory_path;


                $corpusProjectUsers = $corpusProject->users()->get();

                foreach ($corpusProjectUsers as $corpusProjectUser) {
                    $user_role = array();
                    $role = Role::find($corpusProjectUser->pivot->role_id);
                    if ($role && !in_array($corpusProjectUser->id,$foundprojectroleuserids[$corpusProject->id])) {
                        $user_role['user_name'] = $corpusProjectUser->name;
                        $user_role['user_id'] = $corpusProjectUser->id;
                        if (!array_key_exists($corpusProjectUser->id, $project_access_mapping[$corpusProject->id])) {
                            $project_access_mapping[$corpusProject->id][$corpusProjectUser->id] = array();
                        }

                        $user_role['role_name'] = $role->name;
                        array_push($corpus_projects[$corpusProject->id]['user_roles'], $user_role);
                    }
                    array_push($foundprojectroleuserids[$corpusProject->id],$corpusProjectUser->id);
                }

                foreach ($corpusProject->corpora()->get() as $projectCorpus) {
                    if (!array_key_exists($projectCorpus->id, $corpus_projects[$corpusProject->id]['corpora'])) {
                        $corpus_projects[$corpusProject->id]['corpora'][$projectCorpus->id] = array();
                        $corpus_projects[$corpusProject->id]['corpora'][$projectCorpus->id]['user_roles'] = array();
                        $foundcorpusroleuserids[$projectCorpus->id] = array();
                    }
                    $corpus_projects[$corpusProject->id]['corpora'][$projectCorpus->id]['name'] = $projectCorpus->name;

                    $corpus_projects[$corpusProject->id]['corpora'][$projectCorpus->id]['publication_version'] = $projectCorpus->publication_version;
                    $corpus_projects[$corpusProject->id]['corpora'][$projectCorpus->id]['id'] = $projectCorpus->id;
                    $corpus_projects[$corpusProject->id]['corpora'][$projectCorpus->id]['corpus_directorypath'] = $projectCorpus->directory_path;
                    $corpus_projects[$corpusProject->id]['corpora'][$projectCorpus->id]['corpuspath'] = $corpusProject->directory_path . "/" . $projectCorpus->directory_path;
                    $corpus_projects[$corpusProject->id]['corpora'][$projectCorpus->id]['workflow_status'] = $projectCorpus->workflow_status;
                    $corpus_projects[$corpusProject->id]['corpora'][$projectCorpus->id]['elasticsearch_id'] = $projectCorpus->elasticsearch_id;
                    $corpus_projects[$corpusProject->id]['corpora'][$projectCorpus->id]['elasticsearch_index'] = $projectCorpus->elasticsearch_index;
                    $corpus_projects[$corpusProject->id]['corpora'][$projectCorpus->id]['corpus_logo'] = $projectCorpus->corpus_logo;
                    //dd($projectCorpus);
                    $corpusUsers = $projectCorpus->users()->get();
                    foreach ($corpusUsers as $corpusUser) {
                        $user_role = array();
                        $role = Role::find($corpusUser->pivot->role_id);
                        if ($role && !in_array($corpusUser->id,$foundcorpusroleuserids[$projectCorpus->id])) {
                            $user_role['user_name'] = $corpusUser->name;
                            $user_role['user_id'] = $corpusUser->id;
                            $user_role['role_name'] = $role->name;


                            if ($corpusUser->id == $loggedInUser->id && $projectCorpus->workflow_status == 0) {
                                $project_access_mapping[$corpusProject->id][$corpusUser->id] = true;
                                if (!array_key_exists($corpusProject->id . "_" . $projectCorpus->id, $corpus_access_mapping)) {
                                    $corpus_access_mapping[$corpusProject->id . "_" . $projectCorpus->id] = array();
                                }
                                $corpus_access_mapping[$corpusProject->id . "_" . $projectCorpus->id][$corpusUser->id] = true;
                                array_push($corpus_projects[$corpusProject->id]['corpora'][$projectCorpus->id]['user_roles'], $user_role);
                            }
                        }
                        array_push($foundcorpusroleuserids[$projectCorpus->id],$corpusUser->id);
                    }
                    // dd($project_access_mapping);
                }

                $corpusProjectUsers = $corpusProject->users()->get();
                foreach ($corpusProjectUsers as $corpusProjectUser){
                    $user_role = array();
                    $admin_role = array();

                    $corpusProjectRole = Role::find($corpusProjectUser->pivot->role_id);

                    if($corpusProjectRole){
                        if($corpusProjectRole->hasPermissionTo('Can create corpus')){
                            $admin_role['user_name'] = $corpusProjectUser->name;
                            $admin_role['user_id'] = $corpusProjectUser->id;
                            $admin_role['user_affiliation'] = $corpusProjectUser->affiliation;
                            $admin_role['role_name'] = $corpusProjectRole->name;
                            $admin_role['role_id'] = $corpusProjectRole->id;
                        }

                        if(null != $admin_role) {
                            if(!array_key_exists($corpusProjectUser->id,$admin_roles)) {
                                $admin_roles[$corpusProjectUser->id] = array();
                            }
                            array_push($admin_roles[$corpusProjectUser->id], $admin_role);
                        }

                        $user_role['user_name'] = $corpusProjectUser->name;
                        $user_role['user_affiliation'] = $corpusProjectUser->affiliation;
                        $user_role['user_id'] = $corpusProjectUser->id;
                        $user_role['role_name'] = $corpusProjectUser->name;
                        $user_role['role_id'] = $corpusProjectUser->id;


                        if(null != $user_role) {
                            if(!array_key_exists($corpusProjectUser->id,$user_roles)) {
                                $user_roles[$corpusProjectUser->id] = array();
                            }
                            array_push($user_roles[$corpusProjectUser->id],$user_role);
                        }

                    }
                }
            }





            $role_mappings = $this->LaudatioUtilService->getRoleMappingByUser($loggedInUser);
            //dd($role_mappings);
            foreach ($corpus_projects as $corpusProjectId => $corpusProject) {
                if (isset($project_access_mapping[$corpusProjectId][$loggedInUser->id])) {
                    $corpusProjectCount++;
                }
            }


            foreach($corpus_access_mapping as $projectId => $mapping) {
                if(count($mapping) > 0){
                    $corpusAccessCount++;
                }
            }

            $invites = Invite::all();
            $invitations = array();
            foreach($invites as $invite) {
                $corpusId = $invite->corpus_id;
                $userId = $invite->user_id;
                $roleId = $invite->role_id;
                $accepted = $invite->accepted;

                if($loggedInUser->id == $userId && $accepted == 0) {
                    array_push($invitations,$invite);
                }
            }
            //dd($user_roles);
            return view('project.corpusproject.index')
                ->with('corpusProjects',$corpus_projects)
                ->with('isLoggedIn', $isLoggedIn)
                ->with('invitations',$invitations)
                ->with('project_access_mapping',$project_access_mapping)
                ->with('role_mappings',$role_mappings)
                ->with('user_roles',$user_roles)
                ->with('corpus_access_mapping',$corpus_access_mapping)
                ->with('corpusAccessCount',$corpusAccessCount)
                ->with('corpusProjectCount',$corpusProjectCount)
                ->with('user',$loggedInUser);
        }
        else{
            return view('admin.adminIndex')
                ->with('isLoggedIn', $isLoggedIn)
                ->with('user',$loggedInUser);
        }
    }



    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $isLoggedIn = \Auth::check();
        $user = \Auth::user();
        $invites = Invite::all();
        $invitations = array();
        foreach($invites as $invite) {
            $corpusId = $invite->corpus_id;
            $userId = $invite->user_id;
            $roleId = $invite->role_id;
            $accepted = $invite->accepted;


            if($user->id == $userId && $accepted == 0) {
                array_push($invitations,$invite);
            }
        }
        return view('project.corpusproject.create')
            ->with('isLoggedIn', $isLoggedIn)
            ->with('invitations',$invitations)
            ->with('user',$user);
    }


    /**
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function store(Request $request)
    {
        $validated = $this->validate($request,[
            'corpusproject_name' => 'alpha_dash|not_regex:/corpus/|regex:/^[a-z0-9][a-z0-9-]*$/|required|string|max:255',
            'corpusproject_description' => 'required|string|max:2000',
        ]);

        // Create the directory structure for the Corpus Project
        $filePath = $this->GitRepoService->createProjectFileStructure($this->flysystem,request('corpusproject_name'));
        if($filePath){
            $response = new Response();
            $responsearray = array();
            
            try {
                $gitLabResponse = $this->GitLabService->createGitLabGroup(
                    $request->input('corpusproject_name'),
                    $filePath,
                    $request->input('corpusproject_description'),
                    'private'
                );
            }
            catch (GitNetworkException $gitNetworkException){
                $message = $gitNetworkException->createUserFacingMessage($gitNetworkException->getMessage());
                //Log the error to file
                $gitNetworkException->logErrorMessage($gitNetworkException->getMessage(),['file' => __FILE__, 'line' => __LINE__]);

                return back()->withError($message);
            }
            catch (CorpusProjectAlreadyExistsException $corpusProjectAlreadyExistsException) {
                $pattern='~\{(?:[^{}]|(?R))*\}~';
                $json = "";
                $messageData = array();
                if (preg_match($pattern, $this->LaudatioUtilService->stripQuotes($corpusProjectAlreadyExistsException->getMessage()), $match)) {
                    $json = str_replace(":",'"',$match[0]);
                    $json = str_replace("=>",'":', $json);
                    $messageData = json_decode($json);
                }

                $userFacingMessage = $corpusProjectAlreadyExistsException->createUserFacingMessage($messageData);
                return back()->withError($userFacingMessage);
            }



            if($gitLabResponse['id']) {
                $corpusproject = CorpusProject::create([
                    'name' => request('corpusproject_name'),
                    'description' => request('corpusproject_description'),
                    'directory_path' => $filePath,
                    'gitlab_group_path' => $gitLabResponse['path'],
                    'gitlab_id' => $gitLabResponse['id'],
                    'gitlab_web_url' => $gitLabResponse['web_url'],
                    'gitlab_parent_id' => $gitLabResponse['parent_id']
                ]);
            }
            else{
                $corpusproject = CorpusProject::create([
                    'name' => request('corpusproject_name'),
                    'description' => request('corpusproject_description'),
                    'directory_path' => $filePath,
                ]);
            }

            $user = \Auth::user();

            $projectAdminRole = Role::findById(2);
            //$user->roles()->sync($projectAdminRole);
            if($user) {
                if(!$user->roles->contains($projectAdminRole)){
                    $user->roles()->attach($projectAdminRole);
                }

                $corpusproject->users()->save($user,['role_id' => 2]);
            }


        }

        $deletecacheresult = $this->LaudatioUtilService->emptyCorpusProjectCache();

        return redirect()->route('corpusProject.index');
    }


    /**
     * @param CorpusProject $corpusproject
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function show(CorpusProject $corpusproject){
        $isLoggedIn = \Auth::check();
        $user = \Auth::user();

        $user_roles = array();
        $corpusProjectUsers = $corpusproject->users()->get();
        $corpora = $corpusproject->corpora()->get();

        //dd($corpusProjectUsers);
        foreach ($corpusProjectUsers as $corpusProjectUser){
            if(!isset($user_roles[$corpusProjectUser->id])){
                $user_roles[$corpusProjectUser->id]['roles'] = array();
            }
            $user_roles[$corpusProjectUser->id]['user_name'] = $corpusProjectUser->name;

            $role = Role::find($corpusProjectUser->pivot->role_id);
            array_push($user_roles[$corpusProjectUser->id]['roles'],$role->name);
        }


        return view('project.corpusproject.show', compact('corpusproject'))
            ->with('isLoggedIn', $isLoggedIn)
            ->with('user_roles',$user_roles)
            ->with('corpora',$corpora)
            ->with('user',$user);
    }

    /**
     * @param Request $request
     * @return $this
     */
    public function update(Request $request)
    {

        $status = "success";

        $result = array();
        $corpusproject = CorpusProject::findOrFail($request->input('projectid'));

        try{
            $corpusproject->update([
                "name" => $request->input('corpusproject_name'),
                "description" => $request->input('corpusproject_description')
            ]);
            $result['eloquent_response']  = "Project was successfully updated";
            $status = "success";
        }
        catch (\Exception $e) {
            $status = "error";
            $result['eloquent_response']  = "There was a problem updating the Project data. A message has been sent to the site administrator. Please try again later";
            //$e->getMessage();

        }


        $params = array(
            'name' => $request->input('corpusproject_name'),
            'path' => $corpusproject->gitlab_group_path,
            'description' => $request->input('corpusproject_description'),
            'visibility' => 'private'
        );

        try{
            $gitLabResponse = $this->GitLabService->updateGitLabGroup($corpusproject->gitlab_id,$params);
            if($status != 'error'){
                $status = "success";
                $result['gitlab_response']  = "Project was successfully versioned";
            }


        }
        catch (\Exception $e) {
            $status = "error";
            $result['gitlab_response'] =  "There was a problem versioning the Project data. A message has been sent to the site administrator. Please try again later";//$e->getMessage();
        }


        $response = array(
            'status' => $status,
            'message' => $result,
        );

        $deletecacheresult = $this->LaudatioUtilService->emptyCorpusProjectCache();

        return Response::json($response);
    }


    /**
     * @param CorpusProject $corpusproject
     * @return $this
     */
    public function delete(CorpusProject $corpusproject)
    {
        $isLoggedIn = \Auth::check();
        $user = \Auth::user();

        return view('project.corpusproject.delete', compact('corpusproject'))
            ->with('isLoggedIn', $isLoggedIn)
            ->with('user',$user);
    }

    /**
     * @param Request $request
     * @param CorpusProject $corpusproject
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     * @throws \Exception
     */
    public function destroy(Request $request, CorpusProject $corpusproject)
    {
        $isLoggedIn = \Auth::check();
        $user = \Auth::user();

        $projectCorpora = $corpusproject->corpora();

        if(count($projectCorpora) > 0) {
            $corpusproject->corpora()->detach();
        }

        if(count($corpusproject->users()) > 0) {
            $corpusproject->users()->detach();
        }

        $gitLabGroupId = $corpusproject->gitlab_id;

        foreach($projectCorpora as $projectCorpus) {
            $this->GitLabService->unlinkProjectFromGroup($projectCorpus->gitlab_id,$gitLabGroupId);
        }

        $this->GitLabService->deleteGitLabGroup($gitLabGroupId);

        $corpusproject->delete();
        $CorpusProjects = CorpusProject::latest()->get();

        $this->GitRepoService->deleteProjectFileStructure($this->flysystem,$corpusproject->directory_path);
        $deletecacheresult = $this->LaudatioUtilService->emptyCorpusProjectCache();

        return view('project.corpusproject.index', compact('CorpusProjects'))
            ->with('isLoggedIn', $isLoggedIn)
            ->with('user',$user);
    }

    /**
     * @param CorpusProject $corpusproject
     * @return $this
     */
    public function assignCorpora(CorpusProject $corpusproject) {
        $isLoggedIn = \Auth::check();
        $user = \Auth::user();
        $corpora = Corpus::latest()->get();
        $filteredList = array();

        foreach ($corpora as $corpus){
            if(!$corpusproject->corpora->contains($corpus)){
                array_push($filteredList,$corpus);
            }
        }

        return view('project.corpusproject.assign_corpora')
            ->with('corpusproject', $corpusproject)
            ->with('corpora', $filteredList)
            ->with('isLoggedIn', $isLoggedIn)
            ->with('user',$user);
    }

    /**
     * @param $id
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function storeCorpusRelations($id)
    {

        $this->validate(request(), [
            'corpora' => 'required',
        ]);

        $corpusproject = CorpusProject::find($id);
        foreach (request('corpora') as $corpusid){
            $corpus = Corpus::find($corpusid);
            $corpusproject->corpora()->attach($corpus);
        }

        return redirect()->route('project.corpusProject.index');
    }


    /**
     * @param CorpusProject $corpusproject
     * @return $this
     */
    public function assignUsers(CorpusProject $corpusproject) {
        $isLoggedIn = \Auth::check();
        $user = \Auth::user();
        $users = User::latest()->get();
        $filteredList = array();

        $invited = $this->LaudatioUtilService->getInvitedByCorpusProject($corpusproject->id);
        $invitedUsers = array();
        foreach($invited as $invite) {
            $invitedUser = User::findOrFail($invite->user_id);
            $invitedRole = Role::findOrFail($invite->role_id);
            if(!array_key_exists($invite->user_id,$invitedUsers)){
                $invitedUsers[$invite->user_id] = array();
            }
            $invitedUsers[$invite->user_id] = array(
                "user_name" => $invitedUser->name,
                "user_id" => $invitedUser->id,
                "user_affiliation" => $invitedUser->affiliation,
                "user_role_name" => $invitedRole->name,
                "user_role_id" => $invitedRole->id,
            );
        }


        // get all roles
        $roleCollection = Role::all();
        $roles = array();
        foreach ($roleCollection as $role){
            if($role->id != 1) {
                $roles[$role->id] = $role->name;
            }

        }
        ksort($roles);


        $corpusProjectUsers = $corpusproject->users()->get();

        $admin_roles = array();
        $user_roles = array();

        foreach($corpusProjectUsers as $corpusProjectUser) {
            if(!isset($admin_roles[$corpusProjectUser->id])){
                $admin_roles[$corpusProjectUser->id] = array();
            }

            if(!isset($user_roles[$corpusProjectUser->id])){
                $user_roles[$corpusProjectUser->id] = array();
            }

            $corpusProjectUserRole = array();
            $corpusProjectRole = Role::find($corpusProjectUser->pivot->role_id);
            $corpusProjectUserRole['user_name'] = $corpusProjectUser->name;
            $corpusProjectUserRole['user_affiliation'] = $corpusProjectUser->affiliation;
            $corpusProjectUserRole['user_id'] = $corpusProjectUser->id;
            $corpusProjectUserRole['role_name'] = $corpusProjectRole->name;
            $corpusProjectUserRole['role_id'] = $corpusProjectRole->id;

            array_push($admin_roles[$corpusProjectUser->id],$corpusProjectUserRole);
            array_push($user_roles[$corpusProjectUser->id],$corpusProjectUserRole);
        }//end foreach


        $admin_roles = $this->LaudatioUtilService->determineAdminRole($admin_roles);
        $user_roles = $this->LaudatioUtilService->determineUserAdminRole($user_roles);
        $role_mappings = $this->LaudatioUtilService->getRoleMappingByUser($user);
        //dd($role_mappings);
        $corpusproject_data = array(
            'user_roles' => $user_roles,
            'roles' => $roles,
            'corpus_admin' => $admin_roles,
            'authuser_role_mappings' => $role_mappings,
        );
        return view('project.corpusproject.assign_projectroles_to_user')
            ->with('corpusproject', $corpusproject)
            ->with('users', $users)
            ->with('invitedusers', $invitedUsers)
            ->with('corpusproject_data',$corpusproject_data)
            ->with('isLoggedIn', $isLoggedIn)
            ->with('user',$user);


    }


    /**
     *
     * storeUserRelations
     *
     * @param $id
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function storeUserRelations($id)
    {

        $this->validate(request(), [
            'users' => 'required',
        ]);

        $corpusproject = CorpusProject::find($id);
        foreach (request('users') as $userId){
            $user = User::find($userId);
            $corpusproject->users()->attach($user);
        }

        return redirect()->route('project.corpusProject.index');
    }


    /**
     * @param Request $request
     * @return mixed
     */
    public function storeRelationsByProject(Request $request)
    {

        $input =$request ->all();
        $msg = "";
        if ($request->ajax()){
            $msg .= "<p>Assigned the following roles to user </p>";
            $role_users = $input['role_users'];
            $corpus_project = CorpusProject::find($input['project_id']);
            $msg .= "<ul>";
            foreach($role_users as $roleId => $user_data) {
                $role = Role::find($roleId);
                if($role){
                    $msg .= "<li>".$role->name."<ul>";
                    foreach($user_data as $userId) {
                        $user = User::find($userId);

                        if($user) {
                            $user->roles()->attach($role);
                            $msg .= "<li>".$user->name."</li>";
                            $corpus_project->users()->save($user,['role_id' => $roleId]);
                        }
                    }
                    $msg .= "</ul></li>";
                }
            }
            $msg .= "</ul>";
        }

        $response = array(
            'status' => 'success',
            'msg' => $msg,
        );

        return Response::json($response);
    }

    /**
     * @param $corpusProjectId
     * @param $userId
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function  destroyCorpusProjectUser($corpusProjectId, $userId){
        $isLoggedIn = \Auth::check();
        $user = \Auth::user();

        $corpusProject = CorpusProject::find($corpusProjectId);
        $corpusProject->users()->detach($userId);

        $roles = Role::latest()->get();
        return view('admin.useradmin.roles.index', compact('roles'))
            ->with('isLoggedIn', $isLoggedIn)
            ->with('user',$user);
    }

    /**
     * @param Request $request
     * @return mixed
     */
    public function deleteRelationsByProject(Request $request){
        $input =$request ->all();
        $msg = "";
        if ($request->ajax()){

            $msg .= "<p>Removed the following user from the project: </p>";
            $userId = $input['userId'];
            $corpusProject = CorpusProject::find($input['projectId']);
            $user = User::find($userId);
            $corpusProject->users()->detach($userId);
            $msg .= "<ul>";
            $msg .= "<li>".$user->name."</li>";
            $msg .= "</ul>";
        }

        $response = array(
            'status' => 'success',
            'msg' => $msg,
        );

        return Response::json($response);
    }
}
