<?php

namespace App\Http\Controllers;

use App\Custom\DOIInterface;
use App\Custom\LaudatioUtilsInterface;
use App\Doi;
use Illuminate\Http\Request;
use App\Laudatio\Utils\DOIService;
use Response;
use App\Corpus;
use DateTime;
use Log;

class DOIController extends Controller
{

    protected $DOIService;
    protected $LaudatioUtils;

    /**
     * DOIController constructor.
     * @param DOIInterface $DOIService
     * @param LaudatioUtilsInterface $laudatioUtils
     */
    public function __construct(DOIInterface $DOIService, LaudatioUtilsInterface $laudatioUtils)
    {
        $this->DOIService = $DOIService;
        $this->LaudatioUtils = $laudatioUtils;
    }


    public function TestRegisterDoi(){

        $response = $this->DOIService->mintDOIDataIdentifier("http://laudatio58-dev.local/api/elasticapi/v1/corpora/iWEbj24BnoDS0IgtYpcf/corpora");
        return Response::json($response);
    }

    public function mintADOI(){

        $response = $this->DOIService->mintDOIDataIdentifier("http://laudatio58-dev.local/api/elasticapi/v1/corpora/iWEbj24BnoDS0IgtYpcf/corpora");
        return Response::json($response);
    }

    public function deleteADOI(){

        $response = $this->DOIService->deleteDOIData("http://laudatio58-dev.local/api/elasticapi/v1/corpora/7bjqk24BDJzRMrR-rPaa/corpora");
        return Response::json($response);
    }

    public function TestUpdateDoi(){

        $myRequest = new Request();
        $myRequest->setMethod('POST');

        $myRequest->request->add(
            [
                'isActive' => "false",
                "state" => 'unavailable',
                'corpusid' => 3,
            ]
        );

        $this->updateDOIMetadata($myRequest);
    }

    public function TestDeleteDoi(){

        $myRequest = new Request();
        $myRequest->setMethod('POST');

        $myRequest->request->add(
            [
                "corpusid" => 3,
            ]
        );

        Log::info("TESTDELETEDOIR: ".print_r($myRequest->request,1));
        $this->deleteDOIdent($myRequest);
    }

    public function TestGetDoi(){
        $myRequest = new Request();
        $myRequest->setMethod('POST');

        $myRequest->request->add(
            [
                'doi_identifier' => 'doi:10.5072/laudatio-dev-5c98670bdf1b6',
            ]
        );

        $result = $this->getDOIMetadata($myRequest);
    }


    public function registerDOI(Request $request) {


        $result = array();
        $status = "success";

        try{
            $corpus = Corpus::findOrFail($request->input('corpusid'));

            $request->request->add(
                [
                    'resourceTitle' => $corpus->name,
                    'publicationYear' => $corpus->created_at->year,
                ]
            );

            $identifier = $this->DOIService->mintDOIDataIdentifier($request);

            $doiMetadata = $this->DOIService->getDOIData($identifier);
            $data_cite_xml = "";
            if($doiMetadata['success'] == $identifier){
                $data_cite_xml = $doiMetadata['datacite'];


                $doi = new Doi();
                $doi['doi_id'] = $identifier;
                $doi['corpus_id'] = $corpus->id;
                $doi['datacite_resource_xml'] = $data_cite_xml;
                $doi['resolve_uri'] = $request->input('path');
                $doi->save();
            }

            $result['mint_doi_response']  = "The Corpus DOI was successfully minted with the following DOI indentifier: ".$identifier;

        }
        catch (\Exception $e) {
            $status = "error";
            $result['mint_doi_response']  = "There was a problem Creating the DOI Identifier. The error was: ($e) A message has been sent to the site administrator. Please try again later";
        }


        $response = array(
            'status' => $status,
            'msg' => $result,
        );

        return Response::json($response);
    }


    public function getDOIMetadata(Request $request) {


        $result = array();
        $status = "success";

        try{

            $doiDataIdentifier = $request->input('doi_identifier');
            $doiMetadata = $this->DOIService->getDOIData($doiDataIdentifier);

            if($doiMetadata['success'] == $doiDataIdentifier){
                $result['id'] = $doiMetadata['success'];
                $result['_target'] = $doiMetadata['_target'];
                $result['datacite_resource_xml'] = $doiMetadata['datacite'];
                $result['_status'] = $doiMetadata['_status'];
                $result['_profile'] = $doiMetadata['_profile'];
                $result['_export'] = $doiMetadata['_export'];
            }


            $result['get_doi_response']  = "The Corpus DOI was successfully retrieved with the following DOI indentifier: ".$doiDataIdentifier;
        }
        catch (\Exception $e) {
            $status = "error";
            $result['get_doi_response']  = "There was a problem fetching the DOI meta data for ".$doiDataIdentifier." The error was: ($e) A message has been sent to the site administrator. Please try again later";
        }

        $response = array(
            'status' => $status,
            'msg' => $result,
        );

        return Response::json($response);
    }

    public function updateDOIMetadata(Request $request) {


        $result = array();
        $status = "success";


        try{
            $doidata = null;
            $corpus = Corpus::find($request->input('corpusid'));
            $doiIdentifier = "";

            if(!empty($corpus)) {
                $request->request->add(
                    [
                        'resourceTitle' => $corpus->name,

                    ]
                );

                $doidata = $corpus->doi()->get();

                if(!empty($doidata[0])){
                    $doiIdentifier = $doidata[0]->doi_id;
                }
            }
            Log::info("GOT DOIIDEN: ".$doiIdentifier);
            $updated = $this->DOIService->updateDOIData($doiIdentifier,$request);
            Log::info("UPDATED: ".print_r($updated,1));
            if($updated){

                $doiMetadata = $this->DOIService->getDOIData($doiIdentifier);
                $data_cite_xml = "";
                if($doiMetadata['success'] == $doiIdentifier){
                    $data_cite_xml = $doiMetadata['datacite'];
                }

                if(!empty($doidata[0])) {
                    $doi = Doi::findOrFail($doidata[0]->id);
                    $doi['datacite_resource_xml'] = $data_cite_xml;
                    $doi['resolve_uri'] = $request->input('path');
                    $doi->save();
                }

                $result['get_doi_response']  = "The Corpus DOI was successfully updated with the following DOI indentifier: ".$doiIdentifier;
            }
            else{
                $status = "error";
                $result['get_doi_response']  = "There was a problem updating the DOI meta data for ".$doiIdentifier."  Please try again later";
            }

        }
        catch (\Exception $e) {
            $status = "error";
            $result['get_doi_response']  = "There was a problem updating the DOI meta data for ".$doiIdentifier." The error was: ($e) A message has been sent to the site administrator. Please try again later";
        }

        $response = array(
            'status' => $status,
            'msg' => $result,
        );

        return Response::json($response);
    }

    public function deleteDOIdent(Request $request) {


        $result = array();
        $status = "success";

        $doiIdentifier = "";
        //try{
            $corpus = Corpus::find($request->input('corpusid'));
            Log::info("GOT CORPUS: ".$corpus->name);
            $doidata = null;

            if(!empty($corpus)) {

                $doidata = $corpus->doi()->get();

                if(!empty($doidata[0])){
                    $doiIdentifier = $doidata[0]->doi_id;
                }
            }
            Log::info("GOT DOIIDEN: ".$doiIdentifier);
            $deleted = $this->DOIService->deleteDOIData($doiIdentifier);
            if($deleted){

                if(!empty($doidata[0])) {
                    $doi = Doi::findOrFail($doidata[0]->id);
                    $doi->delete();
                }


                $result['delete_doi_response']  = "The Corpus DOI was successfully deleted with the following DOI indentifier: ".$doiIdentifier;
            }
            else{
                $status = "error";
                $result['delete_doi_response']  = "There was a problem deleting the DOI meta data for ".$doiIdentifier."  Please try again later";
            }

       //}
       //catch (\Exception $e) {
         //   $status = "error";
           // $result['delete_doi_response']  = "There was a problem deleting the DOI meta data for ".$doiIdentifier." The error was: ($e) A message has been sent to the site administrator. Please try again later";
        //}

        $response = array(
            'status' => $status,
            'msg' => $result,
        );

        return Response::json($response);
    }
}
