<?php

namespace App\Http\Controllers;

use App\Corpus;
use App\Laudatio\GitLab\GitRepoService;
use Illuminate\Http\Request;
use GrahamCampbell\Flysystem\FlysystemManager;
use App\Custom\LaudatioUtilsInterface;
use Log;
use Response;
use Chumper\Zipper\Zipper;

/**
 * Class DownloadsController
 * @package App\Http\Controllers
 */
class DownloadsController extends Controller
{
    protected $flysystem;
    protected $connection;
    protected $basePath;
    protected $laudatioUtils;
    protected $gitRepoService;

    /**
     * DownloadsController constructor.
     * @param FlysystemManager $flysystem
     * @param LaudatioUtilsInterface $laudatioUtils
     * @param GitRepoService $gitRepoService
     */
    public function __construct(FlysystemManager $flysystem, LaudatioUtilsInterface $laudatioUtils, GitRepoService $gitRepoService)
    {
        $this->flysystem = $flysystem;
        $this->connection = $this->flysystem->getDefaultConnection();
        $this->basePath = config('laudatio.basePath');
        $this->laudatioUtils = $laudatioUtils;
        $this->gitRepoService = $gitRepoService;
    }

    /**
     * @param $corpusid
     * @param $version
     * @param $path
     * @return mixed
     */
    public function teiDownload($corpusid, $version, $path) {
        $corpus = Corpus::find($corpusid);
        if($corpus->workflow_status == 1){
            $corpus = Corpus::find($corpusid);
            $storage_path = public_path();
            $archivedFileName = $this->gitRepoService->archiveFiles($path,$version);
            $response = \Response::download($storage_path."/".$archivedFileName.".zip")->deleteFileAfterSend();
            //ob_end_clean();
        }
        else{
            $response = $this->unpublishedTeiDownload($path);
        }
        
        return $response;
    }
    


    public function unpublishedTeiDownload($path) {
        $uniqueName = strtr(base64_encode(openssl_random_pseudo_bytes(16)), "+/=", "XXX");
        $fileName = str_replace("/","_",$path).'.zip';
        $filePath = $this->basePath.'/'.$path.'/TEI-HEADERS';
        $archivedFileName = $this->gitRepoService->fetchUnPublishedArchive($path);
        $response = \Response::download($storage_path."/".$archivedFileName.".zip")->deleteFileAfterSend();
        return $response;
        
    }
    

    public function formatDownload($formatid,$corpusid) {
        $corpusFiles = $this->laudatioUtils->getFormatFilesByCorpusId($corpusid);
        $zipper = new Zipper();
        $uniqueName = strtr(base64_encode(openssl_random_pseudo_bytes(16)), "+/=", "XXX");
        $zipper->make($uniqueName.'.zip');

        foreach ($corpusFiles as $corpusFileFormatid => $formatdata) {
            if($corpusFileFormatid == $formatid) {
                foreach ($formatdata['format_files'] as $formatPath) {
                    $zipper->folder($formatdata['format_name'])->add($this->basePath.'/'.$formatPath);
                }
            }
        }

        $zipper->close();
        $response = \Response::download($uniqueName.'.zip')->deleteFileAfterSend();
        ob_end_clean();
        return $response;
    }

    /**
     * @param Request $request
     * @return mixed
     */
    public function citeDownload(Request $request) {
        $citeFormat = null;
        $format = "";

        $result = null;


        try{
            $data = $request->input('data');
            $result = $this->laudatioUtils->buildCiteFormat($data);
            $status = "success";
        }
        catch (\Exception $e) {

            $status = "error";
            $result['citedata_error_response']  = "There was a problem fetching the citation. ".$e->getMessage()." A message has been sent to the site administrator. Please try again later";
        }


        $response = array(
            'status' => $status,
            'message' => $result
        );

        return Response::json($response);
    }



}
