<?php

namespace App\Http\Controllers;

use App\Custom\ElasticsearchInterface;
use App\Custom\LaudatioUtilsInterface;
use Illuminate\Pagination\LengthAwarePaginator;
use Response;
use Cache;
use Log;
use JavaScript;

/**
 * Class IndexController
 * @package App\Http\Controllers
 */
class IndexController extends Controller
{

    protected $ElasticService;
    protected $LaudatioUtilService;
    protected $ccBaseUri;

    /**
     * IndexController constructor.
     */
    public function __construct(ElasticsearchInterface $Elasticservice, LaudatioUtilsInterface $laudatioUtils)
    {
        $this->ElasticService = $Elasticservice;
        $this->LaudatioUtilService = $laudatioUtils;
        $this->ccBaseUri = config('laudatio.ccBaseuri');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        try{
            $corpusresponses = $this->ElasticService->pingServer();
        }
        catch (LaudatioElasticsearchException $laudatioElasticsearchException) {
            $message = $laudatioElasticsearchException->createUserFacingMessage($laudatioElasticsearchException->getMessage());
            //Log the error to file
            $laudatioElasticsearchException->logErrorMessage($laudatioElasticsearchException->getMessage(),['file' => __FILE__, 'line' => __LINE__]);
            return back()->withError($message);
        }


        if (Cache::store('redis')->has('latest_published_corpora_front')) {
            $responseArray = Cache::store('redis')->get('latest_published_corpora_front');

        }
        else {
            $corpusresponses = $this->ElasticService->getPublishedCorpora();

            $corpusdata = array();
            $documentcount = 0;
            $annotationcount = 0;

            $entries = null;
            $perPageArray = array();
            $sortedCollection = array();
            $perPage = null;
            $sortKriterium = null;

            $currentPage = LengthAwarePaginator::resolveCurrentPage();
            $responseArray = $this->LaudatioUtilService->getPublishedCorpusData($corpusresponses,$this->ElasticService, $perPage ,$sortKriterium, $currentPage);
            Cache::store('redis')->forever('latest_published_corpora_front', $responseArray);
        }



        JavaScript::put([
            "baseDomain" => config('laudatio.base_domain'),
            "elasticApiVersion" => config('laudatio.elasticApiVersion'),
            "elasticsearchLimit" => config('laudatio.elasticsearch_limit'),
            "corpusdata" => $responseArray['entries'],
            "totalCount" => $responseArray['totalcount'],
            "perPageArray" => $responseArray['perPageArray'],
            "perPage" => $responseArray['perPage'],
            "ccBaseUri" => $this->ccBaseUri
        ]);
        return view('index');
    }

    public function fapp() {
        //$res = $this->LaudatioUtilService->getCurrentPublishedCorpusFromDatabase('3587');
            $corpusId = "2";
            $filePath = "oppdal-project-tirsdag/muspilli_1-0/CORPUS-DATA";
            //Cache::forget("getCorpusFileInfo_".$corpusId.$path);

            $headerType = "corpus";
            //$filePath = $filePath.$headerType;

            $this->LaudatioUtilService->emptyCorpusDataFileCache($filePath,$corpusId);

            //Cache::forget("laudatio_repository_cache:getCorpusFileInfo_".$corpusId.$path);

        return Response::json($filePath);
    }

    public function contact(){
        return view('contact');
    }

    public function projecthome(){
        return view('projecthome');
    }

    public function projectgoals(){
        return view('projectgoals');
    }
    public function projectcollaborators(){
        return view('projectcollaborators');
    }
    public function projectpublications(){
        return view('projectpublications');
    }

    public function projectcontent(){
        return view('projectcontent');
    }
}