<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Adldap\Laravel\Facades\Adldap;
use Adldap\Models\Attributes\DistinguishedName;
use App\Custom\LDAPInterface;
use App\User;
use App\Role;
use Log;


class LDAPController extends Controller
{
    protected $LDAPService;

    /**
     * LDAPController constructor.
     * @param LDAPInterface $ldapService
     */
    public function __construct(LDAPInterface $ldapService)
    {
        $this->LDAPService = $ldapService;
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $isLoggedIn = \Auth::check();
        $loggedInUser = \Auth::user();

        return view('admin.adminIndex')
            ->with('isLoggedIn', $isLoggedIn)
            ->with('user',$loggedInUser);
    }

    /**
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function create(Request $request)
    {

    }

    /**
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function store(Request $request)
    {

        $validated = $this->validate($request,[
            'username' => 'required|string|unique:users|max:255',
            'givenName' => 'required|string|max:255',
            'sn' => 'required|string|max:255',
            'email' => 'required|email|unique:users|max:255',
            //'affiliation' => 'required|string|max:255',
            'desiredPassword' => 'required|between:8,255|confirmed'
        ]);

        $desiredUserName =  $request->input('username');
        $desiredPassword =  $request->input('desiredPassword');
        $hashedDesiredPassword = $this->generateHashedPassword($desiredPassword);
        $email =  $request->input('email');
        $givenName =  $request->input('givenName');
        $sn =  $request->input('sn');

        //connect to the tree
        $this->LDAPService->connectADLDAP();

        //search if the user already exists

        $searchResult = $this->LDAPService->searchTree(array(
            'uid' => $desiredUserName,
            'mail' => $email

        ),array('uid','mail'));

        $createResult = "";

        $message = "The user ".$desiredUserName;
        if(count($searchResult) == 0) {
            $createResult = $this->LDAPService->createUser(
                array(
                    'uid' => $desiredUserName,
                    'userPassword' => $this->generateSaltedSha512($desiredPassword,$desiredUserName),
                    'mail' => $email,
                    'givenName' => $givenName,
                    'sn' => $sn,
                    'cn' => $givenName.' '.$sn
                )
            );

            if($createResult['success']){
                $user = new User();
                $user->username = $desiredUserName;
                $user->name = $givenName.' '.$sn;
                $user->email = $email;

                $user->save();

                $message .= " was successfully created";
                $user->roles()->attach(2);

                return redirect('admin')->with('success',$message);
            }
            else {
                $message .= " could not be created";
                return redirect('admin')->with('error',$message);
            }
        }

        //$response = array($createResult);
        //return response()->json($response);
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }


    private function generateHashedPassword($password) {
        return password_hash($password, PASSWORD_DEFAULT);
    }

    private function verifyHashedPassword($password,$hashedPassword) {
        return password_verify($password, $hashedPassword);
    }

    private function generateSaltedSha512($password,$salt) {
        //$hash = hash('sha512', $salt.$password);
        $hash = base64_encode(hash('sha512', $salt.$password, true));
        return $hash;
    }

    private function encodePassword($password)
    {
        return iconv('UTF-8', 'UTF-16LE', '"'.$password.'"');
    }


}
