<?php

namespace App\Http\Controllers;

use App\Publication;
use Illuminate\Http\Request;
use App\Custom\GitRepoInterface;
use App\Custom\GitLabInterface;
use App\Custom\LaudatioUtilsInterface;
use App\Custom\ElasticsearchInterface;
use App\Custom\DOIInterface;
use App\Corpus;
use App\Doi;
use App\Document;
use App\Annotation;
use App\User;
use App\Role;
use Response;
use JavaScript;
use Log;


/**
 * Class PublicationController
 * @package App\Http\Controllers
 */
class PublicationController extends Controller
{


    protected $GitRepoService;
    protected $LaudatioUtilService;
    protected $GitLabService;
    protected $elasticService;
    protected $DOIService;
    protected $base_domain;

    /**
     * PublicationController constructor.
     * @param GitRepoInterface $Gitservice
     * @param LaudatioUtilsInterface $laudatioUtilService
     * @param GitLabInterface $GitLabService
     * @param ElasticsearchInterface $elasticService
     */
    public function __construct(GitRepoInterface $Gitservice, LaudatioUtilsInterface $laudatioUtilService, GitLabInterface $GitLabService, ElasticsearchInterface $elasticService, DOIInterface $DOIService)
    {
        $this->GitRepoService = $Gitservice;
        $this->LaudatioUtilService = $laudatioUtilService;
        $this->GitLabService = $GitLabService;
        $this->elasticService = $elasticService;
        $this->DOIService = $DOIService;
        $this->base_domain = config('laudatio.base_domain');

    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function publishCorpus(Request $request)
    {
        $status = "";
        $result = array();
        $update_response = null;
        $autenticated_user = \Auth::user();

        $corpusid = $request->input('corpusid');
        $corpus = Corpus::findOrFail($corpusid);

        $corpuspath = $request->input('corpus_path');
        $auth_user_name = $request->input('auth_user_name');
        $auth_user_email = $request->input('auth_user_email');

        $corpus_index =  config("laudatio.elasticsearch_corpus_index","corpora");
        $document_index = config("laudatio.elasticsearch_document_index","documents");
        $annotation_index =  config("laudatio.elasticsearch_annotation_index","annotations");
        $guideline_index = config("laudatio.elasticsearch_guideline_index","guidelines");

        $publicationArray = array();
        $documentArray = array();
        $annotationArray = array();
        $guidelineArray = array();

        $corpusIsPublished = false;
        $documentIsPublished = false;
        $annotationIsPublished = false;
        $guidelineIsPublished = false;

        $publish_message = "<ul>";

        //check for missing elasticsearch ids
        $checkResults = $this->LaudatioUtilService->checkForElasticsearchIds($corpusid);
        $missing = false;
        foreach ($checkResults as $type => $data){
            if($type == "corpus"){
                if(null == $data || empty($data)){
                    $missing = true;
                    break;
                }
            }
            else{
                foreach ($data as $elementId => $elasticSearchId) {
                    if(null == $elasticSearchId || empty($elasticSearchId)){
                        $missing = true;
                        break;
                    }
                }
            }
        }


        if($missing){
            Log::info("CORPUS is missing elasticids: ".print_r($checkResults,1));
            Log::info("Fixing...: ");
            $fixResult = $this->elasticService->fixElasticsearchIds($corpusid,$corpus_index, $document_index,$annotation_index);
            Log::info("Got fixresult: ".print_r($fixResult,1));
        }

        //get documents
        $documentstart = microtime(true);
        Log::info("STARTED FETCHING OF DOCuMENTS");
        foreach ($corpus->documents()->get() as $document) {
            $elasticDocument = $this->elasticService->getDocument($document->elasticsearch_id,true,$document_index);
            $elasticDocumentData = json_decode($elasticDocument->getContent(), true);
            $elasticDocumentData['result']['is_latest_publication'] = "false";
            $elasticDocumentData['result']['publication_status'] = 0;

            array_push($documentArray,array(
                "_id" => $document->elasticsearch_id,
                "db_id" => $document->id,
                "annotations" => $elasticDocumentData['result']['has_annotations'],
                "laudatio_corpus_gitlab_id" => $corpus->gitlab_id,
                "laudatio_corpus_id" => $corpus->id,
                "elasticdata" => $elasticDocumentData['result']
            ));
        }

        $documenttime_elapsed_secs = microtime(true) - $documentstart;
        Log::info("GOT DOCUMENTS: took: ".$documenttime_elapsed_secs);

        //get annotations
        Log::info("STARTED FETCHING OF ANNOTATIONS");
        $annotationstart = microtime(true);
        foreach ($corpus->annotations()->get() as $annotation) {
            if(!empty($annotation->elasticsearch_id)) {
                $elasticAnnotation = $this->elasticService->getAnnotation($annotation->elasticsearch_id,true,$annotation_index);
                $elasticAnnotationData = json_decode($elasticAnnotation->getContent(), true);
                $elasticAnnotationData['result']['is_latest_publication'] = "false";
                $elasticAnnotationData['result']['publication_status'] = 0;
            }
            else{
                $getParams = array();
                $idParams = array();
                array_push($idParams, array(
                    "laudatio_corpus_id" => $annotation->corpus_id,
                ));

                array_push($idParams, array(
                    "laudatio_corpus_gitlab_id" => $annotation->corpus_id
                ));

                $documentParams[$document->id] = $idParams;

                $documentElasticIds = $this->elasticService->getElasticIdByObjectId($this->documentIndexName,$documentParams);
            }


            array_push($annotationArray,array(
                "_id" => $annotation->elasticsearch_id,
                "db_id" => $annotation->id,
                "documents" => $elasticAnnotationData['result']['in_documents'],
                "laudatio_corpus_gitlab_id" => $corpus->gitlab_id,
                "laudatio_corpus_id" => $corpus->id,
                "elasticdata" => $elasticAnnotationData['result']
            ));
        }
        $annotationtime_elapsed_secs = microtime(true) - $annotationstart;
       //Log::info("GOT ANNOTATIONS: ".print_r($annotationArray,1));
        Log::info("GOT ANNOTATIONS: took: ".$annotationtime_elapsed_secs);

        //get guidelines
        $guidelinestart = microtime(true);
        $guidelines = $this->elasticService->getGuidelinesByCorpus($corpus->id,$guideline_index);
        foreach ($guidelines['result']['hits']['hits'] as $guideline) {
            array_push($guidelineArray,array(
                "_id" => $guideline['_id'],
                "db_id" => $document->id,
                "laudatio_corpus_gitlab_id" => $corpus->gitlab_id,
                "laudatio_corpus_id" => $corpus->id,
                "elasticdata" => $guideline
            ));
        }
        $guidelinetime_elapsed_secs = microtime(true) - $annotationstart;
        Log::info("GOT GUIDELINES: took: ".$guidelinetime_elapsed_secs);
        //Log::info("GOT GUIDELINES: ".print_r($guidelineArray,1));


        //unset is_latest_publication for former published corpor   a, documents and annotations
        $currentPublishedData = $this->LaudatioUtilService->getCurrentPublishedCorpusFromDatabase($corpus->gitlab_id);
        Log::info("GOT CURRENTPUBLISHDATA: ".print_r($currentPublishedData,1));
        if(!empty($currentPublishedData['corpus_elasticsearch_id'])){
            Log::info("CORPUS ALREADY PUBLISHED: STARTING UNPUBLISH: ".print_r($currentPublishedData,1));
            $alreadystart = microtime(true);

            $corpus_unpublish_params = [
                'index' => $corpus_index,
                'id' => $currentPublishedData['corpus_elasticsearch_id'],
                'body' => [
                    'doc' => [
                        'is_latest_publication' => "false"
                    ]
                ]
            ];
            $corpus_unpublish_response = $this->elasticService->unsetCorpusToPublished($corpus_unpublish_params);

            foreach($currentPublishedData["document_elasticsearch_ids"] as $document_elasticsearch_id){
                $document_unpublish_params = [
                    'index' => $document_index,
                    'id' => $document_elasticsearch_id,
                    'body' => [
                        'doc' => [
                            'is_latest_publication' => "false"
                        ]
                    ]
                ];
                $document_unpublish_response = $this->elasticService->unsetCorpusToPublished($document_unpublish_params);
            }

            foreach($currentPublishedData["annotation_elasticsearch_ids"] as $annotation_elasticsearch_id){
                $annotation_unpublish_params = [
                    'index' => $annotation_index,
                    'id' => $annotation_elasticsearch_id,
                    'body' => [
                        'doc' => [
                            'is_latest_publication' => "false"
                        ]
                    ]
                ];

                $annotation_unpublish_response = $this->elasticService->unsetCorpusToPublished($annotation_unpublish_params);
            }
            $alreadytime_elapsed_secs = microtime(true) - $alreadystart;
            Log::info("UNPUBLISHING DONE took: ".print_r($alreadytime_elapsed_secs,1));
        }//end if already published


        Log::info("STARTING  corPuS PUBLISH: ");
        $corpusstart = microtime(true);
        $corpus_publish_params = [
            'index' => $corpus_index,
            'id' => $corpus->elasticsearch_id,
            'body' => [
                'doc' => [
                    'publication_status' => '1',
                    'is_latest_publication' => "true"
                ]
            ]
        ];
        $corpus_publish_response = $this->elasticService->setCorpusToPublished($corpus_publish_params);
        $corpustime_elapsed_secs = microtime(true) - $corpusstart;
        Log::info("corPuS PUBLISHED: ".print_r($corpustime_elapsed_secs,1));

        if(
            isset($corpus_publish_response) &&
            isset($corpus_publish_response['result']) &&
            $corpus_publish_response['result'] == "updated" &&
            $corpus_publish_response['_shards']['successful'] > 0 &&
            $corpus_publish_response['_shards']['failed'] == 0) {

            $corpusIsPublished = true;


            Log::info("STARTING  DOCUMENT PUBLISH: ");
            $documentstart = microtime(true);
            $document_publish_params = array();
            foreach($documentArray as $documentdata) {
                array_push($document_publish_params,
                    [
                        'index' => $document_index,
                        'body' => [
                            'doc' => [
                                'publication_status' => '1',
                                'is_latest_publication' => "true"
                            ]
                        ],
                        'id' => $documentdata['_id']
                    ]);
            }//end foreach

            $document_publish_response = $this->elasticService->setDocumentAndAnnotationsToPublished($document_publish_params);


            if(
                isset($document_publish_response) &&
                isset($document_publish_response['result']) &&
                $document_publish_response['result'] == "updated" &&
                $document_publish_response['_shards']['successful'] > 0 &&
                $document_publish_response['_shards']['failed'] == 0) {

                $documentIsPublished = true;
                $documenttime_elapsed_secs = microtime(true) - $documentstart;
                Log::info("DOCUMENTS PUBLISHED: ".print_r($documenttime_elapsed_secs,1));

                Log::info("STARTING  ANNOTATION PUBLISH: ");
                $annotationstart = microtime(true);
                $annotation_publish_params = array();
                foreach($annotationArray as $annotationdata) {
                    array_push($annotation_publish_params,
                        [
                            'index' => $annotation_index,
                            'body' => [
                                'doc' => [
                                    'publication_status' => '1',
                                    'is_latest_publication' => "true"
                                ]
                            ],
                            'id' => $annotationdata['_id']
                        ]);
                }//end foreach

                $annotation_publish_response = $this->elasticService->setDocumentAndAnnotationsToPublished($annotation_publish_params);


                if(
                    isset($annotation_publish_response) &&
                    isset($annotation_publish_response['result']) &&
                    $annotation_publish_response['result'] == "updated" &&
                    $annotation_publish_response['_shards']['successful'] > 0 &&
                    $annotation_publish_response['_shards']['failed'] == 0) {

                    $annotationIsPublished = true;
                    $annotationtime_elapsed_secs = microtime(true) - $annotationstart;
                    Log::info("ANNOTATIONS PUBLISHED: ".print_r($annotationtime_elapsed_secs,1));

                    Log::info("STARTING  GUIDELINE PUBLISH: ");
                    $guidelinestart = microtime(true);
                    $guideline_publish_params = array();
                    foreach($guidelineArray as $guidelinedata) {
                        array_push($guideline_publish_params,
                            [
                                'index' => $guideline_index,
                                'body' => [
                                    'doc' => [
                                        'publication_status' => '1',
                                        'is_latest_publication' => "true"
                                    ]
                                ],
                                'id' => $guidelinedata['_id']
                            ]);
                    }//end foreach


                    $guideline_publish_response = $this->elasticService->setDocumentAndAnnotationsToPublished($guideline_publish_params);


                    if(
                        isset($guideline_publish_response) &&
                        isset($guideline_publish_response['result']) &&
                        $guideline_publish_response['result'] == "updated" &&
                        $guideline_publish_response['_shards']['successful'] > 0 &&
                        $guideline_publish_response['_shards']['failed'] == 0) {
                        $guidelineIsPublished = true;
                        $guidelinetime_elapsed_secs = microtime(true) - $guidelinestart;
                        Log::info("GUIDELINES PUBLISHED: ".print_r($guidelinetime_elapsed_secs,1));
                    }


                }//end if annotation is published
            }//end if document is published



            if($corpusIsPublished && $documentIsPublished && $annotationIsPublished) {


                $duplicationData = array();
                $duplicationData['corpus'] = array(
                    "index" => $corpus_index
                );

                $duplicationData['document'] = array(
                    "index" => $document_index
                );

                $duplicationData['annotation'] = array(
                    "index" => $annotation_index
                );
                $duplicationData['guideline'] = array(
                    "index" => $guideline_index
                );

                Log::info("DUPLICATING CORPUS");
                $duplicatestart = microtime(true);
                $duplicateIds = $this->LaudatioUtilService->duplicateCorpus($corpus, $duplicationData);
                //Log::info("duplicate:duplicateIds: ".print_r($duplicateIds,1));
                $duplicatetime_elapsed_secs = microtime(true) - $guidelinestart;
                Log::info("CORPUS DUPLICATED: ".print_r($duplicatetime_elapsed_secs,1));

                $new_annotations = array();
                $new_documents = array();
                if(isset($duplicateIds["corpus"]['new_corpus_id'])) {
                    Log::info("DUPLICATING RELATIoNS");
                    $relatestart = microtime(true);


                    $old_annotations = array();

                    $old_documents = array();
                    $corpusannotations = array();
                    $corpusdocuments = array();
                    $old_has_documents = array();
                    $new_has_documents = array();

                    if(isset($duplicateIds["annotation"]["new_annotation_ids"])) {
                        foreach ($annotationArray as $annotationArrayData) {
                            $old_annotations[$annotationArrayData['db_id']] = array(
                                "_id" => $annotationArrayData['_id']
                            );

                            $annotationArrayData["elasticdata"]['laudatio_corpus_id'] = $duplicateIds["corpus"]['new_corpus_id'][$corpus->id];
                            $old_annotations[$annotationArrayData['db_id']]['elasticdata'] = $annotationArrayData['elasticdata'];
                        }
                        //Log::info("OLD-ANNOTATIONS: ".print_r($old_annotations,1));

                        foreach($duplicateIds["annotation"]["new_annotation_ids"] as $oldAnnotationId => $annotationId) {
                            $annotationPostData = array();
                            $annotationPostData[$annotationId] = $old_annotations[$oldAnnotationId]['elasticdata'];
                            $annotationReindexResponse = $this->elasticService->postReindex($annotation_index, $annotationPostData);

                            $old_has_documents = $old_annotations[$oldAnnotationId]['elasticdata']['in_documents'];

                            if($annotationReindexResponse[$annotationId]['result'] == "created" &&
                                $annotationReindexResponse[$annotationId]['_shards']['successful'] > 0) {
                                $annotationReindexId = $annotationReindexResponse[$annotationId]['_id'];

                                $newAnnotation = Annotation::find($annotationId);
                                $newAnnotation->elasticsearch_id = $annotationReindexId;
                                $newAnnotation->save();

                                if(! in_array($annotationReindexId,$corpusannotations)) {
                                    array_push($corpusannotations, $annotationReindexId);
                                }

                                $new_annotations[$oldAnnotationId] = array("oldannotationid" => $oldAnnotationId, "oldelasticid" => $old_annotations[$oldAnnotationId]['_id'], "newannotationid" => $annotationId, "newelasticid" => $annotationReindexId);
                            }//end if indexed
                        }//end foreach

                        $publish_message .= "<li>".count($new_annotations)." annotations</li>";

                    }//end if isset annotationids


                    if(isset($duplicateIds["document"])) {
                        foreach ($documentArray as $documentArrayData) {
                            $old_documents[$documentArrayData['db_id']] = array(
                                "_id" => $documentArrayData['_id']
                            );
                            $documentArrayData["elasticdata"]['laudatio_corpus_id'] = $duplicateIds["corpus"]['new_corpus_id'][$corpus->id];
                            $old_documents[$documentArrayData['db_id']]['elasticdata'] = $documentArrayData['elasticdata'];
                        }
                        $new_documents = array();
                        foreach($duplicateIds["document"]["new_document_ids"] as $oldDocumentId => $documentId) {
                            $documentPostData = array();
                            $old_has_annotations = array();
                            $old_has_annotations[$oldDocumentId] = $old_documents[$oldDocumentId]['elasticdata']['has_annotations'];
                            $new_has_annotations = $corpusannotations;//$this->resolveHasAnnotations($new_annotations, $old_has_annotations );
                            $old_documents['elasticdata']['has_annotations'] = $new_has_annotations;

                            $old_documents[$oldDocumentId]['elasticdata']['has_annotations'] = $new_has_annotations;
                            $documentPostData[$documentId] = $old_documents[$oldDocumentId]['elasticdata'];

                            $documentReindexResponse = $this->elasticService->postReindex($document_index, $documentPostData);
                            if( $documentReindexResponse[$documentId]['result'] == "created" &&
                                $documentReindexResponse[$documentId]['_shards']['successful'] > 0) {
                                $documentReindexId = $documentReindexResponse[$documentId]['_id'];
                                //Log::info("saving documentReindexedId: ".print_r($documentReindexId,1));

                                if(! in_array($documentReindexId,$corpusdocuments)) {
                                    array_push($corpusdocuments, $documentReindexId);
                                }


                                $newDocument = Document::find($documentId);
                                $newDocument->elasticsearch_id = $documentReindexId;
                                $newDocument->save();

                                $new_documents[$oldDocumentId] = array("olddocumentid" => $oldDocumentId, "oldelasticid" => $old_documents[$oldDocumentId]['_id'], "newdocumentid" => $documentId, "newelasticid" => $documentReindexId);
                            }//end if indexed
                        }//end foreach
                        $publish_message .= "<li>".count($new_documents)." documents</li>";


                        //update annotation in_documents
                        $old_in_documents  = array();

                        foreach ($new_annotations as $oldAnnotationId => $annotationdata) {
                           $old_in_documents[$annotationdata['newelasticid']] = $old_annotations[$oldAnnotationId]['elasticdata']['in_documents'];
                        }


                        $new_in_documents = $this->resolveHasDocuments($new_documents, $old_in_documents);

                        $in_document_update_response = $this->elasticService->updateDocumentFieldsInAnnotation("annotations",$new_in_documents);
                        if($in_document_update_response) {
                            Log::info(" trying to update in_documents for annos: successfully updated: ");
                        }
                    }//end if isset documentids


                    }//end if we have new corpusi



                   if(count($corpusannotations) > 0 && count($corpusdocuments) > 0) {
                       $postData = array();

                       $new_corpus_id = array_values($duplicateIds["corpus"]['new_corpus_id'])[0];

                       $elasticCorpus = $this->elasticService->getCorpus($corpus->elasticsearch_id,true,$corpus_index);
                       $elasticCorpusData = json_decode($elasticCorpus->getContent(), true);
                       $elasticCorpusData['result']['is_latest_publication'] = "false";
                       $elasticCorpusData['result']['laudatio_corpus_id'] = $new_corpus_id;
                       $elasticCorpusData['result']['publication_status'] = 0;
                       $elasticCorpusData['result']['has_annotations'] = $corpusannotations;
                       $elasticCorpusData['result']['has_documents'] = $corpusdocuments;

                       //@todo: do docs and annos first, in order to add the correct has_documents/has_annotations, and annotations before documents

                       $postData[$new_corpus_id] = $elasticCorpusData['result'];
                       $corpusReindexResponse = $this->elasticService->postReindex($corpus_index, $postData);

                       if( $corpusReindexResponse[$new_corpus_id]['result'] == "created" &&
                           $corpusReindexResponse[$new_corpus_id]['_shards']['successful'] > 0) {
                           $corpusReindexId = $corpusReindexResponse[$new_corpus_id]['_id'];

                           $newCorpus = Corpus::find($new_corpus_id);
                           $newCorpus->elasticsearch_id = $corpusReindexId;
                           $newCorpus->save();


                           //update in_corpora for annotations
                           $annotationUpdateResponse  = $this->elasticService->updateInCorpora($annotation_index,$corpusannotations,$corpusReindexId);
                           //update in_corpora for documents
                           $documentUpdateResponse  = $this->elasticService->updateInCorpora($document_index,$corpusdocuments,$corpusReindexId);


                           //copy guidelines
                           $guidelinepostdata = array();
                           foreach($guidelineArray as $guideline) {
                               $guideline['elasticdata']['_source']['laudatio_corpus_id'] = $new_corpus_id;
                               $guideline['elasticdata']['_source']['in_corpora'] = $corpusReindexId;
                               $guidelinepostdata[$guideline['_id']] = $guideline['elasticdata']['_source'];
                           }

                           $guidelineReindexResponse = $this->elasticService->postReindex($guideline_index, $guidelinepostdata);
                           //Log::info("GUIDELINE resindfexresponse: ".print_r($guidelineReindexResponse,1));
                           $publish_message .= "<li>".count($guidelineArray)." guidelines</li>";
                           $relatetime_elapsed_secs = microtime(true) - $relatestart;
                           Log::info("RELATIONS DUPLICATED: ".print_r($relatetime_elapsed_secs,1));

                           Log::info("SETTING VERSION TAG");
                           $tagstart = microtime(true);
                           $tagMessage = array(
                               "laudatioCorpusid" => $new_corpus_id,
                               "gitlabId" => $corpus->gitlab_id,
                               "username" => $auth_user_name,
                               "useremail" =>  $auth_user_email
                           );
                           //Log::info("PUBLICATION tagMessage: ".print_r($tagMessage,1));
                           $hexedJson = str_replace('"','\"',json_encode( $tagMessage));
                           //Log::info("HEXCED JSON: ".print_r($hexedJson,1));
                           $tag = $this->GitRepoService->setCorpusVersionTag($corpuspath,$hexedJson,$corpus->publication_version,$auth_user_name,$auth_user_email, false);
                           if($tag) {
                               $tagparams = array(
                                   'gitlab_version_tag' => "v".$corpus->publication_version
                               );

                               $updatedcorpus = $this->LaudatioUtilService->updateCorpusAttributes($tagparams,$new_corpus_id);

                               if($updatedcorpus) {
                                   foreach ($updatedcorpus->documents()->get() as $documentToUpdate) {
                                       $updatedDocument = $this->LaudatioUtilService->updateDocumentAttributes($tagparams,$documentToUpdate->id);
                                   }

                                   foreach ($updatedcorpus->annotations()->get() as $annotationToUpdate) {
                                       $updatedAnnotation = $this->LaudatioUtilService->updateAnnotationAttributes($tagparams,$annotationToUpdate->id);
                                   }

                               }
                               $tagtime_elapsed_secs = microtime(true) - $tagstart;
                               Log::info("TAGS UPDATED: ".print_r($tagtime_elapsed_secs,1));
                               Log::info("PUBLICATION tag: ".print_r($tag,1));
                           }



                           $publish_message .= "<li>Versioned at version ".$corpus->publication_version."</li>";

                           Log::info("UPDATING GITLAB PROJECT VISIBILITY");
                           $visibilitystart = microtime(true);
                           //set gitlab repo visibility to public
                           $corpusProject = $corpus->corpusprojects()->get();

                           $gitlabUpdateResponse = $this->GitRepoService->updateCorpusGitLabVisibility($corpusProject[0]->gitlab_id,$corpus->gitlab_id);
                           $visibilitytime_elapsed_secs = microtime(true) - $visibilitystart;
                           Log::info("VISIBILITY UPDATED: ".print_r($visibilitytime_elapsed_secs,1));

                           $publish_message .= "<li>Published in GitLab</li>";

                           // Create DOI
                           Log::info("TRYING TO MINT DOI WITH: ".print_r($corpus->id,1));
                           $doistart = microtime(true);

                           $doi_identifier = $this->mintDOIIdentifier($corpus->id);
                           Log::info("PUBLicATION CONTROLLER CREATED DOI: ".print_r($doi_identifier,1));
                           $doitime_elapsed_secs = microtime(true) - $doistart;
                           Log::info("DOI CREATED: ".print_r($doitime_elapsed_secs,1));
                           $publish_message .= "<li>Created DOI with identifier ".$doi_identifier."</li>";

                           //copy corpusFIles
                           Log::info("TRYING TO MINT DOI WITH: ".print_r($corpus->id,1));
                           $doistart = microtime(true);
                           $oldFileIds = $this->LaudatioUtilService->updateCorpusFiles($corpus,$corpus_index);
                           //$newFileIds = $this->LaudatioUtilService->copyCorpusFiles($corpus,$new_corpus_id,$corpus_index);
                           $doitime_elapsed_secs = microtime(true) - $doistart;
                           Log::info("DOI CREATED: ".print_r($doitime_elapsed_secs,1));


                           $this->LaudatioUtilService->emptyFileCaches($corpusid);
                           Log::info("EMPTIED FILE CACHES  ".print_r($oldFileIds,1));

                           $this->LaudatioUtilService->emptySearchCache();
                           Log::info("EMPTIED SEARCH CACHES  ".print_r($oldFileIds,1));
                       }

                       $publish_message .= "</ul>";


                       if($tag) {
                           $publishMessage = "<p>You published the '".$corpus->name."' Version ".$corpus->publication_version." sucessfully! This version will now be available within the laudatio repository for download and search. <br /> The following operations where performed: </p>".$publish_message;
                           $result['publish_corpus_response']  = $publishMessage;
                           $result['new_corpus_id'] = $new_corpus_id;
                           $status = "success";
                       }
                       else{
                           $status = "error";
                           $result['publish_corpus_response']  = "There was a problem publishing the Corpus. The error was: The corpus could not be published to git due to failed GIT tagging. A message has been sent to the site administrator. Please try again later";
                       }
                       Log::info("PUBLISHresponse: ".print_r($result,1));
                   }//end if corpusdocs and annotations
                }
                else{
                    $status = "error";
                    $result['publish_corpus_response'] = "There was a problem publishing the Corpus. The error was: The corpus could not be published to git due to failed updating of the database. A message has been sent to the site administrator. Please try again";
                }

            }   //end if allArePublished

        $response = array(
            'status' => $status,
            'message' => $result,
        );


        return Response::json($response);

    }


    /**
     * @param Request $request
     * @return mixed
     */
    public function preparePublication(Request $request) {

        $result = array();
        $corpusid = $request->input('corpusid');
        $corpus = Corpus::findOrFail($corpusid);

        $corpuspath = $request->input('corpuspath');

        $result['corpus_header'] = array(
            "title" => "1 Corpus header uploaded"
        );

        $result['document_headers'] = array(
            "title" => "According number of Document headers"
        );

        $result['annotation_headers'] = array(
            "title" => "According number of Annotation headers"
        );

        $workFlowStatus = null;
        $corpusVersion = null;
        $canPublish = true;


        if(!isset($corpus)){
            $result['corpus_header']['corpusHeaderText'] = "Missing corpusheader";
            $result['corpus_header']['corpusIcon'] = '<i class="fas fa-exclamation-triangle align-baseline fa-lg text-wine pull-right"></i>';
            $canPublish = false;
        }
        else{
            $result['corpus_header']['corpusHeaderText'] = "";
            $result['corpus_header']['corpusIcon'] = '<i class="fas fa-check-circle align-baseline fa-lg text-wine pull-right"></i>';
        }



        $result['title'] = "Publish ".$corpus->name.", Version ".$corpus->publication_version;
        $result['subtitle'] = "The following criteria needs to be fulfilled before you can publish a Corpus";
        $result['waiting'] = "Verification is ongoing...";


        $checkResult = json_decode($this->GitRepoService->checkForCorpusFiles($corpuspath."/TEI-HEADERS"), true);

        $missing_document_count = count($checkResult['not_found_documents_in_corpus']);
        $document_plural = "";
        if($missing_document_count > 1){
            $document_plural = "s";
        }

        if($missing_document_count >= 1) {
            $result['document_headers']['documentHeaderText'] = $missing_document_count." missing document".$document_plural;
            $result['document_headers']['documentIcon'] = '<i class="fas fa-exclamation-triangle align-baseline fa-lg text-wine pull-right"></i>';
            $canPublish = false;
        }
        else{
            $result['document_headers']['documentHeaderText'] = "";
            $result['document_headers']['documentIcon'] = '<i class="fas fa-check-circle align-baseline fa-lg text-wine pull-right"></i>';
        }

        $missing_annotation_count = count($checkResult['not_found_annotations_in_corpus']);
        $annotation_plural = "";
        if($missing_annotation_count > 1){
            $annotation_plural = "s";
        }

        if($missing_annotation_count > 0) {
            $result['annotation_headers']['annotationHeaderText'] = $missing_annotation_count." missing annotation".$annotation_plural;
            $result['annotation_headers']['annotationIcon'] = '<i class="fas fa-exclamation-triangle align-baseline fa-lg text-wine pull-right"></i>';
            $canPublish = false;
        }
        else{
            $result['annotation_headers']['annotationHeaderText'] = "";
            $result['annotation_headers']['annotationIcon'] = '<i class="fas fa-check-circle align-baseline fa-lg text-wine pull-right"></i>';
        }

        $result['canPublish'] = $canPublish;

        if($canPublish) {
            $result['subtitle'] = "The criteria has been fulfilled: ";
        }

        /**
         * @todo
         * */
        //corpusdata formats

        //license


        $response = array(
            'status' => 'success',
            'msg' => $result,
        );

        return Response::json($response);
    }

    /**
     * @param Request $request
     * @return mixed
     */
    public function checkCorpusContent(Request $request) {

        $result = array();
        $corpusid = $request->input('corpusid');
        $corpus = Corpus::findOrFail($corpusid);
        $corpuspath = $request->input('corpuspath');


        $checkResult = json_decode($this->GitRepoService->checkForCorpusFiles($corpuspath."/TEI-HEADERS"), true);
        $result['checkdata'] = $checkResult;

        $response = array(
            'status' => 'success',
            'msg' => $result,
        );

        return Response::json($response);
    }

    public function mintDOIIdentifier($corpusid){
        $identifier = null;

        //get publishing metadata
        $corpus = Corpus::findOrFail($corpusid);
        $documentUri = $this->base_domain."api/elasticapi/v1/corpora/".$corpus->elasticsearch_id."/corpora";
        $resultData = $this->DOIService->mintDOIDataIdentifier($documentUri);


        if($resultData['status'] == 200){
            $data_cite_xml = $resultData['xml'];

            $doi = new Doi();
            $doi['doi_id'] = $resultData['identifier'];
            $identifier = $resultData['identifier'];
            $doi['corpus_id'] = $corpus->id;
            $doi['datacite_resource_xml'] = $resultData['xml'];
            $doi['resolve_uri'] =  $resultData['url'];
            $doi->save();
        }

        return $identifier;
    }


    public function resolveHasAnnotations($annotationData, $documentData ){
        $newIds  = array();
        foreach($annotationData as $oldAnnotationId => $annotationDatum) {
            foreach($annotationDatum as $label => $labelId) {
                if($label == "oldelasticid" && in_array($labelId,$documentData)) {
                    array_push($newIds,$annotationDatum["newelasticid"]);
                }
            }
        }
        return $newIds;
    }



    public function resolveHasDocuments($documentData, $annotationData ){
        $newIds = array();
        foreach($documentData as $oldDocumentId => $documentDatum) {
            foreach($documentDatum as $label => $labelId) {

                foreach($annotationData as $annotationId => $annotationDocuments) {
                    if($label = "oldelasticid" && in_array($labelId, $annotationDocuments)) {
                        if(!array_key_exists($annotationId,$newIds)){
                            $newIds[$annotationId] = array();
                        }
                        array_push($newIds[$annotationId],$documentDatum["newelasticid"]);
                    }
                }

            }
        }
        return $newIds;
    }
}
