<?php
/**
 * Created by PhpStorm.
 * User: rolfguescini
 * Date: 24.03.17
 * Time: 16:22
 */

namespace App\Http\Controllers;

use App\Custom\ElasticsearchInterface;
use App\Custom\LaudatioUtilsInterface;
use App\Exceptions\LaudatioElasticsearchException;
use Illuminate\Http\Request;
use JavaScript;
use Log;
use Cache;
use Redis;
use Session;
use App\Http\Requests\SearchRequest;

/**
 * Class SearchController
 * @package App\Http\Controllers
 */
class SearchController extends Controller
{
    protected $ElasticService;
    protected $LaudatioUtils;

    /**
     * SearchController constructor.
     * @param ElasticsearchInterface $Elasticservice
     * @param LaudatioUtilsInterface $laudatioUtils
     */
    public function __construct(ElasticsearchInterface $Elasticservice, LaudatioUtilsInterface $laudatioUtils)
    {
        $this->ElasticService = $Elasticservice;
        $this->LaudatioUtils = $laudatioUtils;
    }

    /**
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function index() {
        $isLoggedIn = \Auth::check();
        try{
            $corpusresponses = $this->ElasticService->pingServer();
        }
        catch (LaudatioElasticsearchException $laudatioElasticsearchException) {
            $message = $laudatioElasticsearchException->createUserFacingMessage($laudatioElasticsearchException->getMessage());
            //Log the error to file
            $laudatioElasticsearchException->logErrorMessage($laudatioElasticsearchException->getMessage(),['file' => __FILE__, 'line' => __LINE__]);
            return back()->withError($message);
        }

        $frontpagequery = "";
        if(!empty(Session::get('frontpagequery'))) {
            $frontpagequery = Session::get('frontpagequery');
        }

        JavaScript::put([
            "baseDomain" => config('laudatio.base_domain'),
            "elasticApiVersion" => config('laudatio.elasticApiVersion'),
            "elasticsearchLimit" => config('laudatio.elasticsearch_limit'),
        ]);
        return view('search.search')
            ->with('isLoggedIn', $isLoggedIn)
            ->with('baseDomain',config('laudatio.base_domain'))
            ->with('elasticApiVersion',config('laudatio.elasticApiVersion'))
            ->with('frontpagequery',$frontpagequery);
        //return view('search.search')
    }

    /**
     * frontPageSearch: catch searches from the frontpage and rout it to the search
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\Routing\Redirector
     *
     */
    public function frontPageSearch(Request $request) {
        $isLoggedIn = \Auth::check();

        if(!empty($request->input('query'))){
            $query = $request->input('query');

            JavaScript::put([
                "baseDomain" => config('laudatio.base_domain'),
                "elasticApiVersion" => config('laudatio.elasticApiVersion'),
                "elasticsearchLimit" => config('laudatio.elasticsearch_limit'),
            ]);

            return redirect('search')
                ->with('isLoggedIn', $isLoggedIn)
                ->with('frontpagequery',$query);
        }
        else{
            return redirect('/');
        }

    }

    public function emptySearchCache(){
        $result = 0;
        $corpusId = "7";
        $index = "poop";
        Cache::tags(['corpus_'.$corpusId.'_'.$index])->flush();

        Cache::tags(['formats_'.$corpusId.'_'.$index])->flush();

        $indexes = $this->LaudatioUtils->transformElasticSearchIndexes($index);
        $guidelineIndex = $indexes["guidelineIndex"];
        Cache::tags(['guidelines_'.$corpusId.'_'.$guidelineIndex])->flush();

        $redis = Cache::getRedis();
        $redis->connect('xxxxxx', 6379); // use your Host from Redis desktop Manager - connection
        $redis->select('1');
        $corpora =  $redis->keys('*corpora*');

        foreach ($corpora as $lkey){
            $result = $redis->del($lkey);
        }



        return response(
            "DONE: ".$result,
            200
        );
    }

}