<?php

/**
 * Created by PhpStorm.
 * User: rolfguescini
 * Date: 04.07.17
 * Time: 15:21
 */
namespace App\Laudatio\Elasticsearch;

use ONGR\ElasticsearchDSL\Query\Compound\BoolQuery;
use ONGR\ElasticsearchDSL\Query\FullText\MatchQuery;
use ONGR\ElasticsearchDSL\Query\MatchAllQuery;
use ONGR\ElasticsearchDSL\Query\FullText\MultiMatchQuery;
use ONGR\ElasticsearchDSL\Query\TermLevel\RangeQuery;
use ONGR\ElasticsearchDSL\Query\TermLevel\TermQuery;
use ONGR\ElasticsearchDSL\Query\TermLevel\IdsQuery;
use ONGR\ElasticsearchDSL\Query\TermLevel\ExistsQuery;
use ONGR\ElasticsearchDSL\Aggregation\Bucketing\DateRangeAggregation;
use ONGR\ElasticsearchDSL\Aggregation\Bucketing\RangeAggregation;
use ONGR\ElasticsearchDSL\Aggregation\Bucketing\TermsAggregation;
use ONGR\ElasticsearchDSL\Highlight\Highlight;
use ONGR\ElasticsearchDSL\Query\TermLevel\WildcardQuery;
use ONGR\ElasticsearchDSL\Search;
use Log;

class QueryBuilder
{

    public function buildMustQuery($data){
        $bool = new BoolQuery();
        foreach($data as $param){
            foreach ($param as $key => $value){
                $bool->add(new MatchQuery($key, $value), BoolQuery::MUST);
            }
        }

        $search = new Search();
        $search->addQuery($bool);

        $queryArray = $search->toArray();

        return $queryArray;
    }

    public function buildTermQuery($data) {
        $termFilter = new TermQuery($data['field'],$data['value']);
        $search = new Search();
        $search->addQuery($termFilter);

        $queryArray = $search->toArray();

        return $queryArray;
    }


    public function buildTermsQuery($data) {
        $search = new Search();
        foreach($data as $datum) {
            $termFilter = new TermQuery($datum['field'],$datum['value']);
            $search->addQuery($termFilter);
        }


        return $search->toArray();
    }

    public function buildFilteredMustExistQuery($data,$filters){
        $bool = new BoolQuery();

        foreach ($data as $key => $value){
            $bool->add(new ExistsQuery($value), BoolQuery::MUST);
        }


        foreach ($filters as $field => $value){
            $termFilter = new TermQuery($field,$value);
            $bool->add($termFilter, BoolQuery::FILTER);
        }


        $search = new Search();
        $search->addQuery($bool);

        $queryArray = $search->toArray();

        return $queryArray;
    }

    public function buildIdsQuery($ids){

        $idsQuery = new IdsQuery($ids);

        $search = new Search();
        $search->addQuery($idsQuery);

        $queryArray = $search->toArray();
        return $queryArray;
    }

    public function buildFilteredMustQuery($data,$filters){
        $bool = new BoolQuery();

        foreach ($data as $key => $value){
            $bool->add(new MatchQuery($key, $value), BoolQuery::MUST);
        }


        foreach ($filters as $field => $value){
            $termFilter = new TermQuery($field,$value);
            $bool->add($termFilter, BoolQuery::FILTER);
        }


        $search = new Search();
        $search->addQuery($bool);

        return $search->toArray();
    }

    public function buildSingleMatchQuery($data){
        $matchQuery = null;
        foreach($data as $param){
            foreach ($param as $key => $value){
                if($value != ""){
                    $matchQuery = new MatchQuery($key, $value);
                }

            }
        }

        $search = new Search();
        $search->addQuery($matchQuery);

        $queryArray = $search->toArray();

        return $queryArray;
    }

    public function buildMultiMatchQuery($data){
        $multiMatchQuery = null;
        $multiMatchQuery = new MultiMatchQuery($data['fields'],$data['query']);
        $search = new Search();
        $search->addQuery($multiMatchQuery);
        $queryArray = $search->toArray();
        return $queryArray;
    }

    public function buildMultiMatchHighlightQuery($data){
        $multiMatchQuery = null;
        $multiMatchQuery = new MultiMatchQuery($data['fields'],$data['query']);
        $search = new Search();
        $search->addQuery($multiMatchQuery);

        $highlight = new Highlight();
        foreach ($data['fields'] as $field) {
            $highlight->addField($field);
        }
        $highlight->setTags(['<span class="laudatiohighlight">'], ['</span>']);
        $search->addHighlight($highlight);
        $queryArray = $search->toArray();
        return $queryArray;
    }


    public function buildMixedQuery($data,  $fullTextQuery, $fulltextFields, $filters, $highlights,$hasWildCards){
        $filtered = false;
        $highlighted = false;
        $bool = new BoolQuery();
        $search = new Search();

        if(count($filters) > 0) {
            $filtered = true;
        }

        if(count($highlights) > 0) {
            $highlighted = true;
        }

        $addedqueries = [];

        $hasMoreThanOneRangeCounter = 0;

        foreach ($data as $object){
            if(empty($object->value)){
                $rangeQuery = null;

                if($object->from && !$object->to){
                    $rangeQuery = new RangeQuery(
                        $object->field,
                        [
                            'gte' => $object->from,
                        ]
                    );
                }
                else if(!$object->from && $object->to){
                    $rangeQuery = new RangeQuery(
                        $object->field,
                        [
                            'lte' => $object->to
                        ]
                    );
                }
                else if($object->from && $object->to){
                    $rangeQuery = new RangeQuery(
                        $object->field,
                        [
                            'gte' => $object->from,
                            'lte' => $object->to
                        ]
                    );
                }

                $bool->add($rangeQuery, BoolQuery::MUST);
            }
            else{

                if($object->field == "corpus_publication_license") {
                    $newValueHTTP = "*/".$object->value."/*";
                    $bool->add(new WildcardQuery($object->field, $newValueHTTP), BoolQuery::MUST);
                }
                else if($object->field == "corpus_merged_formats"){

                    //$bool->add(new MatchQuery($object->field, strtolower($object->value)), BoolQuery::MUST);
                    $bool->add(new MatchQuery($object->field, $object->value), BoolQuery::MUST);
                }
                else{
                    if(strpos($object->value,'?') > -1 || strpos($object->value,'*') > -1 ){
                        $bool->add(new WildcardQuery($object->field, strtolower($object->value)), BoolQuery::MUST);
                    }
                    else{
                        $bool->add(new MatchQuery($object->field, strtolower($object->value)), BoolQuery::MUST);
                    }

                }
            }

        }


        if(null != $fullTextQuery && !$hasWildCards) {
            $multiMatchQuery = new MultiMatchQuery($fulltextFields,$fullTextQuery);
            $bool->add($multiMatchQuery, BoolQuery::MUST);
        }
        else if (null != $fullTextQuery && $hasWildCards) {
            $wildBool = new BoolQuery();
            foreach ($fulltextFields as $fullTextField) {
                $wildBool->add(new WildcardQuery($fullTextField,$fullTextQuery),BoolQuery::SHOULD);
            }
            $bool->add($wildBool,BoolQuery::MUST);
        }

        if($filtered) {
            foreach ($filters as $field => $value){
                $termFilter = new TermQuery($field,$value);
                $bool->add($termFilter, BoolQuery::FILTER);
            }

        }

        $search->addQuery($bool);

        if($highlighted) {
            $highlight = new Highlight();
            foreach ($highlights as $highlightfield) {
                $highlight->addField($highlightfield);
            }

            if($filtered){
                $highlight->setTags(['<span class="laudatiofilterhighlight">'], ['</span>']);
            }
            else{
                $highlight->setTags(['<span class="laudatiohighlight">'], ['</span>']);
            }

            $search->addHighlight($highlight);
        }

        return $search->toArray();
    }


    public function buildDocumentMixedQuery($data,  $fullTextQuery, $fulltextFields, $filters, $highlights, $hasWildCards){
        $filtered = false;
        $highlighted = false;
        $bool = new BoolQuery();
        $search = new Search();

        if(count($filters) > 0) {
            $filtered = true;
        }

        if(count($highlights) > 0) {
            $highlighted = true;
        }


        $rangeBool = new BoolQuery();

        $addedqueries = [];

        $hasMoreThanOneRangeCounter = 0;
        foreach ($data as $object){
            if(empty($object->value) && ($object->field == "document_history_not_before" || $object->field == "document_history_not_after" || $object->field == "document_publication_publishing_date")){
                $hasMoreThanOneRangeCounter++;
            }
        }

        $shouldBool = null;
        if($hasMoreThanOneRangeCounter) {
            $shouldBool = new BoolQuery();
        }

        foreach ($data as $object){
            if(empty($object->value)){
                $rangeQuery = null;

                if($object->from && !$object->to){
                    $rangeQuery = new RangeQuery(
                        $object->field,
                        [
                            'gte' => $object->from,
                        ]
                    );
                }
                else if(!$object->from && $object->to){
                    $rangeQuery = new RangeQuery(
                        $object->field,
                        [
                            'lte' => $object->to
                        ]
                    );
                }
                else if($object->from && $object->to){
                    $rangeQuery = new RangeQuery(
                        $object->field,
                        [
                            'gte' => $object->from,
                            'lte' => $object->to
                        ]
                    );
                }
                if($object->field == "document_history_not_before" || $object->field == "document_history_not_after" || $object->field == "document_publication_publishing_date") {
                    if($hasMoreThanOneRangeCounter > 1){
                        $shouldBool->add($rangeQuery, BoolQuery::SHOULD);
                    }
                    else {
                        $bool->add($rangeQuery, BoolQuery::MUST);
                    }
                }
                else{
                    $bool->add($rangeQuery, BoolQuery::MUST);
                }

                //$bool->add($rangeBool, BoolQuery::MUST);
            }
            else{
                if(!in_array($object->field."_".strtolower($object->value),$addedqueries)) {
                    if(strpos($object->value,'?') > -1 || strpos($object->value,'*') > -1 ){
                        $bool->add(new WildcardQuery($object->field, strtolower($object->value)), BoolQuery::MUST);
                    }
                    else{
                        $bool->add(new MatchQuery($object->field, strtolower($object->value)), BoolQuery::MUST);
                    }
                }
                array_push($addedqueries,$object->field."_".strtolower($object->value));
            }
        }
        if($hasMoreThanOneRangeCounter > 1){
            $shouldBool->addParameter("minimum_should_match",1);
            $bool->add($shouldBool, BoolQuery::MUST);
        }

        //Log::info("documentrangebuikderbuildDocumentMixedQuery: before wildcardtest: bool: ".print_r($bool,1));
        if(null != $fullTextQuery && !$hasWildCards) {
            $multiMatchQuery = new MultiMatchQuery($fulltextFields,strtolower($fullTextQuery));
            $bool->add($multiMatchQuery, BoolQuery::MUST);
        }
        else if (null != $fullTextQuery && $hasWildCards) {
            $wildBool = new BoolQuery();
            foreach ($fulltextFields as $fullTextField) {
                $wildBool->add(new WildcardQuery($fullTextField,strtolower($fullTextQuery)),BoolQuery::SHOULD);
            }
            $bool->add($wildBool,BoolQuery::MUST);
        }

        if($filtered) {
            foreach ($filters as $field => $value){
                $termFilter = new TermQuery($field,$value);
                $bool->add($termFilter, BoolQuery::FILTER);
            }
        }

        $search->addQuery($bool);

        if($highlighted) {
            $highlight = new Highlight();
            foreach ($highlights as $highlightfield) {
                $highlight->addField($highlightfield);
            }

            if($filtered){
                $highlight->setTags(['<span class="laudatiofilterhighlight">'], ['</span>']);
            }
            else{
                $highlight->setTags(['<span class="laudatiohighlight">'], ['</span>']);
            }

            $search->addHighlight($highlight);
        }

        return $search->toArray();
    }

    public function buildFilteredHighlightedWildCardQuery($data){
        $bool = new BoolQuery();
        $search = new Search();
        foreach($data['fields'] as $field) {
                if($field == "corpus_publication_license") {
                    $shouldbool = new BoolQuery();
                    if(is_array($data['query'])) {
                        foreach($data['query'] as $valueitem) {
                            $newValueHTTP = "*/".strtolower($valueitem)."/*";
                            $bool->add(new WildcardQuery($field, $newValueHTTP), BoolQuery::SHOULD);
                        }
                    }
                    else{
                        $newValueHTTP = "*/".strtolower($data['query'])."/*";
                        $bool->add(new WildcardQuery($field, $newValueHTTP), BoolQuery::SHOULD);
                    }

                }
                else if($field == "corpus_merged_formats" || $field == "annotation_merged_formats") {
                    if(is_array($data['query'])) {
                        foreach ($data['query'] as $format) {
                            $bool->add(new WildcardQuery($field, strtolower($format)), BoolQuery::SHOULD);
                        }
                    }
                    else{
                        $bool->add(new WildcardQuery($field, strtolower($data['query'])), BoolQuery::SHOULD);
                    }
                }
                else{
                    if(is_array($data['query'])) {
                        foreach ($data['query'] as $item) {
                            $bool->add(new WildcardQuery($field, strtolower($item)), BoolQuery::SHOULD);
                        }
                    }
                    else{
                        $bool->add(new WildcardQuery($field, strtolower($data['query'])), BoolQuery::SHOULD);
                    }

                }
        }
        $termFilter = new TermQuery("is_latest_publication","true");
        $bool->add($termFilter,BoolQuery::FILTER);
        $bool->addParameter("minimum_should_match",1);
        $search->addQuery($bool);

        $highlight = new Highlight();
        foreach ($data['fields'] as $field) {
            $highlight->addField($field);
        }
        $highlight->setTags(['<span class="laudatiofilterhighlight">'], ['</span>']);
        $search->addHighlight($highlight);


        return $search->toArray();
    }

    public function buildFilteredHighlightedMustQuery($data, $fullTextQuery, $hasWildCards, $filters){
        $bool = new BoolQuery();
        $search = new Search();

        foreach ($data as $key => $value){
            if($key != "fields"){
                if($key == "corpus_publication_license") {
                    $shouldbool = new BoolQuery();
                    foreach($value as $valueitem) {
                        $newValueHTTP = "*/".strtolower($valueitem)."/*";
                        $shouldbool->add(new WildcardQuery($key, $newValueHTTP), BoolQuery::MUST);
                    }
                    $bool->add($shouldbool, BoolQuery::MUST);

                }
                else if($key == "corpus_merged_formats" || $key == "annotation_merged_formats") {
                    $shouldbool = new BoolQuery();
                    foreach ($value as $format) {
                        $shouldbool->add(new MatchQuery($key, $format), BoolQuery::MUST);
                    }
                    $bool->add($shouldbool, BoolQuery::MUST);
                }
                else{
                   // $shouldbool = new BoolQuery();
                    if(is_array($value)) {
                        foreach($value as $valueitem) {
                            if(strpos($valueitem,'?') > -1 || strpos($valueitem,'*') > -1 ){
                                //$shouldbool->add(new WildcardQuery($key, strtolower($valueitem)), BoolQuery::MUST);
                                $bool->add(new WildcardQuery($key, strtolower($valueitem)), BoolQuery::MUST);
                            }
                            else{
                                //$shouldbool->add(new MatchQuery($key, strtolower($valueitem)), BoolQuery::MUST);
                                $bool->add(new MatchQuery($key, strtolower($valueitem)), BoolQuery::MUST);
                            }

                        }
                    }
                    else{
                        if(strpos($value,'?') > -1 || strpos($value,'*') > -1 ){
                            //$shouldbool->add(new WildcardQuery($key, strtolower($valueitem)), BoolQuery::MUST);
                            $bool->add(new WildcardQuery($key, strtolower($value)), BoolQuery::MUST);
                        }
                        else{
                            //$shouldbool->add(new MatchQuery($key, strtolower($valueitem)), BoolQuery::MUST);
                            $bool->add(new MatchQuery($key, strtolower($value)), BoolQuery::MUST);
                        }
                    }

                    //$bool->add($shouldbool, BoolQuery::MUST);
                }
            }
        }

        if(null != $fullTextQuery && !$hasWildCards) {
            $multiMatchQuery = new MultiMatchQuery($data['fields'],$fullTextQuery);
            $bool->add($multiMatchQuery, BoolQuery::MUST);
        }
        else if (null != $fullTextQuery && $hasWildCards) {
            $wildBool = new BoolQuery();
            foreach ($data['fields'] as $fullTextField) {
                $wildBool->add(new WildcardQuery($fullTextField,strtolower($fullTextQuery)),BoolQuery::SHOULD);
            }
            $wildBool->addParameter("minimum_should_match",1);
            $bool->add($wildBool,BoolQuery::MUST);
        }

        foreach ($filters as $field => $value){
            $termFilter = new TermQuery($field,$value);
            $bool->add($termFilter, BoolQuery::FILTER);
        }



        $search->addQuery($bool);

        $highlight = new Highlight();
        foreach ($data['fields'] as $field) {
            $highlight->addField($field);
        }
        $highlight->setTags(['<span class="laudatiofilterhighlight">'], ['</span>']);
        $search->addHighlight($highlight);


        return $search->toArray();
    }


    public function buildBoolMultiMatchFilteredHighlightQuery($data,$size){
        $bool = new BoolQuery();
        $multiMatchQuery = null;
        $multiMatchQuery = new MultiMatchQuery($data['fields'],$data['query']);


        $bool->add($multiMatchQuery, BoolQuery::MUST);

        $termFilter = new TermQuery("is_latest_publication","true");
        $bool->add($termFilter,BoolQuery::FILTER);

        $search = new Search();
        $search->addQuery($bool);

        if($size < 10000){
            $highlight = new Highlight();
            foreach ($data['fields'] as $field) {
                $highlight->addField($field);
            }
            $highlight->setTags(['<span class="laudatiohighlight">'], ['</span>']);
            $search->addHighlight($highlight);
        }

        $queryArray = $search->toArray();
        return $queryArray;
    }



    public function buildMatchAllQuery($data){
        $matchAllQuery = new MatchAllQuery($data);
        $search = new Search();
        $search->addQuery($matchAllQuery);
        $queryArray = $search->toArray();

        return $queryArray;
    }



    public function buildRangeQuery($field, $from = null, $to = null) {
        $rangeQuery = null;
        if($from && !$to){
            $rangeQuery = new RangeQuery(
                $field,
                [
                    'gte' => $from,
                ]
            );
        }
        else if(!$from && $to){
            $rangeQuery = new RangeQuery(
                $field,
                [
                    'lte' => $to
                ]
            );
        }
        else if($from && $to){
            $rangeQuery = new RangeQuery(
                $field,
                [
                    'gte' => $from,
                    'lte' => $to
                ]
            );
        }

        $search = new Search();
        $search->addQuery($rangeQuery);

        return $search->toArray();
    }

    public function buildMustFilterRangeQuery($fielddata, $rangedata) {
        $bool = new BoolQuery();
        $fieldMap = array(
            "corpusYearTo" => "corpus_publication_publication_date",
            "corpus_publication_publication_date" => "corpus_publication_publication_date",
            "corpusSizeTo" => "corpus_size_value",
            "corpus_size_value" => "corpus_size_value",
            "document_publication_publishing_date" => "document_publication_publishing_date",
            "document_publication_publishing_date_to" => "document_publication_publishing_date",
            "document_size_extent" => "document_size_extent",
            "document_size_extent_to" => "document_size_extent"

        );


        foreach($fielddata as $param){
            foreach ($param as $key => $value){
                $bool->add(new MatchQuery($key, $value), BoolQuery::MUST);
            }
        }
        foreach ($rangedata as $rangeDataKey => $rangedatum){
            $rangeQuery =  null;
            if($rangedatum != null && $rangedatum != ""){
                if(strpos($rangeDataKey,"To") !== false || strpos($rangeDataKey,"to") !== false){
                    $rangeQuery = new RangeQuery(
                        $fieldMap[$rangeDataKey],
                        [
                            'lte' => $rangedatum
                        ]
                    );
                }
                else{

                    if($fieldMap[$rangeDataKey] == "corpus_publication_publication_date" || $fieldMap[$rangeDataKey] == "document_publication_publishing_date"){
                        $rangeQuery = new RangeQuery(
                            $fieldMap[$rangeDataKey],
                            [
                                'gt' => $rangedatum
                            ]
                        );
                    }
                    else{
                        $rangeQuery = new RangeQuery(
                            $fieldMap[$rangeDataKey],
                            [
                                'gte' => $rangedatum
                            ]
                        );
                    }
                }
                $bool->add($rangeQuery, BoolQuery::FILTER);
            }
        }

        $search = new Search();
        $search->addQuery($bool);

        $queryArray = $search->toArray();
        return $queryArray;
    }


    public function buildMustRangeQuery($data, $filters) {
        $bool = new BoolQuery();
        foreach ($data as $value){
            if($value != ""){
                $rangeQuery = null;
                if($value->from && !$value->to){
                    $rangeQuery = new RangeQuery(
                        $value->field,
                        [
                            'gte' => $value->from,
                        ]
                    );
                }
                else if(!$value->from && $value->to){
                    $rangeQuery = new RangeQuery(
                        $value->field,
                        [
                            'lte' => $value->to
                        ]
                    );
                }
                else if($value->from && $value->to){
                    $rangeQuery = new RangeQuery(
                        $value->field,
                        [
                            'gte' => $value->from,
                            'lte' => $value->to
                        ]
                    );
                }

                $bool->add($rangeQuery, BoolQuery::MUST);
            }

        }
        $search = new Search();
        $search->addQuery($bool);

        foreach ($filters as $field => $value){
            $termFilter = new TermQuery($field,$value);
            $bool->add($termFilter, BoolQuery::FILTER);
        }

        $search->addQuery($bool);

        return $search->toArray();
    }


    public function buildHighlightedCorpusRangeQuery($data, $fullTextSearch, $fullTextFields,  $filterfields, $highlightfields, $hasWildCards) {

        $bool = new BoolQuery();

        foreach ($data as $value){
            if($value != ""){
                $rangeQuery = null;
                if($value->from && !$value->to){
                    $rangeQuery = new RangeQuery(
                        $value->field,
                        [
                            'gte' => $value->from,
                        ]
                    );
                }
                else if(!$value->from && $value->to){
                    $rangeQuery = new RangeQuery(
                        $value->field,
                        [
                            'lte' => $value->to
                        ]
                    );
                }
                else if($value->from && $value->to){
                    $rangeQuery = new RangeQuery(
                        $value->field,
                        [
                            'gte' => $value->from,
                            'lte' => $value->to
                        ]
                    );
                }

                $bool->add($rangeQuery, BoolQuery::SHOULD);
            }

        }

        if(null != $fullTextSearch && !$hasWildCards) {
            $multiMatchQuery = new MultiMatchQuery($fullTextFields,$fullTextSearch);
            $bool->add($multiMatchQuery, BoolQuery::MUST);
        }
        else if (null != $fullTextSearch && $hasWildCards) {
            $wildBool = new BoolQuery();
            foreach ($fullTextFields as $fullTextField) {
                $wildBool->add(new WildcardQuery($fullTextField,$fullTextSearch),BoolQuery::SHOULD);
            }
            $bool->add($wildBool,BoolQuery::MUST);
        }

        $search = new Search();
        $highlight = new Highlight();

        foreach ($filterfields as $field => $value){
            $termFilter = new TermQuery($field,$value);
            $bool->add($termFilter, BoolQuery::FILTER);
        }

        foreach($highlightfields as $highlightfield) {
            $highlight->addField($highlightfield);
        }

        $bool->addParameter("minimum_should_match",1);
        $search->addQuery($bool);



        $highlight->setTags(['<span class="laudatiofilterhighlight">'],  ['</span>']);
        $search->addHighlight($highlight);

        return $search->toArray();
    }

    public function buildHighlightedDateDocumentRangeQuery($data, $fullTextSearch, $fullTextFields, $filterfields, $highlightfields,$hasWildCards) {
        $shouldbool = new BoolQuery();
        $bool = new BoolQuery();

        foreach ($data as $value){

                $rangeQuery = null;
                if($value->from && !$value->to){
                    $rangeQuery = new RangeQuery(
                        $value->field,
                        [
                            'gte' => $value->from,
                        ]
                    );

                }
                else if(!$value->from && $value->to){
                    $rangeQuery = new RangeQuery(
                        $value->field,
                        [
                            'lte' => $value->to
                        ]
                    );

                }
                else if($value->from && $value->to){
                    $rangeQuery = new RangeQuery(
                        $value->field,
                        [
                            'gte' => $value->from,
                            'lte' => $value->to
                        ]
                    );
                }

                if( $value->field == "document_history_not_before" || $value->field == "document_history_not_after"){
                    $bool->add($rangeQuery, BoolQuery::MUST);

                    if(!$this->shouldBoolHasQuery($shouldbool->toArray(), $bool->toArray())) {
                        $shouldbool->add($bool,BoolQuery::SHOULD);
                    }

                }
                else{
                    $otherbool = new BoolQuery();
                    $otherbool->add($rangeQuery, BoolQuery::MUST);
                    $shouldbool->add($otherbool,BoolQuery::SHOULD);
                }
        }


        $search = new Search();
        $shouldbool->addParameter("minimum_should_match",1);



        if(null != $fullTextSearch && !$hasWildCards) {
            $multiMatchQuery = new MultiMatchQuery($fullTextFields,strtolower($fullTextSearch));
            $shouldbool->add($multiMatchQuery, BoolQuery::MUST);
        }
        else if (null != $fullTextSearch && $hasWildCards) {
            $wildBool = new BoolQuery();
            foreach ($fullTextFields as $fullTextField) {
                $wildBool->add(new WildcardQuery($fullTextField,strtolower($fullTextSearch)),BoolQuery::SHOULD);
            }
            $shouldbool->add($wildBool,BoolQuery::MUST);
        }

        $highlight = new Highlight();

        foreach ($filterfields as $field => $value){
            $termFilter = new TermQuery($field,$value);
            $shouldbool->add($termFilter, BoolQuery::FILTER);
        }

        foreach($highlightfields as $highlightfield) {
            $highlight->addField($highlightfield);
        }

        $search->addQuery($shouldbool);
        //$search->addQuery($bool);



        $highlight->setTags(['<span class="laudatiofilterhighlight">'],  ['</span>']);
        $search->addHighlight($highlight);


        return $search->toArray();
    }

    public function buildHighlightedDocumentRangeQuery($data, $fullTextSearch, $fullTextFields, $filterfields, $highlightfields,$hasWildCards) {

        $bool = new BoolQuery();

        $hasMoreThanOneRangeCounter = 0;
        foreach ($data as $object){
            if(empty($object->value) && ($object->field == "document_size_extent.string"  || $object->field == "document_size_extent_to.string")){
                $hasMoreThanOneRangeCounter++;
            }
        }

        foreach ($data as $value){
            if($value != ""){
                $rangeQuery = null;
                if($value->from && !$value->to){
                    $rangeQuery = new RangeQuery(
                        $value->field,
                        [
                            'gte' => $value->from,
                        ]
                    );
                }
                else if(!$value->from && $value->to){
                    $rangeQuery = new RangeQuery(
                        $value->field,
                        [
                            'lte' => $value->to
                        ]
                    );
                }
                else if($value->from && $value->to){
                    $rangeQuery = new RangeQuery(
                        $value->field,
                        [
                            'gte' => $value->from,
                            'lte' => $value->to
                        ]
                    );
                }

                if($hasMoreThanOneRangeCounter > 1){
                    $bool->add($rangeQuery, BoolQuery::SHOULD);
                    $bool->addParameter("minimum_should_match",1);
                }
                else {
                    $bool->add($rangeQuery, BoolQuery::MUST);
                }
                //$bool->add($rangeQuery, BoolQuery::SHOULD);
            }

        }


        if(null != $fullTextSearch && !$hasWildCards) {
            $multiMatchQuery = new MultiMatchQuery($fullTextFields,strtolower($fullTextSearch));
            $bool->add($multiMatchQuery, BoolQuery::MUST);
        }
        else if (null != $fullTextSearch && $hasWildCards) {
            $wildBool = new BoolQuery();
            foreach ($fullTextFields as $fullTextField) {
                $wildBool->add(new WildcardQuery($fullTextField,strtolower($fullTextSearch)),BoolQuery::SHOULD);
            }
            $bool->add($wildBool,BoolQuery::MUST);
        }

        $search = new Search();
        $highlight = new Highlight();

        foreach ($filterfields as $field => $value){
            $termFilter = new TermQuery($field,$value);
            $bool->add($termFilter, BoolQuery::FILTER);
        }

        foreach($highlightfields as $highlightfield) {
            $highlight->addField($highlightfield);
        }


        $search->addQuery($bool);



        $highlight->setTags(['<span class="laudatiofilterhighlight">'],  ['</span>']);
        $search->addHighlight($highlight);

        return $search->toArray();
    }

    public function buildTermsAggregationQuery($data){
        $termAggregation = new TermsAggregation($data['name']);
        $termAggregation->setField($data['field']);

        $matchAllQuery = new MatchAllQuery();
        $search = new Search();
        $search->addQuery($matchAllQuery);

        $search->addAggregation($termAggregation);
        $queryArray = $search->toArray();

        return $queryArray;
    }

    public function buildTermsAggregationQueryByMatchQuery($matchData, $aggregationData){
        $termAggregation = new TermsAggregation($aggregationData['name']);
        $termAggregation->setField($aggregationData['field']);

        $matchQuery = new MatchQuery($matchData['field'], $matchData['value']);
        $search = new Search();
        $search->addQuery($matchQuery);

        $search->addAggregation($termAggregation);
        $queryArray = $search->toArray();

        return $queryArray;
    }

    public function shouldBoolHasQuery($containerboolArray,$boolToAddArray) {
        $hasQuery = false;
        foreach ($containerboolArray as $item) {
            $itemarray =  (array) $item;
            if(isset($itemarray['should']) && in_array($boolToAddArray,$itemarray['should'])){
                $hasQuery = true;
                break;
            }
        }
        return $hasQuery;
    }
}