<?php
/**
 * Created by PhpStorm.
 * User: rolfguescini
 * Date: 11.01.19
 * Time: 13:03
 */

namespace App\Laudatio\GitLab;

use App\Exceptions\CorpusProjectAlreadyExistsException;
use App\Exceptions\CreateGitLabResourceException;
use App\Exceptions\GitNetworkException;
use GrahamCampbell\GitLab\Facades\GitLab;
use App\Custom\GitLabInterface;
use Log;
use Gitlab\Exception\ErrorException;
use Http\Client\Exception\NetworkException;
use Predis\Cluster\Distributor\EmptyRingException;

/**
 * Class GitLabService
 * @package App\Laudatio\GitLab
 */
class GitLabService implements GitLabInterface
{
    /**
     *
     * getGitLabGroups
     *
     * @return mixed
     */
    public function getGitLabGroups(){
        return GitLab::api('groups')->all();
    }

    /**
     *
     * showGitLabGroup
     *
     * @param $groupId
     * @return mixed
     */
    public function showGitLabGroup($groupId){
        return GitLab::api('groups')->show($groupId);
    }

    /**
     *
     * createGitLabGroup
     *
     * @param $name
     * @param $path
     * @param null $description
     * @param string $visibility
     * @return mixed
     */
    public function createGitLabGroup($name, $path, $description = null, $visibility = 'public'){

        try{
            $gitLabResponse = GitLab::api('groups')->create($name, $path, $description,$visibility);
        }
        catch(NetworkException $networkException){
            throw new GitNetworkException($networkException->getMessage(),0,null);
        }
        catch(\Gitlab\Exception\ErrorException $ex){
            throw new CorpusProjectAlreadyExistsException($ex->getMessage(),0,null);
        }

        return $gitLabResponse;
    }

    /**
     *
     * updateGitLabGroup
     *
     * @param $groupId
     * @param $params
     * @return mixed
     */
    public function updateGitLabGroup($groupId,$params){
        return GitLab::api('groups')->update($groupId,$params);
    }

    /**
     *
     * deleteGitLabGroup
     *
     * @param $groupId
     * @return mixed
     */
    public function deleteGitLabGroup($groupId){
        return GitLab::api('groups')->remove($groupId);
    }


    /**
     *
     * createGitLabProject
     *
     * @param $name
     * @param $groupId
     * @param null $description
     * @param string $visibility
     * @return mixed
     */

    public function createGitLabProject($name, $groupId, $description = null,$visibility = 'public'){

        $gitLabResponse = GitLab::api('projects')->create($name, $groupId, $description, $visibility);
        return $gitLabResponse;
    }


    public function updateGitLabProject($projectId,$params) {
        return  GitLab::api('projects')->update($projectId,$params);
    }

    /**
     *
     * deleteGitLabProject
     *
     * @param $projectId
     * @return mixed
     */
    public function deleteGitLabProject($projectId){
        return GitLab::api('projects')->remove($projectId);
    }


    /**
     *
     * updateGitLabTag
     *
     * @param $gitLabId
     * @param $tagName
     * @param $releaseDescription
     * @return mixed
     */
    public function updateGitLabTag($gitLabId,$tagName,$releaseDescription){
        return GitLab::api('projects')->update();
    }


    /**
     *
     * unlinkProjectFromGroup
     *
     * @param $projectId
     * @param $groupId
     * @return mixed
     */
    public function unlinkProjectFromGroup($projectId,$groupId){
        return $this->delete('projects/'.$this->encodePath($projectId).'/share/'.$this->encodePath($groupId));
    }



}