<?php
/**
 * Created For Laudatio.
 * User: rolfguescini
 * Date: 2019-03-24
 * Time: 01:21
 */

namespace App\Laudatio\Utils;

use App\Custom\DOIInterface;
use Symfony\Component\Process\Process;
use Symfony\Component\Process\Exception\ProcessFailedException;
use Symfony\Component\Process\Exception\RuntimeException;
use Log;

class DOIService implements DOIInterface
{
    protected $client;
    protected $username;
    protected $password;
    protected $doi_shoulder;
    protected $ark_shoulder;
    protected $basePath;
    protected $scriptPath;


    public function __construct()
    {
        $datacite_environment = config('laudatio.datacite_environment');
        if($datacite_environment == "debug") {
            $this->username = config('laudatio.datacite_test_username');
            $this->password = config('laudatio.datacite_test_password');
            $this->doi_shoulder = config('laudatio.datacite_doi_test_shoulder');
        }
        else if($datacite_environment == "production") {
            $this->username = config('laudatio.datacite_username');
            $this->password = config('laudatio.datacite_password');
            $this->doi_shoulder = config('laudatio.datacite_doi_shoulder');
        }

        $this->basePath = config('laudatio.base_domain');
        $this->scriptPath = config('laudatio.scriptPath');

        $opts = array(
            "username" => $this->username,
            "password" => $this->password,
            "doi_shoulder" => $this->doi_shoulder,
        );
    }

    public function mintDOIDataIdentifier($apiUrl){
        $results = array();

        $pythonScript = $this->scriptPath.'/src/DataCiteClient.py';
        //Log::info("mintDOIDataIdentifier: APIURL : ".$apiUrl);
        //Log::info("python3 ".$pythonScript." --u ".$this->username." -p ".$this->password." -a create -f ".$this->doi_shoulder." -d ".$apiUrl." -b ".$this->basePath);
        $process = new Process("python3 ".$pythonScript." -u ".$this->username." -p ".$this->password." -a create -f ".$this->doi_shoulder." -d ".$apiUrl." -b ".$this->basePath);
        $process->run();

        // executes after the command finishes

        if (!$process->isSuccessful()) {
            //  throw new ProcessFailedException($process);
            /*
             * @todo throw Exception
             */
        }
        else{
            $result = $process->getOutput();

            $result_decoded = json_decode($result,true);
            if($result_decoded['result'] == "OK") {
                $results['identifier'] = $result_decoded['doi'];
                $results['xml'] = $result_decoded['xml'];
                $results['url'] = $result_decoded['url'];
                $results['status'] = 200;
            }
            else{
                $results['error'] = "There was an error creating the DOI";
                $results['errortext'] = $result_decoded['errortext'];
                $results['status'] = $result_decoded['error']['status'];
            }

        }

        return $results;

    }

    public function getDOIData($identifier){
        //https://search.datacite.org/data-centers/tib.hu-laudatio
        /*To retrieve DOI metadata records users should use our DataCite REST API, which provides better performance, better query options, and doesn't require authentication.*/
        $response = $this->client->get_identifier_metadata($identifier);  // will get the meta sent in create()
        $meta_array = $this->client->parse_response_metadata((string)$response->getBody()); // Guzzle returns a stream, cast it to a string

        return $meta_array;
    }


    public function updateDOIData($identifier, $request) {
        $updated = false;

        $resourcePath = $request->input('path');
        $isActive = $request->input('isActive');
        $state = $request->input('state');

        $new_meta = array(
            "isActive" => $isActive,
            "state" => $state,
            "_target" => $resourcePath
        );

        $response = $this->client->modify_identifier_metadata($identifier, $new_meta);

        if($response->GetStatusCode() == 200) {
            $updated = true;

        }

        return $updated;
    }

    public function deleteDOIData($apiUrl){
        $results = array();

        $pythonScript = $this->scriptPath.'/src/DataCiteClient.py';

        //Log:info("python3 ".$pythonScript." --u ".$this->username." -p ".$this->password." -a delete -f ".$this->doi_shoulder." -d ".$apiUrl." -b ".$this->basePath);

        $process = new Process("python3 ".$pythonScript." --u ".$this->username." -p ".$this->password." -a delete -f ".$this->doi_shoulder." -d ".$apiUrl." -b ".$this->basePath);
        $process->run();

        // executes after the command finishes

        if (!$process->isSuccessful()) {
            //  throw new ProcessFailedException($process);
        }
        else{
            $result = $process->getOutput();
            $result_decoded = json_decode($result,true);
            if($result_decoded['result'] == "OK") {
                $results['status'] = 200;
            }
            else{
                $results['error'] = "There was an error deleting the DOI";
                $results['errortext'] = $result_decoded['errortext'];
                $results['status'] = $result_decoded['error']['status'];
            }

        }

        return $results;
    }

    public function deleteDoiByCurlRequest($doi){
        $header = array(
            "content-type: application/plain; charset=UTF-8"
        );

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl,CURLOPT_HTTPHEADER, $header);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $queries);
        $results = curl_exec($curl);
        $info = curl_getinfo($curl);
        curl_close($curl);
        $resultset = array(
            "curlinfo" => $info,
            "resultdata" => $results
        );
        return $resultset;
    }

}