<?php
/**
 * Created For Laudatio.
 * User: rolfguescini
 * Date: 2019-07-10
 * Time: 16:09
 */

namespace App\Laudatio\Utils;
use Adldap\Adldap;
use App\Custom\LDAPInterface;
use Log;


class LDAPService implements LDAPInterface
{

    protected $config;
    protected $adldap;
    protected $provider;
    public function __construct()
    {
        $this->config = [
            // Mandatory Configuration Options
            'hosts'            => ['ldapmaster.cms.hu-berlin.de'],
            'base_dn'          => config('external_base_dn','ou=users,ou=Laudatio,ou=Services,ou=Computer- und Medienservice,o=Humboldt-Universitaet zu Berlin,c=de'),
            'username'         => 'uid='.config('laudatio.external_admin_username').','.config('external_base_dn','ou=Laudatio,ou=Services,ou=Computer- und Medienservice,o=Humboldt-Universitaet zu Berlin,c=de'),
            'password'         => config('laudatio.external_admin_password'),
            'schema'           => config('ldap.connections.default.settings.schema'),
            // Optional Configuration Options
            'account_prefix'   => '',
            'account_suffix'   => '',
            'port'             => 389,
            'follow_referrals' => false,
            'use_ssl'          => false,
            'use_tls'          => true,
            'version'          => 3,
            'timeout'          => 10,
        ];

        $this->adldap = new Adldap();
        $this->adldap->addProvider($this->config,'master-config');

    }


    public function connectADLDAP() {
        try {
            $this->provider = $this->adldap->connect('master-config');
            // Great, we're connected!
        } catch (Adldap\Auth\BindException $e) {
            // Failed to connect.
        }
    }

    public function searchTree($searchData,$fields){
        $search = $this->provider->search();
        $search->select($fields);
        $results = $search->where($searchData)->get();
        return $results;
    }

    public function createUser($userData){
        $result = array();

        $factory = $this->provider->make();

        $user = $factory->user($userData);

        $ldaprecord['objectclass'][0] = "top";
        $ldaprecord['objectclass'][1] = "person";
        $ldaprecord['objectclass'][2] = "organizationalPerson";
        $ldaprecord['objectclass'][3] = "inetOrgPerson";

        $user->fill($ldaprecord);
        $user->setDn('uid=' . $userData['uid'] . ',' . $this->config['base_dn']);

        if ($user->save()) {
            $result = array(
                'success' => true,
                'message' => 'The user was successfully created'
            );

        } else {
            $result = array(
                'success' => false,
                'message' => 'There was an error creating the user'
            );
        }
        return $result;
    }
}