<?php

namespace App;

use Illuminate\Notifications\Notifiable;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Spatie\Permission\Traits\HasRoles;
use Adldap\Laravel\Traits\HasLdapUser;
use Illuminate\Database\Eloquent\SoftDeletes;
use App\Role;
use Log;
/**
 * Class User
 * @package App
 */

class User extends Authenticatable
{
    use Notifiable;
    use HasRoles;
    use HasLdapUser;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name', 'email','password','affiliation', 'username', 'gitlab-use-agree', 'terms-of-use-agree', 'authenticated','objectguid'
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token',
        'deleted_at',
        'created_at',
        'updated_at'
    ];

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function roles() {
        return $this->belongsToMany(Role::class);
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function invites() {
        return $this->hasMany(Invite::class);
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function corpus_projects() {
        return $this->belongsToMany(CorpusProject::class)->withPivot('role_id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsToMany
     */
    public function corpora() {
        return $this->belongsToMany(Corpus::class)->withPivot('role_id');
    }

    public function isSuperUser()
    {
        foreach ($this->roles()->get() as $role)
        {
            if ($role->id == 4)
            {
                return true;
            }
        }

        return false;
    }

    public function isCorpusAdmin()
    {
        foreach ($this->roles()->get() as $role)
        {
            if ($role->id == 2 || $role->id == 3)
            {
                return true;
            }
        }

        return false;
    }

}
