<?php
namespace Deployer;

require 'recipe/laravel.php';

// Configuration

set('repository', 'git@scm.cms.hu-berlin.de:hu-berlin-laudatio-repository/laudatio-repository.git');
set('git_tty', true); // [Optional] Allocate tty for git on first deployment
add('shared_files', ['.env','config/flysystem.php','config/filesystems.php','config/gitlab.php','config/laudatio.php']);
add('shared_dirs', ['storage']);
add('writable_dirs', ['releases','storage','vendor','public']);
set('ssh_type', 'native');
set('ssh_multiplexing', true);
set('http_user', 'www-data');
set('permission_method', 'chmod_777');
set('writable_use_sudo', true);
set('writable_dirs', [
    'bootstrap/cache',
    'storage',
    'storage/app',
    'storage/app/public',
    'storage/framework',
    'storage/framework/cache',
    'storage/framework/sessions',
    'storage/framework/views',
    'storage/framework/views/gitLab',
    'storage/logs',
    'public'
]);


/*

*/
/*
host('laudatio1.cms.hu-berlin.de')
    ->user('root')
    ->identityFile('/Users/rolfguescini/.ssh/deploy', '/Users/rolfguescini/.ssh/deploy', '')
    ->set('deploy_path', '/var/www/html/laudatio1-beta.cms.hu-berlin.de')
    ->set('branch', 'develop')
    ->user('laudatiouser');

host('depot1-6.cms.hu-berlin.de')
    ->user('root')
    ->identityFile('/Users/rolfguescini/.ssh/deploy', '/Users/rolfguescini/.ssh/deploy', '')
    ->set('deploy_path', '/var/www/html/laravelaudatio')
    ->set('branch', 'Anzeige-Design')
    ->user('rolfguescini');



host('depot1-5.cms.hu-berlin.de')
    ->identityFile('/Users/rolfguescini/.ssh/deploy', '/Users/rolfguescini/.ssh/deploy', '')
    ->set('deploy_path', '/var/www/html/laravelaudatio')
    ->set('branch', 'develop')
    ->user('rolfguescini');
*/



host('laudatio1.cms.hu-berlin.de')
    ->identityFile('/Users/rolfguescini/.ssh/deploy', '/Users/rolfguescini/.ssh/deploy', '')
    ->set('deploy_path', '/var/www/html/laudatio1.cms.hu-berlin.de')
    ->set('branch', 'master')
    ->user('laudatiouser');

// Tasks
task('15environment', function () {
    run('cp /var/www/html/laravelaudatio/flysystem.php {{release_path}}/config/flysystem.php');
    run('cp /var/www/html/laravelaudatio/filesystems.php {{release_path}}/config/filesystems.php');
    run('cp /var/www/html/laravelaudatio/gitlab.php {{release_path}}/config/gitlab.php');
    run('cp /var/www/html/laravelaudatio/laudatio.php {{release_path}}/config/laudatio.php');
    run('cp /var/www/html/laravelaudatio/deployenv {{release_path}}/.env');
    run('mkdir {{release_path}}/public/images/corpuslogos');
    //run('cp /var/www/html/laravelaudatio/Groups.php {{release_path}}/vendor/m4tthumphrey/php-gitlab-api/lib/Gitlab/Api/Groups.php');
})->desc('Environment setup');

task('16environment', function () {
    run('cp /var/www/html/laravelaudatio/flysystem.php {{release_path}}/config/flysystem.php');
    run('cp /var/www/html/laravelaudatio/filesystems.php {{release_path}}/config/filesystems.php');
    run('cp /var/www/html/laravelaudatio/gitlab.php {{release_path}}/config/gitlab.php');
    run('cp /var/www/html/laravelaudatio/laudatio2.php {{release_path}}/config/laudatio.php');
    run('cp /var/www/html/laravelaudatio/deployenv {{release_path}}/.env');
    run('mkdir {{release_path}}/public/images/corpuslogos');
    //run('cp /var/www/html/laravelaudatio/Groups.php {{release_path}}/vendor/m4tthumphrey/php-gitlab-api/lib/Gitlab/Api/Groups.php');
})->desc('Environment setup');

task('stage_environment', function () {
    run('cp /var/www/html/laudatio1.cms.hu-berlin.de/flysystem.php {{release_path}}/config/flysystem.php');
    run('cp /var/www/html/laudatio1.cms.hu-berlin.de/filesystems.php {{release_path}}/config/filesystems.php');
    run('cp /var/www/html/laudatio1.cms.hu-berlin.de/gitlab.php {{release_path}}/config/gitlab.php');
    run('cp /var/www/html/laudatio1.cms.hu-berlin.de/laudatio.php {{release_path}}/config/laudatio.php');
    run('cp /var/www/html/laudatio1.cms.hu-berlin.de/deployenv {{release_path}}/.env');
    run('mkdir {{release_path}}/public/images/corpuslogos');
    //run('cp /var/www/html/laravelaudatio/Groups.php {{release_path}}/vendor/m4tthumphrey/php-gitlab-api/lib/Gitlab/Api/Groups.php');
})->desc('Staging Environment setup');

task('stage_beta_environment', function () {
    run('cp /var/www/html/laudatio1-beta.cms.hu-berlin.de/flysystem.php {{release_path}}/config/flysystem.php');
    run('cp /var/www/html/laudatio1-beta.cms.hu-berlin.de/filesystems.php {{release_path}}/config/filesystems.php');
    run('cp /var/www/html/laudatio1-beta.cms.hu-berlin.de/gitlab.php {{release_path}}/config/gitlab.php');
    run('cp /var/www/html/laudatio1-beta.cms.hu-berlin.de/laudatio.php {{release_path}}/config/laudatio.php');
    run('cp /var/www/html/laudatio1-beta.cms.hu-berlin.de/deployenv {{release_path}}/.env');
    run('mkdir {{release_path}}/public/images/corpuslogos');
    //run('cp /var/www/html/laravelaudatio/Groups.php {{release_path}}/vendor/m4tthumphrey/php-gitlab-api/lib/Gitlab/Api/Groups.php');
})->desc('Staging BETA Environment setup');

/*
task('makespace', function () {
    run('sudo -su www-data');
    run('cd /var/www/html/laravelaudatio/releases');
    run('rm -rf *');
    run('exit');
})->desc('Environment setup');

before('deploy','makespace');
*/



desc('Execute artisan config:clear');
task('clearconfig', function () {
    run('{{bin/php}} {{release_path}}/artisan config:clear');
    run('{{bin/php}} {{release_path}}/artisan cache:clear');
});

//after('deploy','15environment');
//after('deploy','16environment');
after('deploy','stage_environment');
//after('deploy','stage_beta_environment');
after('deploy','clearconfig');
after('deploy:failed', 'deploy:unlock');