import Vue from 'vue'
import Vuex from 'vuex'
Vue.use(Vuex)

import ApiService from '../services/ApiService.js';
import Functions from '../functions.js';
let initialState = {
    "token": null,
    "user": {}
}

export default new Vuex.Store({
    state: {
        results: [],
        corpusresults: [],
        corpusformats: [],
        corpushighlights: {},
        corpusdescriptioncollapsed: [],
        documentresults: [],
        documenthighlights: {},
        annotationresults: [],
        annotationformats: [],
        annotationhighlights: {},
        corpusresultcounter: 0,
        corpustotalresultcounter: 0,
        currentcorpusoffset: 0,
        documentresultcounter: 0,
        currentdocumentoffset: 0,
        documenttotalresultcounter: 0,
        annotationresultcounter: 0,
        annotationtotalresultcounter: 0,
        currentannotationoffset: 0,
        currentMainSearchOffset: 0,
        corporaloading: false,
        corporaarefiltered: false,
        documentsloading: false,
        documentsarefiltered: false,
        annotationsloading: false,
        annotationsarefiltered: false,
        showCorpusResult: true,
        showDocumentResult: false,
        showAnnotationResult: false,
        dataloading: false,
        currentCorpusPage: 1,
        currentDocumentPage: 1,
        currentAnnotationPage: 1,
        corpusResultPerPage: 5,
        corpusPagerPerPage: 5,
        documentResultsPerPage: 5,
        documentPagerPerPage: 5,
        annotationResultsPerPage: 5,
        annotationPagerPerPage: 5,
        elasticsearchdocumentlimit: window.laudatioApp.elasticsearchLimit,
        visibleCorpora: [],
        visibleDocuments: [],
        visibleAnnotations: [],
        mainSearchTerm: '',
        mainSearchTermHit: false,
        frontpagequery: '',
        frontPageTermHit: false,
        dismissSecs: 3,
        dismissCountDown: 0,
        searchsuggestions: [],
        activefilters: [],
        activefilterhits: {},
        activefiltersmap: {},
        filteredcorpushighlights: {},
        filteredcorpushighlightmap: {},
        filtereddocumenthighlights: {},
        filtereddocumenthighlightmap: {},
        filteredannotationhighlights: {},
        filteredannotationhighlightmap: {},
        annotationFormatFilterMarkup : {},
        corpusFormatFilterMarkup : {},
        corpusFilterData : {
            corpus_title: [],
            corpus_publication_publisher: [],
            corpus_publication_publication_date: '',
            corpusYearTo: '',
            corpus_size_value: '',
            corpusSizeTo: '',
            corpus_merged_languages: [],
            corpus_merged_formats: [],
            corpus_publication_license: []
        },
        documentFilterData : {
            document_title: [],
            document_author_forename: '',
            document_author_surname: '',
            document_merged_authors: [],
            document_publication_place: [],
            document_publication_publishing_date: '',
            document_publication_publishing_date_to: '',
            document_size_extent: '',
            document_size_extent_to: '',
            document_merged_languages: []
        },
        annotationFilterData : {
            preparation_title: [],
            annotation_merged_formats: [],
            preparation_encoding_full_name: [],
            preparation_encoding_annotation_group: [],
        },
        corpusuislideroptions: {
            dotSize: 14,
            width: 'auto',
            height: 4,
            contained: false,
            direction: 'ltr',
            data: null,
            min: 0,
            max: 1000000,
            interval: 100,
            disabled: false,
            clickable: true,
            duration: 0.5,
            adsorb: false,
            lazy: true,
            tooltip: 'active',
            tooltipPlacement: 'top',
            tooltipFormatter: void 0,
            useKeyboard: false,
            keydownHook: null,
            dragOnClick: false,
            enableCross: false,
            fixed: false,
            minRange: void 0,
            maxRange: void 0,
            order: true,
            marks: false,
            dotOptions: void 0,
            process: true,
            dotStyle: void 0,
            railStyle: void 0,
            processStyle: void 0,
            tooltipStyle: void 0,
            stepStyle: void 0,
            stepActiveStyle: void 0,
            labelStyle: void 0,
            labelActiveStyle: void 0
        },
        documentuislideroptions: {
            dotSize: 14,
            width: 'auto',
            height: 4,
            contained: false,
            direction: 'ltr',
            data: null,
            min: 0,
            max: 10000,
            interval: 100,
            disabled: false,
            clickable: true,
            duration: 0.5,
            adsorb: false,
            lazy: true,
            tooltip: 'active',
            tooltipPlacement: 'top',
            tooltipFormatter: void 0,
            useKeyboard: false,
            keydownHook: null,
            dragOnClick: false,
            enableCross: false,
            fixed: false,
            minRange: void 0,
            maxRange: void 0,
            order: true,
            marks: false,
            dotOptions: void 0,
            process: true,
            dotStyle: void 0,
            railStyle: void 0,
            processStyle: void 0,
            tooltipStyle: void 0,
            stepStyle: void 0,
            stepActiveStyle: void 0,
            labelStyle: void 0,
            labelActiveStyle: void 0
        },
        baseDomain: window.laudatioApp.baseDomain,
        elasticApiVersion: window.laudatioApp.elasticApiVersion,
        corpusdata: window.laudatioApp.corpusdata,
        totalCount: window.laudatioApp.totalCount,
        perPageArray:window.laudatioApp.perPageArray,
        perPage: window.laudatioApp.perPage,
        ccbaseuri: window.laudatioApp.ccBaseUri
    },
    actions: {
        commitCorpora ({commit}, corpora) {
            commit('PUSH_CORPUS_RESULTS', corpora)
        },
        suggestSearch( { commit, state }, payload) {
            ApiService.searchSuggest(payload).
              then( response => {
                  commit('SET_MAINSEARCH_SUGGESTIONS', response.data.data)
              }).catch( error => {
                  console.log('There was an error fetching the autocomplete suggestions:', error.response)
              })
        },
        fetchLatestCorpora ({ commit, state }) {
            ApiService.getLatestCorpora(state.currentcorpusoffset,state.elasticsearchdocumentlimit)
                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        commit('SET_CORPORAAREFILTERED_STATE',false)
                        commit("EMPTY_CORPUSDESCRIPTION_COLLAPSED");
                        commit('PUSH_CORPUS_RESULTS', response.data.data)
                        commit('PUSH_CORPUS_FORMATS', response.data.data);
                    }
                    commit('UNSET_CORPORA_LOADING');
                    commit('CALCULATE_TABS');

                })
                .catch(error => {
                    console.log('There was an error fetching the Corpora:', error.response)
                });
        },
        fetchLatestCorporaCount ({ commit, state }) {
            ApiService.getLatestCorporaCount()
                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        commit("SET_CORPORA_TOTALCOUNT",JSON.parse(response.data.data))
                    }
                    commit('UNSET_CORPORA_LOADING');
                    //commit('CALCULATE_TABS');
                })
                .catch(error => {
                    console.log('There was an error fetching the Corpora total count:', error.response)
                });
        },
        commitDocuments ({commit}, documents) {
            commit('PUSH_DOCUMENT_RESULTS', documents)
        },
        fetchLatestDocuments ({ commit, state }) {
            ApiService.getLatestDocuments(state.currentcorpusoffset,state.elasticsearchdocumentlimit)
                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        commit('SET_DOCUMENTSAREFILTERED_STATE',false)
                        commit("PUSH_DOCUMENT_RESULTS",response.data.data)
                    }
                    commit('UNSET_DOCUMENTS_LOADING');
                    commit('CALCULATE_TABS');
                })
                .catch(error => {
                    console.log('There was an error fetching the Documents:', error)
                });
        },
        fetchLatestDocumentCount ({ commit, state }) {
            ApiService.getLatestDocumentsCount()
                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        commit("SET_DOCUMENTS_TOTALCOUNT",JSON.parse(response.data.data))
                    }
                    commit('UNSET_DOCUMENTS_LOADING');
                    //commit('CALCULATE_TABS');
                })
                .catch(error => {
                    console.log('There was an error fetching the Document total count:', error.response)
                });
        },
        commitAnnotations ({commit}, annotations) {
            commit('PUSH_ANNOTATION_RESULTS', annotations)
        },
        fetchLatestAnnotations ({ commit, state }) {
            ApiService.getLatestAnnotations(state.currentcorpusoffset,state.elasticsearchdocumentlimit)
                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        commit('SET_ANNOTATIONSAREFILTERED_STATE',false)
                        commit("PUSH_ANNOTATION_RESULTS",response.data.data)
                        commit('PUSH_ANNOTATION_FORMATS', response.data.data);
                    }
                    commit('UNSET_ANNOTATIONS_LOADING');
                    commit('CALCULATE_TABS');
                })
                .catch(error => {
                    console.log('There was an error fetching the Annotations:', error.response)
                });
        },
        fetchLatestAnnotationCount ({ commit, state }) {
            ApiService.getLatestAnnotationsCount()
                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        commit("SET_ANNOTATIONS_TOTALCOUNT",JSON.parse(response.data.data))
                    }
                    commit('UNSET_ANNOTATIONS_LOADING');
                    //commit('CALCULATE_TABS');
                })
                .catch(error => {
                    console.log('There was an error fetching the Annotations total count:', error.response)
                });
        },
        commitUpdateCorpora ({ commit, state },offset) {
            ApiService.getLatestCorpora(offset,state.elasticsearchdocumentlimit)
                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        commit('PUSH_CORPUS_RESULTS', response.data.data)
                    }
                    commit('UNSET_CORPORA_LOADING');
                    commit('SET_CURRENT_CORPUS_OFFSET',offset)
                })
                .catch(error => {
                    console.log('There was an error fetching the Corpora:', error.response)
                });
        },
        commitUpdateDocuments({ commit, state },offset) {
            ApiService.getLatestDocuments(offset,state.elasticsearchdocumentlimit)
                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        commit("PUSH_DOCUMENT_RESULTS",response.data.data)
                    }
                    commit('UNSET_DOCUMENTS_LOADING');
                    commit('SET_CURRENT_DOCUMENT_OFFSET',offset)
                })
                .catch(error => {
                    console.log('There was an error fetching the Documents:', error.response)
                });
        },
        commitUpdateAnnotations({ commit, state },offset) {
            ApiService.getLatestAnnotations(offset,state.elasticsearchdocumentlimit)
                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        commit("PUSH_ANNOTATION_RESULTS",response.data.data)
                    }
                    commit('UNSET_ANNOTATIONS_LOADING');
                    commit('SET_CURRENT_ANNOTATION_OFFSET',offset)
                })
                .catch(error => {
                    console.log('There was an error fetching the Annotations:', error.response)
                });
        },
        mainCorpusSearch({ commit, state },payload) {
            ApiService.mainCorpusSearch(payload)
                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        if(response.data.data.length > 0) {
                            commit('SET_CORPORAAREFILTERED_STATE',false)
                            commit("EMPTY_CORPUSDESCRIPTION_COLLAPSED");
                            commit('PUSH_CORPUS_RESULTS', response.data.data)
                            commit('PUSH_CORPUS_FORMATS', response.data.data);
                            //commit("SET_CORPORA_MAINSEARCH_TOTALCOUNT",response.data.count)
                            commit('SET_CURRENT_CORPUS_HIGHLIGHTS')
                            if(state.frontpagequery != '') {
                                commit('SET_FRONT_PAGE_TERM_HIT',true);
                            }
                            else{
                                commit('SET_MAIN_SEARCH_TERM_HIT',true);
                            }
                        }
                        else{
                            commit('SET_DISMISS_COUNTDOWN');
                        }
                        commit('UNSET_CORPORA_LOADING');
                        commit('SET_CURRENT_CORPUS_OFFSET',payload.searchData.from)
                        commit('CALCULATE_TABS');
                    }
                })
                .catch(error => {
                    console.log('There was an error fetching the corpus search results:', error)
                });
        },
        getLatestCorporaMainSearchCount({ commit, state },payload) {
            ApiService.getLatestCorporaMainSearchCount(payload)

                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        if(typeof response.data.data != "undefined" && response.data.data != "") {
                            commit("SET_CORPORA_MAINSEARCH_TOTALCOUNT",response.data.data)
                        }

                        commit('UNSET_CORPORA_LOADING');
                        commit('SET_CURRENT_CORPUS_OFFSET',payload.searchData.from)
                        commit('CALCULATE_TABS');

                    }
                })
                .catch(error => {
                    console.log('There was an error fetching the corpus count search results:', error)
                });
        },
        mainCorpusFilterSearch ({commit, state},payload) {
            payload.searchData.filterData = state.corpusFilterData;

            if(state.mainSearchTerm) {
                payload.searchData.mainSearchTerm = state.mainSearchTerm;
            }
            else if(state.frontpagequery) {
                payload.searchData.frontpagequery = state.frontpagequery;
            }

            ApiService.mainCorpusFilterSearch(payload)
                . then(response => {
                    if(response.data.success && response.data.status == 200) {

                        commit('SET_CORPORAAREFILTERED_STATE',false)
                        commit("EMPTY_CORPUSDESCRIPTION_COLLAPSED");
                        commit("UPDATE_CORPUS_RESULT_VISIBILITY",response.data.data)
                        commit('PUSH_CORPUS_FORMATS', response.data.data);
                        commit('UNSET_CORPORA_LOADING');
                        commit('SET_CURRENT_CORPUS_OFFSET',payload.searchData.from);
                        commit('SET_CURRENT_CORPUS_FILTER');
                        commit('SET_CORPORAAREFILTERED_STATE',true)
                        commit('SET_ACTIVE_TAB','corpora');
                    }
                })
                .catch(error => {
                    console.log('There was an error fetching the Corpus Filter Data:', error)
                });
        },
        mainDocumentFilterSearch ({commit, state}, payload) {
            payload.searchData.filterData = state.documentFilterData;
            if(state.mainSearchTerm) {
                payload.searchData.mainSearchTerm = state.mainSearchTerm;
            }
            else if(state.frontpagequery) {
                payload.searchData.frontpagequery = state.frontpagequery;
            }
            ApiService.mainDocumentFilterSearch(payload)
            . then(response => {
                if(response.data.success && response.data.status == 200) {
                    commit('SET_DOCUMENTSAREFILTERED_STATE', false)
                    commit("UPDATE_DOCUMENT_RESULT_VISIBILITY",response.data.data)
                    commit('UNSET_DOCUMENTS_LOADING');
                    commit('SET_CURRENT_DOCUMENT_OFFSET',payload.searchData.from);
                    commit('SET_CURRENT_DOCUMENT_FILTER');
                    commit('SET_DOCUMENTSAREFILTERED_STATE', true)
                }
                commit('SET_ACTIVE_TAB','documents');
            })
            .catch(error => {
                console.log('There was an error fetching the Document Filter Data:', error)
            });
        },
        mainDocumentFilterSearchCount({ commit, state },payload) {
            payload.searchData.filterData = state.documentFilterData;
            if(state.mainSearchTerm) {
                payload.searchData.mainSearchTerm = state.mainSearchTerm;
            }
            else if(state.frontpagequery) {
                payload.searchData.frontpagequery = state.frontpagequery;
            }

            ApiService.mainDocumentFilterSearchCount(payload)
                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        if(typeof response.data.data != "undefined" && response.data.data != "") {
                            commit("SET_DOCUMENT_MAINSEARCH_TOTALCOUNT",response.data.data)
                        }
                        commit('UNSET_DOCUMENTS_LOADING');
                        commit('SET_CURRENT_DOCUMENT_OFFSET',payload.searchData.from)
                        commit('CALCULATE_TABS');

                    }
                })
                .catch(error => {
                    console.log('There was an error fetching the document count search results:', error)
                });
        },
        mainAnnotationFilterSearchCount({ commit, state },payload) {
            payload.searchData.filterData = state.annotationFilterData

            if(state.mainSearchTerm) {
                payload.searchData.mainSearchTerm = state.mainSearchTerm;
            }
            else if(state.frontpagequery) {
                payload.searchData.frontpagequery = state.frontpagequery;
            }

            ApiService.mainAnnotationFilterSearchCount(payload)
                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        if(typeof response.data.data != "undefined" && response.data.data != "") {
                            commit("SET_ANNOTATION_MAINSEARCH_TOTALCOUNT",response.data.data)
                        }
                        commit('UNSET_ANNOTATIONS_LOADING');
                        commit('SET_CURRENT_ANNOTATION_OFFSET',payload.searchData.from)
                        commit('CALCULATE_TABS');

                    }
                })
                .catch(error => {
                    console.log('There was an error fetching the document count search results:', error)
                });
        },
        mainCorpusFilterSearchCount({ commit, state },payload) {
            payload.searchData.filterData = state.corpusFilterData;

            if(state.mainSearchTerm) {
                payload.searchData.mainSearchTerm = state.mainSearchTerm;
            }
            else if(state.frontpagequery) {
                payload.searchData.frontpagequery = state.frontpagequery;
            }

            ApiService.mainCorpusFilterSearchCount(payload)
                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        if(typeof response.data.data != "undefined" && response.data.data != "") {
                            commit("SET_CORPORA_MAINSEARCH_TOTALCOUNT",response.data.data)
                        }
                        commit('UNSET_CORPORA_LOADING');
                        commit('SET_CURRENT_CORPUS_OFFSET',payload.searchData.from)
                        commit('CALCULATE_TABS');

                    }
                })
                .catch(error => {
                    console.log('There was an error fetching the document count search results:', error)
                });
        },
        mainAnnotationFilterSearch ({commit, state}, payload) {
            payload.searchData.filterData = state.annotationFilterData

            if(state.mainSearchTerm) {
                payload.searchData.mainSearchTerm = state.mainSearchTerm;
            }
            else if(state.frontpagequery) {
                payload.searchData.frontpagequery = state.frontpagequery;
            }

            ApiService.mainAnnotationFilterSearch(payload)
                . then(response => {
                    if(response.data.success && response.data.status == 200) {
                        commit('SET_ANNOTATIONSAREFILTERED_STATE', false)
                        commit("UPDATE_ANNOTATION_RESULT_VISIBILITY",response.data.data)
                        commit('UNSET_ANNOTATIONS_LOADING');
                        commit('SET_CURRENT_ANNOTATION_OFFSET',payload.searchData.from);
                        commit('SET_CURRENT_ANNOTATION_FILTER');
                        commit('SET_ANNOTATIONSAREFILTERED_STATE', true)
                        commit('SET_ACTIVE_TAB','annotations');
                    }
                })
                .catch(error => {
                    console.log('There was an error fetching the Annotation Filter Data:', error)
                });
        },
        mainDocumentSearch({ commit, state },payload) {
            ApiService.mainDocumentSearch(payload)

                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        if(response.data.data.length > 0) {
                            commit('SET_DOCUMENTSAREFILTERED_STATE', false)
                            commit("PUSH_DOCUMENT_RESULTS",response.data.data)
                            commit("SET_CURRENT_DOCUMENT_HIGHLIGHTS")
                            //  commit("SET_DOCUMENT_MAINSEARCH_TOTALCOUNT",response.data.count)
                            if(state.frontpagequery != '') {
                                commit('SET_FRONT_PAGE_TERM_HIT',true);
                            }
                            else{
                                commit('SET_MAIN_SEARCH_TERM_HIT',true);
                            }
                        }
                        else{
                            commit('SET_DISMISS_COUNTDOWN');
                        }
                        commit('UNSET_DOCUMENTS_LOADING');
                        commit('SET_CURRENT_DOCUMENT_OFFSET',payload.searchData.from)
                        commit('CALCULATE_TABS');

                    }
                })
                .catch(error => {
                    console.log('There was an error fetching the document search results:', error)
                });
        },
        getLatestDocumentMainSearchCount({ commit, state },payload) {
            ApiService.getLatestDocumentMainSearchCount(payload)

                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        if(typeof response.data.data != "undefined" && response.data.data != "") {
                            commit("SET_DOCUMENT_MAINSEARCH_TOTALCOUNT",response.data.data)
                        }

                        commit('UNSET_DOCUMENTS_LOADING');
                        commit('SET_CURRENT_DOCUMENT_OFFSET',payload.searchData.from)
                        commit('CALCULATE_TABS');

                    }
                })
                .catch(error => {
                    console.log('There was an error fetching the document count search results:', error)
                });
        },
        mainAnnotationSearch({ commit, state },payload) {
            ApiService.mainAnnotationSearch(payload)
                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        if(response.data.data.length > 0) {
                            commit('SET_ANNOTATIONSAREFILTERED_STATE', false)
                            commit("PUSH_ANNOTATION_RESULTS",response.data.data)
                            commit('PUSH_ANNOTATION_FORMATS', response.data.data);
                            commit("SET_CURRENT_ANNOTATION_HIGHLIGHTS")
                            //commit("SET_ANNOTATION_MAINSEARCH_TOTALCOUNT",response.data.count)
                            if(state.frontpagequery != '') {
                                commit('SET_FRONT_PAGE_TERM_HIT',true);
                            }
                            else{
                                commit('SET_MAIN_SEARCH_TERM_HIT',true);
                            }
                        }
                        else{
                            commit('SET_DISMISS_COUNTDOWN');
                        }
                        commit('UNSET_ANNOTATIONS_LOADING');
                        commit('SET_CURRENT_ANNOTATION_OFFSET',payload.searchData.from)
                        commit('CALCULATE_TABS');
                    }
                })
                .catch(error => {
                    console.log('There was an error fetching the annotation search results:', error)
                });
        },
        getLatestAnnotationMainSearchCount({ commit, state },payload) {
            ApiService.getLatestAnnotationMainSearchCount(payload)

                .then(response => {
                    if(response.data.success && response.data.status == 200) {
                        if(typeof response.data.data != "undefined" && response.data.data != "") {
                            commit("SET_ANNOTATION_MAINSEARCH_TOTALCOUNT",response.data.data)
                        }

                        commit('UNSET_ANNOTATIONS_LOADING');
                        commit('SET_CURRENT_ANNOTATION_OFFSET',payload.searchData.from)
                        commit('CALCULATE_TABS');

                    }
                })
                .catch(error => {
                    console.log('There was an error fetching the annotation count search results:', error)
                });
        }
    },
    getters: {
        dismissSecs: state => {
            return state.dismissSecs
        },
        dismissCountDown: state => {
            return state.dismissCountDown
        },
        searchSuggestions: state => {
          return state.searchsuggestions
        },
        visibleCorpora: state => {
            return state.visibleCorpora;
        },
        visibleDocuments: state => {
            return state.visibleDocuments;
        },
        visibleAnnotations: state => {
            return state.visibleAnnotations;
        },
        currentCorpusPage: state => {
            return state.currentCorpusPage;
        },
        currentDocumentPage: state => {
            return state.currentDocumentPage;
        },
        currentAnnotationPage: state => {
            return state.currentAnnotationPage;
        },
        corpusPagerPerPage: state => {
            return state.corpusPagerPerPage;
        },
        documentPagerPerPage: state => {
            return state.documentPagerPerPage;
        },
        annotationPagerPerPage: state => {
            return state.annotationPagerPerPage;
        },
        corpushighlights: state => {
            return state.corpushighlights;
        },
        corporaarefiltered: state => {
          return state.corporaarefiltered;
        },
        documentsarefiltered: state => {
            return state.documentsarefiltered;
        },
        annotationsarefiltered: state => {
            return state.annotationsarefiltered;
        },
        documenthighlights: state => {
            return state.documenthighlights;
        },
        annotationhighlights: state => {
            return state.annotationhighlights;
        },
        filteredcorpushighlightmap: state => {
            return state.filteredcorpushighlightmap;
        },
        filtereddocumenthighlights: state => {
            return state.filtereddocumenthighlights;
        },
        filtereddocumenthighlightmap: state => {
            return state.filtereddocumenthighlightmap;
        },
        filteredannotationhighlights: state => {
            return state.filteredannotationhighlights;
        },
        filteredannotationhighlightmap: state => {
            return state.filteredannotationhighlightmap;
        },
        corpusresultcounter: state => {
            return state.corpusresultcounter;
        },
        corpusdescriptioncollapsed: state => {
            return state.corpusdescriptioncollapsed;
        },
        documentresultcounter: state => {
            return state.documentresultcounter;
        },
        annotationresultcounter: state => {
            return state.annotationresultcounter;
        },
        corpustotalresultcounter: state => {
            return state.corpustotalresultcounter;
        },
        documenttotalresultcounter: state => {
            return state.documenttotalresultcounter;
        },
        annotationtotalresultcounter: state => {
            return state.annotationtotalresultcounter;
        },
        showCorpusResult: state => {
            return state.showCorpusResult;
        },
        showDocumentResult: state => {
            return state.showDocumentResult;
        },
        showAnnotationResult: state => {
            return state.showAnnotationResult;
        },
        currentCorpusOffset: state => {
            return state.currentcorpusoffset;
        },
        currentDocumentOffset: state => {
            return state.currentdocumentoffset;
        },
        currentAnnotationOffset: state => {
            return state.currentannotationoffset;
        },
        elasticsearchDocumentLimit: state => {
            return state.elasticsearchdocumentlimit;
        },
        currentMainSearchOffset: state => {
            return state.currentMainSearchOffset;
        },
        mainSearchTerm: state => {
            return state.mainSearchTerm;
        },
        frontpagequery: state => {
            return state.frontpagequery;
        },
        mainSearchTermHit: state => {
            return state.mainSearchTermHit;
        },
        frontPageTermHit: state => {
            return state.frontPageTermHit;
        },
        activefilters: state => {
            return state.activefilters;
        },
        activefilterhits: state => {
            return state.activefilterhits;
        },
        activefiltersmap: state => {
            return state.activefiltersmap;
        },
        corpusformats: state => {
            return  Functions.uniqueArray(state.corpusformats);
        },
        annotationformats: state => {
            return Functions.uniqueArray(state.annotationformats);
        },
        corpusFilterData: state => {
            return state.corpusFilterData;
        },
        documentFilterData: state => {
            return state.documentFilterData;
        },
        annotationFilterData: state => {
            return state.annotationFilterData;
        },
        dataloading : state => {
            return state.dataloading
        },
        baseDomain : state => {
            return state.baseDomain
        },
        elasticApiVersion:  state => {
            return state.elasticApiVersion
        },
        corpusdata: state => {
            return state.corpusdata
        },
        totalCount: state => {
            return state.totalCount
        },
        perPageArray: state => {
            return state.perPageArray
        },
        perPage: state => {
            return state.perPage
        },
        ccbaseuri: state => {
            return state.ccbaseuri
        },
        uislideroptions: state => {
            return state.uislideroptions
        }
    },
    mutations: {
        CALCULATE_TABS (state) {
            if((state.corpusresultcounter ==  0 && state.documentresultcounter == 0 && state.annotationresultcounter == 0) || (state.corpusresultcounter >  0 && state.documentresultcounter == 0 && state.annotationresultcounter == 0)) {
                state.showCorpusResult = true;
                state.showDocumentResult = false;
                state.showAnnotationResult = false;
            }
            else if(state.corpusresultcounter ==  0 && state.documentresultcounter > 0 && state.annotationresultcounter == 0) {
                state.showCorpusResult = false;
                state.showDocumentResult = true;
                state.showAnnotationResulte = false;
            }
            else if(state.corpusresultcounter ==  0 && state.documentresultcounter == 0 && state.annotationresultcounter > 0) {
                state.showCorpusResult = false;
                state.showDocumentResult = false;
                state.showAnnotationResult = true;
            }
            else if(state.corpusresultcounter >  0 && state.documentresultcounter > 0 && state.annotationresultcounter > 0) {
                state.showCorpusResult = true;
                state.showDocumentResult = false;
                state.showAnnotationResult = false;
            }
        },
        SET_ACTIVE_TAB (state,header) {
            switch (header) {
                case "corpora":
                    state.showCorpusResult = true;
                    state.showDocumentResult = false;
                    state.showAnnotationResult = false;
                    break;
                case "documents":
                    state.showCorpusResult = false;
                    state.showDocumentResult = true;
                    state.showAnnotationResult = false;
                    break;
                case "annotations":
                    state.showCorpusResult = false;
                    state.showDocumentResult = false;
                    state.showAnnotationResult = true;
                    break;
            }
        },
        SET_MAINSEARCH_SUGGESTIONS (state, data) {
            state.searchsuggestions = data.items
        },
        PUSH_CORPUS_RESULTS (state, corpusresults) {
            state.corpusresults = corpusresults;
            for(let i = 0; i < state.corpusresults.length; i++) {
                if(typeof state.corpusresults[i]._source != 'undefined') {
                    if(state.corpusresults[i]._source.visibility == 1) {
                        state.visibleCorpora.push(state.corpusresults[i])
                        state.corpusdescriptioncollapsed.push(false)
                        state.corpusresultcounter++;
                    }
                }
            }
        },
        PUSH_CORPUS_FORMATS (state, data) {
            let foundFormats = []
            if(state.corpusformats.length == 0) {
                for(let i = 0; i < data.length; i++) {
                    if(typeof data[i]._source.corpus_merged_formats != 'undefined') {


                        for(let j = 0; j < data[i]._source.corpus_merged_formats.length; j++) {
                            if( !foundFormats.includes(data[i]._source.corpus_merged_formats[j])) {
                                state.corpusformats.push(data[i]._source.corpus_merged_formats[j]);
                            }
                        }
                    }
                }
            }
        },
        PUSH_ANNOTATION_FORMATS (state, annotationresults) {
            state.annotationresults = annotationresults;
            let foundFormats = []
            for(let i = 0; i < state.annotationresults.length; i++) {
                if(typeof state.annotationresults[i]._source.annotation_merged_formats != 'undefined') {
                    for(let j = 0; j < state.annotationresults[i]._source.annotation_merged_formats.length; j++) {
                        if( !foundFormats.includes(state.annotationresults[i]._source.annotation_merged_formats[j])) {
                            state.annotationformats.push(state.annotationresults[i]._source.annotation_merged_formats[j]);
                        }
                    }
                }
            }
        },
        PUSH_DOCUMENT_RESULTS (state, documentresults) {
            state.documentresults = documentresults
            for(let j = 0; j < state.documentresults.length; j++) {
                if(typeof state.documentresults[j]._source != 'undefined') {
                    if(state.documentresults[j]._source.visibility == 1) {
                        state.visibleDocuments.push(state.documentresults[j])
                        state.documentresultcounter++;
                    }
                }
            }
        },
        PUSH_ANNOTATION_RESULTS (state, annotationresults) {
            state.annotationresults = annotationresults
            for(let k = 0; k < state.annotationresults.length; k++) {
                if(typeof state.annotationresults[k]._source != 'undefined') {
                    if(state.annotationresults[k]._source.visibility == 1) {
                        state.visibleAnnotations.push(state.annotationresults[k])
                        state.annotationresultcounter++;
                    }
                }
            }
        },
        SET_DISMISS_COUNTDOWN (state) {
            state.dismissCountDown = state.dismissSecs
        },
        EMPTY_CORPUS_RESULTS (state) {
            state.corpusresults = [];
        },
        EMPTY_DOCUMENT_RESULTS (state) {
            state.documentresults = [];
        },
        EMPTY_ANNOTATION_RESULTS (state) {
            state.annotationresults = [];
        },
        EMPTY_VISIBLE_CORPUS_RESULTS (state) {
            state.visibleCorpora = [];
        },
        EMPTY_CORPUSDESCRIPTION_COLLAPSED (state) {
            state.corpusdescriptioncollapsed = []
        },
        EMPTY_VISIBLE_DOCUMENT_RESULTS (state) {
            state.visibleDocuments = [];
        },
        EMPTY_VISIBLE_ANNOTATION_RESULTS (state) {
            state.visibleAnnotations = [];
        },
        SET_CORPORA_LOADING (state) {
            state.corporaloading = true;
            state.dataloading = true;
        },
        SET_DOCUMENTS_LOADING (state) {
            state.documentsloading = true;
            state.dataloading = true;
        },
        SET_ANNOTATIONS_LOADING (state) {
            state.annotationsloading = true;
            state.dataloading = true;
        },
        UNSET_CORPORA_LOADING (state) {
            state.corporaloading = false;
            if(!state.documentsloading && !state.annotationsloading) {
                state.dataloading = false;
            }

        },
        UNSET_DOCUMENTS_LOADING (state) {
            state.documentsloading = false;
            if(!state.corporaloading && !state.annotationsloading) {
                state.dataloading = false;
            }
        },
        UNSET_ANNOTATIONS_LOADING (state) {
            state.annotationsloading = false;
            if(!state.documentsloading && !state.corporaloading) {
                state.dataloading = false;
            }
        },
        CORPUS_CURRENT_PAGE_CHANGE: function(state, page) {
            state.currentCorpusPage = page;
        },
        DOCUMENT_CURRENT_PAGE_CHANGE: function(state, page) {
            state.currentDocumentPage = page;
        },
        ANNOTATION_CURRENT_PAGE_CHANGE: function(state, page) {
            state.currentAnnotationPage = page;
        },
        CORPUS_PER_PAGE_CHANGE: function(state, perpage) {
            state.corpusPagerPerPage = perpage;
            state.currentCorpusPage = 1;
        },
        DOCUMENT_PER_PAGE_CHANGE: function(state, perpage) {
            state.documentPagerPerPage = perpage;
            state.currentDocumentPage = 1;
        },
        ANNOTATION_PER_PAGE_CHANGE: function(state, perpage) {
            state.annotationPagerPerPage = perpage;
            state.currentAnnotationPage = 1;
        },
        TOGGLE_CORPUS_RESULT: function(state, show) {
            state.showCorpusResult = show;
        },
        TOGGLE_DOCUMENT_RESULT: function(state, show) {
            state.showDocumentResult = show;
        },
        TOGGLE_ANNOTATION_RESULT: function(state, show) {
            state.showAnnotationResult = show;
        },
        SET_CORPORA_TOTALCOUNT: function(state, countdata) {
            state.corpustotalresultcounter = countdata.count
        },
        SET_CORPORA_MAINSEARCH_TOTALCOUNT: function(state, count) {
            state.corpustotalresultcounter = count
        },
        SET_DOCUMENT_MAINSEARCH_TOTALCOUNT: function(state, count) {
            state.documenttotalresultcounter = count
        },
        SET_ANNOTATION_MAINSEARCH_TOTALCOUNT: function(state, count) {
            state.annotationtotalresultcounter = count
        },
        SET_DOCUMENTS_TOTALCOUNT: function(state, countdata) {
            state.documenttotalresultcounter = countdata.count
        },
        SET_ANNOTATIONS_TOTALCOUNT: function(state, countdata) {
            state.annotationtotalresultcounter = countdata.count
        },
        SET_CORPUS_RESULTS_COUNT: function(state, count) {
            state.corpusresultcounter = count
        },
        SET_DOCUMENT_RESULTS_COUNT: function(state, count) {
            state.documentresultcounter = count
        },
        SET_ANNOTATION_RESULTS_COUNT: function(state, count) {
            state.annotationresultcounter = count
        },
        SET_CURRENT_CORPUS_OFFSET: function(state,offset) {
            state.currentcorpusoffset = offset;
        },
        SET_CURRENT_DOCUMENT_OFFSET: function(state, offset) {
            state.currentdocumentoffset = offset;
        },
        SET_CURRENT_ANNOTATION_OFFSET: function (state, offset) {
            state.currentannotationoffset = offset;
        },
        SET_CURRENT_CORPUS_HIGHLIGHTS: function (state) {
            for(let i = 0; i < state.corpusresults.length; i++) {
                if(typeof state.corpusresults[i].highlight != 'undefined') {
                    let highlightobject = state.corpusresults[i].highlight;

                    if(!state.corpushighlights.hasOwnProperty(state.corpusresults[i]._id)){
                        state.corpushighlights[state.corpusresults[i]._id] = [];
                    }
                    let tempobject = {}
                    for (let fieldkey in highlightobject) {
                        let tempstring = '';
                        for (let d = 0; d < highlightobject[fieldkey].length;d++) {
                            tempstring += highlightobject[fieldkey][d].concat(' ');
                        }
                        tempobject[fieldkey] = tempstring;
                    }

                    if(!Functions.hasObject(state.corpushighlights[state.corpusresults[i]._id],tempobject)) {
                        state.corpushighlights[state.corpusresults[i]._id].push(tempobject);
                    }

                }
            }
        },
        EMPTY_CURRENT_CORPUS_HIGHLIGHTS: function (state) {
            state.corpushighlights = {}
        },
        SET_CURRENT_DOCUMENT_HIGHLIGHTS: function (state) {
            for(let i = 0; i < state.documentresults.length; i++) {
                if(typeof state.documentresults[i].highlight != 'undefined') {
                    let highlightobject = state.documentresults[i].highlight;

                    if(!state.documenthighlights.hasOwnProperty(state.documentresults[i]._id)){
                        state.documenthighlights[state.documentresults[i]._id] = [];
                    }
                    let tempobject = {}
                    for (let fieldkey in highlightobject) {
                        let tempstring = '';
                        for (let d = 0; d < highlightobject[fieldkey].length;d++) {
                            tempstring += highlightobject[fieldkey][d].concat(' ');
                        }
                        tempobject[fieldkey] = tempstring;
                    }

                    if(!Functions.hasObject(state.documenthighlights[state.documentresults[i]._id],tempobject)) {
                        state.documenthighlights[state.documentresults[i]._id].push(tempobject);
                    }

                }
            }
        },
        EMPTY_CURRENT_DOCUMENT_HIGHLIGHTS: function (state) {
            state.documenthighlights = {}
        },
        SET_CURRENT_ANNOTATION_HIGHLIGHTS: function (state) {
            for(let i = 0; i < state.annotationresults.length; i++) {
                if(typeof state.annotationresults[i].highlight != 'undefined') {
                    let highlightobject = state.annotationresults[i].highlight;

                    if(!state.annotationhighlights.hasOwnProperty(state.annotationresults[i]._id)){
                        state.annotationhighlights[state.annotationresults[i]._id] = [];
                    }
                    let tempobject = {}
                    for (let fieldkey in highlightobject) {
                        let tempstring = '';
                        for (let d = 0; d < highlightobject[fieldkey].length;d++) {
                            tempstring += highlightobject[fieldkey][d].concat(' ');
                        }
                        tempobject[fieldkey] = tempstring;
                    }

                    if(!Functions.hasObject(state.annotationhighlights[state.annotationresults[i]._id],tempobject)) {
                        state.annotationhighlights[state.annotationresults[i]._id].push(tempobject);
                    }

                }
            }
        },
        EMPTY_CURRENT_ANNOTATION_HIGHLIGHTS: function (state) {
            state.annotationhighlights = {}
        },
        SET_MAIN_SEARCH_TERM: function(state,term) {
            state.mainSearchTerm = term;
        },
        SET_FRONT_PAGE_QUERY: function(state,query) {
            state.frontpagequery = query;
        },
        EMPTY_FRONT_PAGE_QUERY: function(state) {
            state.frontpagequery = '';
            window.laudatioApp.frontpagequery = "";
        },
        EMPTY_MAIN_SEARCH_TERM: function(state) {
            state.mainSearchTerm = '';
        },
        SET_MAIN_SEARCH_TERM_HIT: function(state,bool) {
            state.mainSearchTermHit = bool;
        },
        SET_FRONT_PAGE_TERM_HIT: function(state,bool) {
            state.frontPageTermHit = bool;
        },
        SET_CURRENT_ANNOTATION_FILTER: function (state) {
            for(let key in state.annotationFilterData) {
                if (state.annotationFilterData.hasOwnProperty(key)) {
                    if (state.annotationFilterData[key] != 'undefined' && state.annotationFilterData[key] != '') {
                        if (key == "annotation_merged_formats") {
                            for (var formatkey in state.annotationFilterData.annotation_merged_formats) {
                                if(!state.activefilters.includes('A_'+state.annotationFilterData.annotation_merged_formats[formatkey])) {
                                    state.activefilters.push('A_'+state.annotationFilterData.annotation_merged_formats[formatkey]);
                                    state.activefiltersmap['A_'+state.annotationFilterData.annotation_merged_formats[formatkey]] = 'annotation_merged_formats';
                                }
                            }

                        }
                        else {
                            if(key.indexOf('preparation') > -1){
                                for(let i = 0; i < state.annotationFilterData[key].length; i++) {
                                    if(state.annotationFilterData[key][i].indexOf('D_') == -1
                                        && !state.activefilters.includes(state.annotationFilterData[key][i]) && !state.activefiltersmap.hasOwnProperty(state.annotationFilterData[key][i])
                                        && !state.activefilters.includes('A_'+state.annotationFilterData[key][i]) && !state.activefiltersmap.hasOwnProperty('A_'+state.annotationFilterData[key][i])
                                    ) {
                                        state.activefilters.push('A_'+state.annotationFilterData[key][i]);
                                        state.activefiltersmap['A_'+state.annotationFilterData[key][i]] = key;
                                    }
                                }
                            }
                        }
                    }
                }
            }

            var matches = [];

            for(var i = 0; i < state.activefilters.length; i++) {
                var filterkey = state.activefiltersmap[state.activefilters[i]];
                var filtervalue = state.activefilters[i];
                if(!state.activefilterhits.hasOwnProperty(filtervalue)){
                    state.activefilterhits[filtervalue] = 0;
                }
            }//end for activefilters

        },
        SET_CURRENT_DOCUMENT_FILTER: function (state) {
            for(let key in state.documentFilterData) {
                if (state.documentFilterData.hasOwnProperty(key)) {
                    if (state.documentFilterData[key] != 'undefined' && state.documentFilterData[key] != '') {
                        if (key == "document_size_extent" || key == "document_publication_publishing_date" || key == "document_size_extent_to" || key == 'document_publication_publishing_date_to') {

                            if(key == "document_size_extent"  && state.documentFilterData.document_size_extent != ""  && state.documentFilterData.document_size_extent_to != "") {
                                    if(!state.activefilters.includes('D_'+state.documentFilterData.document_size_extent+":"+state.documentFilterData.document_size_extent_to)
                                        && !state.activefiltersmap.hasOwnProperty('D_'+state.documentFilterData.document_size_extent+":"+state.documentFilterData.document_size_extent_to)) {

                                        let fromkey = Functions.getFilterKeyFromMap('document_size_extent', state.activefiltersmap);
                                        let tokey = Functions.getFilterKeyFromMap('document_size_extent_to', state.activefiltersmap);
                                        let filterkey = ""
                                        if(typeof fromkey != "undefined" && fromkey != "") {
                                            filterkey = fromkey;
                                        }
                                        else if(typeof tokey != "" && tokey != "") {
                                            filterkey = tokey;
                                        }

                                        delete state.activefilterhits[filterkey]
                                        state.activefilters = Functions.removeValueFromActivefilters(filterkey, state.activefilters)
                                        state.activefiltersmap = Functions.removeValueFromMap('document_size_extent', state.activefiltersmap)

                                        state.activefilters.push('D_'+state.documentFilterData.document_size_extent+":"+state.documentFilterData.document_size_extent_to);
                                        state.activefiltersmap['D_'+state.documentFilterData.document_size_extent+":"+state.documentFilterData.document_size_extent_to] = key;
                                    }
                            }//end if document_size_extent

                            if(key == "document_size_extent"  && state.documentFilterData.document_size_extent != ""  && state.documentFilterData.document_size_extent_to == "") {

                                if(!state.activefilters.includes('D_'+state.documentFilterData.document_size_extent+":&infin;")
                                    && !state.activefiltersmap.hasOwnProperty('D_'+state.documentFilterData.document_size_extent + ":&infin;")){
                                    let fromkey = Functions.getFilterKeyFromMap('document_size_extent', state.activefiltersmap);
                                    let tokey = Functions.getFilterKeyFromMap('document_size_extent_to', state.activefiltersmap);

                                    let filterkey = ""
                                    if(typeof fromkey != "undefined" && fromkey != "") {
                                        filterkey = fromkey;
                                    }
                                    else if(typeof tokey != "" && tokey != "") {
                                        filterkey = tokey;
                                    }
                                    delete state.activefilterhits[filterkey]
                                    state.activefilters = Functions.removeValueFromActivefilters(filterkey, state.activefilters)
                                    state.activefiltersmap = Functions.removeKeyFromMap(filterkey, state.activefiltersmap)

                                    state.activefilters.push('D_'+state.documentFilterData.document_size_extent+ ":&infin;");
                                    state.activefiltersmap['D_'+state.documentFilterData.document_size_extent+ ":&infin;"] = key;
                                }
                            }

                            if(key == "document_size_extent_to"  && state.documentFilterData.document_size_extent == ""  && state.documentFilterData.document_size_extent_to != "") {

                                if(!state.activefilters.includes('D_0:'+state.documentFilterData.document_size_extent_to)
                                    && !state.activefiltersmap.hasOwnProperty('D_0:'+state.documentFilterData.document_size_extent_to)){
                                    let fromkey = Functions.getFilterKeyFromMap('document_size_extent', state.activefiltersmap);
                                    let tokey = Functions.getFilterKeyFromMap('document_size_extent_to', state.activefiltersmap);

                                    let filterkey = ""
                                    if(typeof fromkey != "undefined" && fromkey != "") {
                                        filterkey = fromkey;
                                    }
                                    else if(typeof tokey != "" && tokey != "") {
                                        filterkey = tokey;
                                    }

                                    delete state.activefilterhits[filterkey]
                                    state.activefilters = Functions.removeValueFromActivefilters(filterkey, state.activefilters)
                                    state.activefiltersmap = Functions.removeKeyFromMap(filterkey, state.activefiltersmap)

                                    state.activefilters.push('D_0:'+state.documentFilterData.document_size_extent_to);
                                    state.activefiltersmap['D_0:'+state.documentFilterData.document_size_extent_to] = key;
                                }
                            }

                            if(key == "document_publication_publishing_date" && state.documentFilterData.document_publication_publishing_date != '' && state.documentFilterData.document_publication_publishing_date_to != '') {
                                if(!state.activefilters.includes('D_'+state.documentFilterData.document_publication_publishing_date+":"+state.documentFilterData.document_publication_publishing_date_to)
                                    && !state.activefiltersmap.hasOwnProperty('D_'+state.documentFilterData.document_publication_publishing_date+":"+state.documentFilterData.document_publication_publishing_date_to)) {

                                    let fromkey = Functions.getFilterKeyFromMap('document_publication_publishing_date', state.activefiltersmap);
                                    let tokey = Functions.getFilterKeyFromMap('document_publication_publishing_date_to', state.activefiltersmap);

                                    let filterkey = ""
                                    if(typeof fromkey != "undefined" && fromkey != "") {
                                        filterkey = fromkey;
                                    }
                                    else if(typeof tokey != "" && tokey != "") {
                                        filterkey = tokey;
                                    }
                                    delete state.activefilterhits[filterkey]
                                    state.activefilters = Functions.removeValueFromActivefilters(filterkey, state.activefilters)
                                    state.activefiltersmap = Functions.removeKeyFromMap(filterkey, state.activefiltersmap)

                                    state.activefilters.push('D_'+state.documentFilterData.document_publication_publishing_date+":"+state.documentFilterData.document_publication_publishing_date_to);
                                    state.activefiltersmap['D_'+state.documentFilterData.document_publication_publishing_date+":"+state.documentFilterData.document_publication_publishing_date_to] = key;
                                }
                            }//end if publishing date

                            if(key == "document_publication_publishing_date" && state.documentFilterData.document_publication_publishing_date != '' && state.documentFilterData.document_publication_publishing_date_to == '') {
                                if(!state.activefilters.includes('D_'+state.documentFilterData.document_publication_publishing_date+":&infin;")
                                    && !state.activefiltersmap.hasOwnProperty('D_'+state.documentFilterData.document_publication_publishing_date+":&infin;")){
                                    let fromkey = Functions.getFilterKeyFromMap('document_publication_publishing_date', state.activefiltersmap);
                                    let tokey = Functions.getFilterKeyFromMap('document_publication_publishing_date_to', state.activefiltersmap);

                                    let filterkey = ""
                                    if(typeof fromkey != "undefined" && fromkey != "") {
                                        filterkey = fromkey;
                                    }
                                    else if(typeof tokey != "" && tokey != "") {
                                        filterkey = tokey;
                                    }
                                    delete state.activefilterhits[filterkey]
                                    state.activefilters = Functions.removeValueFromActivefilters(filterkey, state.activefilters)
                                    state.activefiltersmap = Functions.removeKeyFromMap(filterkey, state.activefiltersmap)

                                    state.activefilters.push('D_'+state.documentFilterData.document_publication_publishing_date+":&infin;");
                                    state.activefiltersmap['D_'+state.documentFilterData.document_publication_publishing_date+":&infin;"] = key;
                                }
                            }//end if publishing date

                            if(key == "document_publication_publishing_date_to" && state.documentFilterData.document_publication_publishing_date == '' && state.documentFilterData.document_publication_publishing_date_to != '') {
                                if(!state.activefilters.includes('D_'+"&infin;:"+state.documentFilterData.document_publication_publishing_date_to)
                                    && !state.activefiltersmap.hasOwnProperty('D_'+"&infin;:"+state.documentFilterData.document_publication_publishing_date_to)){

                                    let fromkey = Functions.getFilterKeyFromMap('document_publication_publishing_date', state.activefiltersmap);
                                    let tokey = Functions.getFilterKeyFromMap('document_publication_publishing_date_to', state.activefiltersmap);

                                    let filterkey = ""
                                    if(typeof fromkey != "undefined" && fromkey != "") {
                                        filterkey = fromkey;
                                    }
                                    else if(typeof tokey != "" && tokey != "") {
                                        filterkey = tokey;
                                    }
                                    delete state.activefilterhits[filterkey]
                                    state.activefilters = Functions.removeValueFromActivefilters(filterkey, state.activefilters)
                                    state.activefiltersmap = Functions.removeKeyFromMap(filterkey, state.activefiltersmap)

                                    state.activefilters.push('D_'+"&infin;:"+state.documentFilterData.document_publication_publishing_date_to);
                                    state.activefiltersmap['D_'+"&infin;:"+state.documentFilterData.document_publication_publishing_date_to] = key;
                                }
                            }//end if publishing date

                        }
                        else {
                            if(key.indexOf('document') > -1){
                                for(let i = 0; i < state.documentFilterData[key].length; i++) {
                                    if( state.documentFilterData[key][i].indexOf('D_') == -1
                                        && !state.activefilters.includes(state.documentFilterData[key][i]) && !state.activefiltersmap.hasOwnProperty(state.documentFilterData[key][i])
                                        && !state.activefilters.includes('D_'+state.documentFilterData[key][i]) && !state.activefiltersmap.hasOwnProperty('D_'+state.documentFilterData[key][i])
                                    ) {
                                        state.activefilters.push('D_'+state.documentFilterData[key][i]);
                                        state.activefiltersmap['D_'+state.documentFilterData[key][i]] = key;
                                    }
                                }
                            }
                        }
                    }
                }
            }


            let matches = [];

            let activeFilterCount = Functions.getActiveFilterCount('document',state.activefiltersmap);
            for(var i = 0; i < state.activefilters.length; i++) {
                var filterkey = state.activefiltersmap[state.activefilters[i]];
                var filtervalue = state.activefilters[i];
                if(!state.activefilterhits.hasOwnProperty(filtervalue)){
                    state.activefilterhits[filtervalue] = 0;
                }
            }//end for activefilters


        },
        SET_CURRENT_CORPUS_FILTER: function(state) {
            for (let key in state.corpusFilterData) {
                if (state.corpusFilterData.hasOwnProperty(key)) {
                    if (state.corpusFilterData[key] != 'undefined' && state.corpusFilterData[key] != '') {
                        if(key == "corpus_publication_publication_date" || key == "corpusYearTo"  || key == "corpus_publication_license" || key == "corpus_merged_formats" || key == "corpus_size_value" || key == "corpusSizeTo" ) {
                            if(key == "corpus_size_value" && state.corpusFilterData.corpus_size_value != ""  && state.corpusFilterData.corpusSizeTo != "") {

                                    if(!state.activefilters.includes('C_'+state.corpusFilterData.corpus_size_value+":"+state.corpusFilterData.corpusSizeTo)
                                        && !state.activefiltersmap.hasOwnProperty('C_'+state.corpusFilterData.corpus_size_value+":"+state.corpusFilterData.corpusSizeTo)
                                    ) {

                                        let fromkey = Functions.getFilterKeyFromMap('corpus_size_value', state.activefiltersmap);
                                        let tokey = Functions.getFilterKeyFromMap('corpusSizeTo', state.activefiltersmap);

                                        let filterkey = ""
                                        if(typeof fromkey != "undefined" && fromkey != "") {
                                            filterkey = fromkey;
                                        }
                                        else if(typeof tokey != "" && tokey != "") {
                                            filterkey = tokey;
                                        }

                                        delete state.activefilterhits[filterkey]
                                        state.activefilters = Functions.removeValueFromActivefilters(filterkey, state.activefilters)
                                        state.activefiltersmap = Functions.removeKeyFromMap(filterkey, state.activefiltersmap)

                                        state.activefilters.push('C_'+state.corpusFilterData.corpus_size_value+":"+state.corpusFilterData.corpusSizeTo);
                                        state.activefiltersmap['C_'+state.corpusFilterData.corpus_size_value+":"+state.corpusFilterData.corpusSizeTo] = key;
                                    }
                            }
                            if(key == "corpus_size_value" && state.corpusFilterData.corpus_size_value != ""  && state.corpusFilterData.corpusSizeTo == "") {
                                if(!state.activefilters.includes('C_'+state.corpusFilterData.corpus_size_value+":&infin;")
                                    && !state.activefiltersmap.hasOwnProperty('C_'+state.corpusFilterData.corpus_size_value + ":&infin;")){
                                    let fromkey = Functions.getFilterKeyFromMap('corpus_size_value', state.activefiltersmap);
                                    let tokey = Functions.getFilterKeyFromMap('corpusSizeTo', state.activefiltersmap);

                                    let filterkey = ""
                                    if(typeof fromkey != "undefined" && fromkey != "") {
                                        filterkey = fromkey;
                                    }
                                    else if(typeof tokey != "" && tokey != "") {
                                        filterkey = tokey;
                                    }
                                    delete state.activefilterhits[filterkey]
                                    state.activefilters = Functions.removeValueFromActivefilters(filterkey, state.activefilters)
                                    state.activefiltersmap = Functions.removeKeyFromMap(filterkey, state.activefiltersmap)

                                    state.activefilters.push('C_'+state.corpusFilterData.corpus_size_value+ ":&infin;");
                                    state.activefiltersmap['C_'+state.corpusFilterData.corpus_size_value+ ":&infin;"] = key;
                                }
                            }

                            if(key == "corpusSizeTo" && state.corpusFilterData.corpus_size_value == ""  && state.corpusFilterData.corpusSizeTo != "") {
                                if(!state.activefilters.includes('C_0:'+state.corpusFilterData.corpusSizeTo)
                                    && !state.activefiltersmap.hasOwnProperty('C_0:'+state.corpusFilterData.corpusSizeTo)){
                                    let fromkey = Functions.getFilterKeyFromMap('corpus_size_value', state.activefiltersmap);
                                    let tokey = Functions.getFilterKeyFromMap('corpusSizeTo', state.activefiltersmap);

                                    let filterkey = ""
                                    if(typeof fromkey != "undefined" && fromkey != "") {
                                        filterkey = fromkey;
                                    }
                                    else if(typeof tokey != "" && tokey != "") {
                                        filterkey = tokey;
                                    }

                                    delete state.activefilterhits[filterkey]
                                    state.activefilters = Functions.removeValueFromActivefilters(filterkey, state.activefilters)
                                    state.activefiltersmap = Functions.removeKeyFromMap(filterkey, state.activefiltersmap)

                                    state.activefilters.push('C_0:'+state.corpusFilterData.corpusSizeTo);
                                    state.activefiltersmap['C_0:'+state.corpusFilterData.corpusSizeTo] = key;
                                }
                            }

                            if(key == "corpus_merged_formats" && state.corpusFilterData.corpus_merged_formats != ""){
                                for (let formatkey in state.corpusFilterData.corpus_merged_formats) {
                                    if(
                                        !state.activefilters.includes(state.corpusFilterData.corpus_merged_formats[formatkey])
                                        && !state.activefiltersmap.hasOwnProperty(state.corpusFilterData.corpus_merged_formats[formatkey])
                                        &&
                                        !state.activefilters.includes('C_'+state.corpusFilterData.corpus_merged_formats[formatkey])
                                        && !state.activefiltersmap.hasOwnProperty('C_'+state.corpusFilterData.corpus_merged_formats[formatkey])) {
                                        state.activefilters.push('C_'+state.corpusFilterData.corpus_merged_formats[formatkey]);
                                        state.activefiltersmap['C_'+state.corpusFilterData.corpus_merged_formats[formatkey]] = 'corpus_merged_formats';
                                    }

                                }
                            }
                            if(key == "corpus_publication_license" && state.corpusFilterData[key][0].toLowerCase() != "") {
                                for (let i = 0; i < state.corpusFilterData[key].length; i++) {
                                    if(state.corpusFilterData[key][i].indexOf('C_') == -1
                                        && !state.activefilters.includes(state.corpusFilterData[key][i]) && !state.activefiltersmap.hasOwnProperty(state.corpusFilterData[key][i])
                                        && !state.activefilters.includes('C_'+state.corpusFilterData[key][i]) && !state.activefiltersmap.hasOwnProperty('C_'+state.corpusFilterData[key][i])) {
                                        state.activefilters.push('C_'+state.corpusFilterData[key][i]);
                                        state.activefiltersmap['C_'+state.corpusFilterData[key][i]] = key;
                                    }
                                }
                            }
                            if(key == "corpus_publication_publication_date" && state.corpusFilterData.corpus_publication_publication_date != '' && state.corpusFilterData.corpusYearTo != '') {
                                if(!state.activefilters.includes('C_'+state.corpusFilterData.corpus_publication_publication_date+":"+state.corpusFilterData.corpusYearTo)
                                    && !state.activefiltersmap.hasOwnProperty('C_'+state.corpusFilterData.corpus_publication_publication_date + ":" + state.corpusFilterData.corpusYearTo)) {

                                    let fromkey = Functions.getFilterKeyFromMap('corpus_publication_publication_date', state.activefiltersmap);
                                    let tokey = Functions.getFilterKeyFromMap('corpusYearTo', state.activefiltersmap);

                                    let filterkey = ""
                                    if(typeof fromkey != "undefined" && fromkey != "") {
                                        filterkey = fromkey;
                                    }
                                    else if(typeof tokey != "" && tokey != "") {
                                        filterkey = tokey;
                                    }
                                    delete state.activefilterhits[filterkey]
                                    state.activefilters = Functions.removeValueFromActivefilters(filterkey, state.activefilters)
                                    state.activefiltersmap = Functions.removeKeyFromMap(filterkey, state.activefiltersmap)

                                    state.activefilters.push('C_'+state.corpusFilterData.corpus_publication_publication_date + ":" + state.corpusFilterData.corpusYearTo);
                                    state.activefiltersmap['C_'+state.corpusFilterData.corpus_publication_publication_date + ":" + state.corpusFilterData.corpusYearTo] = key;
                                }
                            }
                            if(key == "corpus_publication_publication_date" && state.corpusFilterData.corpus_publication_publication_date != '' && state.corpusFilterData.corpusYearTo == '') {
                                if(!state.activefilters.includes('C_'+state.corpusFilterData.corpus_publication_publication_date+":&infin;")
                                    && !state.activefiltersmap.hasOwnProperty('C_'+state.corpusFilterData.corpus_publication_publication_date + ":&infin;")) {

                                    let fromkey = Functions.getFilterKeyFromMap('corpus_publication_publication_date', state.activefiltersmap);
                                    let tokey = Functions.getFilterKeyFromMap('corpusYearTo', state.activefiltersmap);

                                    let filterkey = ""
                                    if(typeof fromkey != "undefined" && fromkey != "") {
                                        filterkey = fromkey;
                                    }
                                    else if(typeof tokey != "" && tokey != "") {
                                        filterkey = tokey;
                                    }
                                    delete state.activefilterhits[filterkey]
                                    state.activefilters = Functions.removeValueFromActivefilters(filterkey, state.activefilters)
                                    state.activefiltersmap = Functions.removeKeyFromMap(filterkey, state.activefiltersmap)

                                    state.activefilters.push('C_'+state.corpusFilterData.corpus_publication_publication_date + ":&infin;");
                                    state.activefiltersmap['C_'+state.corpusFilterData.corpus_publication_publication_date + ":&infin;"] = key;
                                }
                            }

                            if(key == "corpusYearTo" && state.corpusFilterData.corpus_publication_publication_date == '' && state.corpusFilterData.corpusYearTo != '') {

                                if(!state.activefilters.includes('C_'+"&infin;:"+state.corpusFilterData.corpusYearTo)
                                    && !state.activefiltersmap.hasOwnProperty('C_'+ "&infin;:" + state.corpusFilterData.corpusYearTo)) {

                                    let fromkey = Functions.getFilterKeyFromMap('corpus_publication_publication_date', state.activefiltersmap);
                                    let tokey = Functions.getFilterKeyFromMap('corpusYearTo', state.activefiltersmap);

                                    let filterkey = ""
                                    if(typeof fromkey != "undefined" && fromkey != "") {
                                        filterkey = fromkey;
                                    }
                                    else if(typeof tokey != "" && tokey != "") {
                                        filterkey = tokey;
                                    }
                                    delete state.activefilterhits[filterkey]
                                    state.activefilters = Functions.removeValueFromActivefilters(filterkey, state.activefilters)
                                    state.activefiltersmap = Functions.removeKeyFromMap(filterkey, state.activefiltersmap)

                                    state.activefilters.push('C_'+ "&infin;:" + state.corpusFilterData.corpusYearTo);
                                    state.activefiltersmap['C_'+"&infin;:" + state.corpusFilterData.corpusYearTo] = key;
                                }
                            }
                        }
                        else{
                            if(key.indexOf('corpus') > -1) {
                                for (let i = 0; i < state.corpusFilterData[key].length; i++) {
                                    if (state.corpusFilterData[key][i].indexOf('C_') === -1
                                        && !state.activefilters.includes(state.corpusFilterData[key][i]) && !state.activefiltersmap.hasOwnProperty(state.corpusFilterData[key][i])
                                        && !state.activefilters.includes('C_'+state.corpusFilterData[key][i]) && !state.activefiltersmap.hasOwnProperty('C_'+state.corpusFilterData[key][i])) {
                                        state.activefilters.push('C_'+state.corpusFilterData[key][i]);
                                        state.activefiltersmap['C_'+state.corpusFilterData[key][i]] = key;
                                    }
                                }

                            }
                        }
                    }
                }
            }

            let matches = [];

            let activeFilterCount = Functions.getActiveFilterCount('corpus',state.activefiltersmap);
            for(let i = 0; i < state.activefilters.length; i++) {
                let filterkey = state.activefiltersmap[state.activefilters[i]];

                if(typeof filterkey != 'undefined' && filterkey != 'undefined') {
                    let filtervalue = state.activefilters[i];
                    if(!state.activefilterhits.hasOwnProperty(filtervalue)){
                        state.activefilterhits[filtervalue] = 0;
                    }
                }//end if filterkey


            }//end for activefilters

        },
        RESET_CORPUS_RESULT_VISIBILITY: function(state){
            for(let i = 0; i < state.corpusresults.length; i++) {
                if(typeof state.filteredcorpushighlightmap[state.corpusresults[i]._id] != 'undefined') {
                    state.corpusresults[i]._source.visibility = 1;
                    state.visibleCorpora.push(state.corpusresults[i])
                    state.corpusresultcounter++;
                }
                else{
                    state.corpusresults[i]._source.visibility = 0;
                }
            }


            if(state.visibleCorpora.length == 0) {
                for(let j = 0; j < state.corpusresults.length; j++) {
                    if(state.corpusresults[j]._source.visibility == 0) {
                        state.corpusresults[j]._source.visibility = 1;
                    }
                    state.corpusresultcounter++;
                    state.visibleCorpora.push(state.corpusresults[j])
                }
            }

        },
        RESET_DOCUMENT_RESULT_VISIBILITY: function(state){
            for(let i = 0; i < state.documentresults.length; i++) {
                if(typeof state.filtereddocumenthighlightmap[state.documentresults[i]._id] != 'undefined') {
                    state.documentresults[i]._source.visibility = 1;
                    state.visibleDocuments.push(state.documentresults[i])
                    state.documentresultcounter++;
                }
                else{
                    state.documentresults[i]._source.visibility = 0;
                }
            }

            if(state.visibleDocuments.length == 0) {
                for(let j = 0; j < state.documentresults.length; j++) {
                    if(state.documentresults[j]._source.visibility == 0) {
                        state.documentresults[j]._source.visibility = 1;
                    }
                    state.documentresultcounter++;
                    state.visibleDocuments.push(state.documentresults[j])
                }
            }
        },
        RESET_ANNOTATION_RESULT_VISIBILITY: function(state){
            for(let i = 0; i < state.annotationresults.length; i++) {
                if(typeof state.filtereddocumenthighlightmap[state.annotationresults[i]._id] != 'undefined') {
                    state.annotationresults[i]._source.visibility = 1;
                    state.visibleAnnotations.push(state.annotationresults[i])
                    state.annotationresultcounter++;
                }
                else{
                    state.annotationresults[i]._source.visibility = 0;
                }
            }

            if(state.visibleAnnotations.length == 0) {
                for(let j = 0; j < state.annotationresults.length; j++) {
                    if(state.annotationresults[j]._source.visibility == 0) {
                        state.annotationresults[j]._source.visibility = 1;
                    }
                    state.annotationresultcounter++;
                    state.visibleAnnotations.push(state.annotationresults[j])
                }
            }
        },
        UPDATE_CORPUS_RESULT_VISIBILITY: function(state,data){
            let hits = []

            for(let i = 0; i < data.length; i++) {
                hits.push(data[i]._id)
                state.corpusdescriptioncollapsed.push(false)

                if(typeof data[i].highlight != 'undefined') {
                    let highlightobject = data[i].highlight;

                    if(!state.filteredcorpushighlightmap.hasOwnProperty(data[i]._id)){
                        state.filteredcorpushighlightmap[data[i]._id] = [];
                        if(!state.corpusFormatFilterMarkup.hasOwnProperty(data[i]._id)) {
                            state.corpusFormatFilterMarkup[data[i]._id] = []
                        }
                    }

                    let tempobject = {}

                    for (let fieldkey in highlightobject) {
                        let tempstring = '';

                        for (let d = 0; d < highlightobject[fieldkey].length;d++) {

                            if(fieldkey == "corpus_merged_formats") {
                                if(!state.corpusFormatFilterMarkup[data[i]._id].includes(highlightobject[fieldkey][d])){
                                    state.corpusFormatFilterMarkup[data[i]._id].push(highlightobject[fieldkey][d])
                                }


                                if(state.corpusFormatFilterMarkup[data[i]._id].length > 0) {
                                    for(let e = 0; e < state.corpusFormatFilterMarkup[data[i]._id].length; e++) {
                                        if(tempstring.indexOf(state.corpusFormatFilterMarkup[data[i]._id][e]) < 0){
                                            tempstring += state.corpusFormatFilterMarkup[data[i]._id][e].concat(' ');
                                        }

                                        let strippedmarkup = state.corpusFormatFilterMarkup[data[i]._id][e].replace(/<[^>]+>/g, '');
                                        if(state.corpusFilterData.hasOwnProperty(fieldkey) && !state.corpusFilterData[fieldkey].includes(strippedmarkup.trim())) {
                                            state.corpusFilterData[fieldkey].push(strippedmarkup.trim())
                                        }
                                    }
                                }
                            }
                            else {
                                tempstring += highlightobject[fieldkey][d].concat(' ');
                            }
                        }

                        tempobject[fieldkey] = tempstring;
                    }

                    state.filteredcorpushighlightmap[data[i]._id] = Functions.filteredMapHasObject(state.filteredcorpushighlightmap[data[i]._id],tempobject);


/*
                    if(!Functions.hasObject(state.filteredcorpushighlightmap[data[i]._id],tempobject)) {
                        state.filteredcorpushighlightmap[data[i]._id].push(tempobject);
                    }
*/

                }

                if(data[i]._source.visibility == 1) {
                    state.visibleCorpora.push(data[i])
                }
            }

            state.corpusresultcounter = state.visibleCorpora.length
        },
        SET_CORPORAAREFILTERED_STATE: function(state, bool){
            state.corporaarefiltered = bool
        },
        SET_DOCUMENTSAREFILTERED_STATE: function(state, bool){
            state.documentsarefiltered = bool
        },
        SET_ANNOTATIONSAREFILTERED_STATE: function(state, bool){
            state.annotationsarefiltered = bool
        },
        UPDATE_DOCUMENT_RESULT_VISIBILITY: function(state,data){
            let hits = []
            for(let i = 0; i < data.length; i++) {
                hits.push(data[i]._id)

                if(typeof data[i].highlight != 'undefined') {
                    let highlightobject = data[i].highlight;

                    if(!data[i].hasOwnProperty(data[i]._id)){
                        state.filtereddocumenthighlightmap[data[i]._id] = [];
                    }

                    let tempobject = {}
                    for (let fieldkey in highlightobject) {
                        let tempstring = '';

                        for (let d = 0; d < highlightobject[fieldkey].length;d++) {
                            tempstring += highlightobject[fieldkey][d].concat(' ');
                        }
                        tempobject[fieldkey] = tempstring;
                    }

                    if(!Functions.hasObject(state.filtereddocumenthighlightmap[data[i]._id],tempobject)) {
                        state.filtereddocumenthighlightmap[data[i]._id].push(tempobject);
                    }
                }

                if(data[i]._source.visibility == 1) {
                    state.visibleDocuments.push(data[i])
                }
            }

            state.documentresultcounter = state.visibleDocuments.length

        },
        UPDATE_ANNOTATION_RESULT_VISIBILITY: function(state,data){
            /*
            annotationFormatFilterMarkup : {
            annotation_merged_formats: []
        }
             */
            let hits = []
            for(let i = 0; i < data.length; i++) {

                if(typeof  data[i]._source != 'undefined') {

                    if(typeof data[i].highlight != 'undefined') {
                        let highlightobject = data[i].highlight;

                        if(!data[i].hasOwnProperty(data[i]._id)){
                            state.filteredannotationhighlightmap[data[i]._id] = [];

                            if(!state.annotationFormatFilterMarkup.hasOwnProperty(data[i]._id)) {
                                state.annotationFormatFilterMarkup[data[i]._id] = []
                            }

                        }
                        let tempobject = {}
                        for (let fieldkey in highlightobject) {
                            let tempstring = '';

                            for (let d = 0; d < highlightobject[fieldkey].length;d++) {

                                if(fieldkey == "annotation_merged_formats") {

                                    if(!state.annotationFormatFilterMarkup[data[i]._id].includes(highlightobject[fieldkey][d])){
                                        state.annotationFormatFilterMarkup[data[i]._id].push(highlightobject[fieldkey][d])
                                    }

                                    if(state.annotationFormatFilterMarkup[data[i]._id].length > 0) {
                                        for(let e = 0; e < state.annotationFormatFilterMarkup[data[i]._id].length; e++) {
                                            if(tempstring.indexOf(state.annotationFormatFilterMarkup[data[i]._id][e]) < 0){
                                                tempstring += state.annotationFormatFilterMarkup[data[i]._id][e].concat(' ');
                                            }

                                            let strippedmarkup = state.annotationFormatFilterMarkup[data[i]._id][e].replace(/<[^>]+>/g, '');
                                            if(state.annotationFilterData.hasOwnProperty(fieldkey) && !state.annotationFilterData[fieldkey].includes(strippedmarkup.trim())) {
                                                state.annotationFilterData[fieldkey].push(strippedmarkup.trim())
                                            }
                                        }
                                    }

                                }
                                else{
                                    tempstring += highlightobject[fieldkey][d].concat(' ');
                                }
                            }
                            tempobject[fieldkey] = tempstring;
                        }

                        if(!Functions.hasObject(state.filteredannotationhighlightmap[data[i]._id],tempobject)) {
                            state.filteredannotationhighlightmap[data[i]._id].push(tempobject);
                        }
                    }


                    if(data[i]._source.visibility == 1) {
                        state.visibleAnnotations.push(data[i])
                    }
                }
            }

            state.annotationresultcounter = state.visibleAnnotations.length
        },
        RESET_CORPUS_FILTER: function(state, payload) {
            let key = payload.key
            let filter = payload.filter

            if(state.corpusFilterData.hasOwnProperty(key)) {
                if(key != "corpus_size_value"
                    && key != "corpusSizeTo"
                    && key != "corpus_publication_publication_date"
                    && key != "corpusYearTo"
                ){
                    for(let i = 0; i < state.corpusFilterData[key].length; i++ ) {
                        let content = filter.substring(filter.indexOf("C_")+2)

                        if(state.corpusFilterData[key][i] == content){
                            state.corpusFilterData[key].splice(i,1);
                            break;
                        }
                    }
                }
                else {
                    if(key == "corpus_size_value"){
                        state.corpusFilterData[key] = ""
                        state.corpusFilterData["corpusSizeTo"] = ""
                    }
                    else if(key == "corpusSizeTo"){
                        state.corpusFilterData[key] = ""
                        state.corpusFilterData["corpus_size_value"] = ""
                    }
                    else if(key == "corpus_publication_publication_date"){
                        state.corpusFilterData[key] = ""
                        state.corpusFilterData["corpusYearTo"] = ""
                    }
                    else if(key == "corpusYearTo"){
                        state.corpusFilterData[key] = ""
                        state.corpusFilterData["corpus_publication_publication_date"] = ""
                    }
                }
            }
        },
        RESET_CORPUS_FORMAT: function(state, payload) {
            let key = payload.key
            let filter = payload.filter

            if(state.corpusFilterData.hasOwnProperty(key)){
                for(let i = 0; i < state.corpusFilterData[key].length; i++ ) {

                    if(state.corpusFilterData[key][i] == filter){
                        state.corpusFilterData[key].splice(state.corpusFilterData[key].indexOf(filter),1);
                        break;
                    }
                }
            }
        },
        RESET_ANNOTATION_FORMAT: function(state, payload) {
            let key = payload.key
            let filter = payload.filter

            if(state.annotationFilterData.hasOwnProperty(key)){
                for(let i = 0; i < state.annotationFilterData[key].length; i++ ) {
                    if(state.annotationFilterData[key][i] == filter){
                        state.annotationFilterData[key].splice(state.annotationFilterData[key].indexOf(filter),1);
                        break;
                    }
                }
            }
        },
        RESET_DOCUMENT_FILTER: function(state, payload) {
            let key = payload.key
            let filter = payload.filter

            if(state.documentFilterData.hasOwnProperty(key)){
                if(key != "document_publication_publishing_date" &&
                    key != "document_publication_publishing_date_to"
                    && key != "document_size_extent"
                    && key != "document_size_extent_to") {
                    for(let i = 0; i < state.documentFilterData[key].length; i++ ) {
                        let content = filter.substring(filter.indexOf("D_")+2)
                        if(state.documentFilterData[key][i] == content){
                            state.documentFilterData[key].splice(i,1);
                            break;
                        }
                    }
                }
                else{
                    if(key == "document_publication_publishing_date") {
                        state.documentFilterData.document_publication_publishing_date = "";
                        if(state.documentFilterData.document_publication_publishing_date_to != "") {
                            state.documentFilterData.document_publication_publishing_date_to = "";
                        }

                    }
                    if(key == "document_publication_publishing_date_to") {
                        state.documentFilterData.document_publication_publishing_date_to = "";
                    }

                    if(key == "document_size_extent") {
                        state.documentFilterData['document_size_extent'] = "";
                        if(state.documentFilterData['document_size_extent_to'] != "") {
                            state.documentFilterData['document_size_extent_to'] = "";
                        }

                    }

                    if(key == "document_size_extent_to") {
                        state.documentFilterData['document_size_extent_to'] = "";
                    }
                }
            }
         }
        ,
        RESET_ANNOTATION_FILTER: function(state, payload) {
            let key = payload.key
            let filter = payload.filter

            if(state.annotationFilterData.hasOwnProperty(key)){

                for(let i = 0; i < state.annotationFilterData[key].length; i++ ) {
                    let content = filter.substring(filter.indexOf("A_")+2)
                    if(state.annotationFilterData[key][i] == content){
                        state.annotationFilterData[key].splice(i,1);
                        break;
                    }
                }
            }
        },
        RESET_ALL_FILTERS: function(state) {
            for (let key in state.corpusFilterData){
                if(key == "corpus_publication_publication_date") {
                    state.corpusFilterData['corpus_publication_publication_date'] = "";
                }
                else if(key == "corpusYearTo") {
                    state.corpusFilterData['corpusYearTo'] = "";
                }
                else if(key == " corpus_size_value") {
                    state.corpusFilterData['corpus_size_value'] = "";
                }
                else if(key == " corpusSizeTo") {
                    state.corpusFilterData['corpusSizeTo'] = "";
                }
                else{
                    state.corpusFilterData[key] = []
                }

            }

            for (let dkey in state.documentFilterData){
                if(dkey == "document_publication_publishing_date") {
                    state.documentFilterData['document_publication_publishing_date'] = "";
                    state.documentFilterData['document_publication_publishing_date_to'] = "";
                }
                else if(dkey == "document_merged_authors" || dkey == "document_merged_languages" || dkey == "document_publication_place" || dkey == "document_title") {
                    state.documentFilterData[dkey] = []
                }
                else{
                    state.documentFilterData[dkey] = ""
                }

            }

            for (let akey in state.annotationFilterData){
                state.annotationFilterData[akey] = []
            }

            for(let corpusId in state.filteredcorpushighlightmap) {
                delete state.filteredcorpushighlightmap[corpusId];
            }

            for(let corpusMarkupId in state.corpusFormatFilterMarkup) {
                delete state.corpusFormatFilterMarkup[corpusMarkupId]
            }



            for(let documentId in state.filtereddocumenthighlightmap) {
                delete state.filtereddocumenthighlightmap[documentId];
            }

            for(let annotationId in state.filteredannotationhighlightmap) {
                delete state.filteredannotationhighlightmap[annotationId];
            }

            for(let annotationMarkupId in state.annotationFormatFilterMarkup) {
                delete state.annotationFormatFilterMarkup[annotationMarkupId];
            }

            for(let activefilterkey in state.activefilterhits) {
                delete state.activefilterhits[activefilterkey]
                delete state.activefiltersmap[activefilterkey]
            }

            state.activefilters = []
            state.visibleCorpora = [];
            state.visibleDocuments = [];
            state.visibleAnnotations = [];
        },
        RESET_ACTIVE_FILTER_HIGHLIGHT: function (state,filter) {
            let key = state.activefiltersmap[filter];
            if(filter.indexOf("C_") > -1) {
                for(let corpusId in state.filteredcorpushighlightmap) {
                    if(key == "corpusYearTo") {
                        key = "corpus_publication_publication_date"
                    }

                    if(key == "corpus_merged_formats") {
                        let filterMarkup = '<span class="laudatiofilterhighlight">' + filter.substring(2) + '</span>';
                        state.corpusFormatFilterMarkup[corpusId] = Functions.removeFilterMarkup(state.corpusFormatFilterMarkup[corpusId],filterMarkup);
                        if(state.filteredcorpushighlightmap[corpusId][0].hasOwnProperty(key)){
                            let returnedMarkup = Functions.removeFilterHighLightMarkup(state.filteredcorpushighlightmap[corpusId][0][key],filterMarkup);
                            if(null != returnedMarkup && returnedMarkup != "undefined") {
                                state.filteredcorpushighlightmap[corpusId][0][key] = returnedMarkup
                            }
                            else{
                                delete state.filteredcorpushighlightmap[corpusId][0][key]
                            }
                        }
                    }
                    else{
                        delete state.filteredcorpushighlightmap[corpusId][0][key]
                    }


                    if(Object.keys(state.filteredcorpushighlightmap[corpusId][0]).length == 0) {
                        delete state.filteredcorpushighlightmap[corpusId];
                    }

                }
            }
            else if(filter.indexOf("D_") > -1) {
                for(let documentId in state.filtereddocumenthighlightmap) {
                    delete state.filtereddocumenthighlightmap[documentId][0][key]
                    if(key == "document_publication_publishing_date" || key == "document_publication_publishing_date_to") {
                        if(state.filtereddocumenthighlightmap[documentId][0].hasOwnProperty("document_history_not_after")){
                            delete state.filtereddocumenthighlightmap[documentId][0]["document_history_not_after"]
                        }

                        if(state.filtereddocumenthighlightmap[documentId][0].hasOwnProperty("document_history_not_before")){
                            delete state.filtereddocumenthighlightmap[documentId][0]["document_history_not_before"]
                        }
                        delete state.filtereddocumenthighlightmap[documentId][0]["document_publication_publishing_date"]
                        delete state.filtereddocumenthighlightmap[documentId][0]["document_publication_publishing_date_to"]
                    }

                    if(Object.keys(state.filtereddocumenthighlightmap[documentId][0]).length == 0) {
                        delete state.filtereddocumenthighlightmap[documentId];
                    }
                }
            }
            else if(filter.indexOf("A_") > -1) {
                for(let annotationId in state.filteredannotationhighlightmap) {

                    if(key == "annotation_merged_formats") {
                        let filterMarkup = '<span class="laudatiofilterhighlight">' + filter.substring(2) + '</span>';
                        state.annotationFormatFilterMarkup[annotationId] = Functions.removeFilterMarkup(state.annotationFormatFilterMarkup[annotationId],filterMarkup);
                        if(state.filteredannotationhighlightmap[annotationId][0].hasOwnProperty(key)){
                            let returnedMarkup = Functions.removeFilterHighLightMarkup(state.filteredannotationhighlightmap[annotationId][0][key],filterMarkup);
                            if(null != returnedMarkup && returnedMarkup != "undefined") {
                                state.filteredannotationhighlightmap[annotationId][key] = returnedMarkup
                            }
                            else{
                                delete state.filteredannotationhighlightmap[annotationId][0][key]
                            }
                        }
                        //state.filteredannotationhighlightmap[annotationId][0][key] = Functions.removeFilterHighLightMarkup(state.filteredannotationhighlightmap[annotationId][0][key],filterMarkup);
                    }
                    else {
                        delete state.filteredannotationhighlightmap[annotationId][0][key]
                    }


                    if(Object.keys(state.filteredannotationhighlightmap[annotationId][0]).length == 0) {
                        delete state.filteredannotationhighlightmap[annotationId];
                    }

                    /*
                    if(typeof state.annotationFormatFilterMarkup[annotationId] != 'undefined' && Object.keys(state.annotationFormatFilterMarkup[annotationId]).length == 0) {
                        delete state.annotationFormatFilterMarkup[annotationId];
                    }
                     */
                }
            }//end if what filter
        },
        DELETE_FROM_ACTIVE_FILTER: function(state, filter) {
            let key = state.activefiltersmap[filter];

            if(typeof filter != 'undefined' && state.activefiltersmap.hasOwnProperty(filter)){
                state.activefilters.splice(state.activefilters.indexOf(filter),1);
                delete state.activefiltersmap[filter];
                delete state.activefilterhits[filter];
            }
        },
        SET_CORPUS_TOKEN_RANGE: function (state, payload) {
            state.corpusFilterData['corpus_size_value'] = payload.from;
            state.corpusFilterData['corpusSizeTo'] = payload.to;
        },
        SET_DOCUMENT_TOKEN_RANGE: function (state, payload) {
            state.documentFilterData['document_size_extent'] = payload.from;
            state.documentFilterData['document_size_extent_to'] = payload.to;
        },
        RESET_ACTIVE_FILTER: function (state, filter) {
            let key = state.activefiltersmap[filter];

            if(typeof key != 'undefined') {
                if(key.indexOf('corpus') > -1) {
                    for(let i = 0; i < state.corpusresults.length; i++) {
                        if (state.corpusresults[i]._source.hasOwnProperty(key)){
                            if(state.corpusresults[i]._source.visibility == 0) {
                                state.corpusresults[i]._source.visibility = 1;
                                state.corpusresultcounter++;
                            }
                        }
                    }


                    let corpusFilterData = {}
                    let j = 0;
                    if(state.activefilters.length > 0) {
                        for(j = j; j < state.activefilters.length; j++) {
                            let active_key = state.activefiltersmap[state.activefilters[j]];
                            if(active_key == 'corpus_merged_formats') {
                                corpusFilterData[active_key] = [state.activefilters[j]];
                            }
                            else if(active_key == 'corpus_size_value') {
                                let numberarray = state.activefilters[j].split(":");
                                corpusFilterData['corpus_size_value'] = numberarray[0].trim().replace("C_","");
                                corpusFilterData['corpusSizeTo'] = numberarray[1].trim();
                            }
                            else if(active_key == 'corpus_publication_publication_date') {
                                let numberarray = state.activefilters[j].split(":");
                                corpusFilterData['corpus_publication_publication_date'] = numberarray[0].trim();
                                corpusFilterData['corpusYearTo'] = numberarray[1].trim();
                            }
                            else{
                                corpusFilterData[active_key] = state.activefilters[j];
                            }

                        }
                    }
                }
                else if(key.indexOf('document') > -1) {
                    for(let i = 0; i < state.documentresults.length; i++) {
                        if (state.documentresults[i]._source.hasOwnProperty(key)){
                            if(state.documentresults[i]._source.visibility == 0) {
                                state.documentresults[i]._source.visibility = 1;
                                state.documentresultcounter++;
                            }
                        }
                    }//end for

                    let documentFilterData = {}
                    let j = 0;
                    if(state.activefilters.length > 0) {
                        for(let j = j; j < state.activefilters.length; j++) {
                            let active_key = state.activefiltersmap[state.activefilters[j]];
                            if(active_key == 'document_size_extent'){
                                let numberarray = state.activefilters[j].split(":");
                                documentFilterData['document_size_extent'] = numberarray[0].trim().replace("D_","");
                                documentFilterData['document_size_extent_to'] = numberarray[1].trim();
                            }
                            else  if(active_key == 'document_publication_publishing_date'){
                                let publishingnumberarray = state.activefilters[j].split(":");
                                documentFilterData['document_publication_publishing_date'] = publishingnumberarray[0].trim();
                                documentFilterData['document_publication_publishing_date_to'] = publishingnumberarray[1].trim();
                            }
                            else{
                                documentFilterData[active_key] = state.activefilters[j];
                            }
                        }
                    }
                }
                else if(key.indexOf('annotation') > -1 || key.indexOf('preparation') > -1) {
                    for(let i = 0; i < state.annotationresults.length; i++) {
                        if (state.annotationresults[i]._source.hasOwnProperty(key)){
                            if(state.annotationresults[i]._source.visibility == 0) {
                                state.annotationresults[i]._source.visibility = 1;
                                state.annotationresultcounter++;
                            }
                        }
                    }//end for

                    let annotationFilterData = {}
                    if(state.activefilters.length > 0) {
                        for(let j = 0; j < state.activefilters.length; j++) {
                            let active_key = state.activefiltersmap[state.activefilters[j]];
                            if(active_key == 'annotation_merged_formats') {
                                annotationFilterData[active_key] = [state.activefilters[j]];
                            }
                            else{
                                annotationFilterData[active_key] = state.activefilters[j];
                            }

                        }
                    }
                }
            }

        },
    },
})
