@extends('layouts.admin_ux', ['isLoggedIn' => $isLoggedIn])

@section('content')
    <div class="container-fluid">
        <div class="container">
            <div class="alert alert-dismissible fade show" role="alert" id="alert-laudatio">
                <span class="alert-laudatio-message"></span>
                <button type="button" class="close" data-dismiss="alert" aria-label="Close">
                    <span aria-hidden="true">&times;</span>
                </button>
            </div>
            @if (session('error'))
                <div class="alert alert-danger alert-dismissible fade show" role="alert" id="alert-laudatio-danger">
                    <span class="alert-laudatio-message">
                        We are really sorry, but something unexpected happened:  <br />
                    <ul>
                        @foreach (session('error') as $errorid => $error)
                            <li>{{lcfirst($errorid)}} {{ $error[0] }}</li>
                        @endforeach
                    </ul>
                    </span>
                    <button type="button" class="close" data-dismiss="alert" aria-label="Close">
                        <span aria-hidden="true">&times;</span>
                    </button>
                </div>
            @endif
            <div class="row mt-5">
                <div class="col-3 mt-2">
                    <nav class="sidebar text-14 nav flex-column border-top border-light mt-7" role="navigation">´
                        <a class="font-weight-normal text-uppercase py-3 px-0 border-bottom border-light nav-link active">Role administration</a>
                        <a class="font-weight-normal text-uppercase py-3 px-0 border-bottom border-light nav-link" href="{{ route('admin.permissions.index') }}">Permission administration</a>
                        <a class="font-weight-normal text-uppercase py-3 px-0 border-bottom border-light nav-link" href="{{ route('admin.roles.assignsuperusers') }}">Assign roles to users</a>
                    </nav>
                </div>
                <div class="col">
                    <form  method="POST" action="/admin/roles/{{$role->id}}" class="w-100 roleupdateform">
                        @method('PATCH')
                        @csrf
                        <div class="row">
                            <div class="col-8 pl-7 pr-7">
                                <div class="d-flex justify-content-between mt-7 mb-3">
                                    <div class="h3 font-weight-normal"><i class="fa fa-key"></i> <em> Edit Role: {{$role->name}}</em> </div>
                                </div>
                                <div class="form-group">
                                    <label class="text-14 text-dark-trans mb-1 pl-3" for="name">
                                        <small>Role name</small>
                                    </label>
                                    <input type="text" class="corpusProject-title-edit form-control" name="name" id="name" required
                                           placeholder="Name of the role?" value="{{$role->name}}">
                                </div>
                                <div class="form-group mt-3">
                                    <label class="text-14 text-dark-trans mb-1 pl-3" for="description">
                                        <small>Role Description</small>
                                    </label>
                                    <textarea id="description" name="description" class="corpusProject-description-edit form-control py-3"
                                              cols="30" rows="8" placeholder="Role Description">{{$role->description}}</textarea>
                                </div>
                                <div class="form-row mt-3">
                                    <div class="col offset-7">
                                        <a href="{{ route('admin.useradmin.role.index') }}"><button type="button" class="role-endEdit btn btn-outline-corpus-mid text-uppercase font-weight-bold rounded w-100">Cancel</button></a>
                                    </div>
                                    <div class="col">
                                        <button type="submit"  class="role-saveEdit btn btn-primary rounded text-uppercase font-weight-bold w-100">Save</button>
                                    </div>
                                </div>
                            </div>
                            <div class="col-4">
                                <div class="d-flex justify-content-between mt-7 mb-3">
                                    <div class="h3 font-weight-normal">Edit permission assigments</div>
                                </div>
                                <div class="custom-control custom-checkbox">
                                    <input type="checkbox" class="custom-control-input" id="selectAll_permission">
                                    <label class="custom-control-label text-14" for="selectAll_permission">
                                        Select / Deselect all
                                    </label>
                                </div>
                                <br />
                                @foreach ($permissions as $permission)
                                    <div class="custom-control custom-checkbox">
                                        @if(in_array ($permission->id,$role_permissions))
                                            <input type="checkbox" class="custom-control-input" name="permissions[]" value="{{$permission->id}}" id="permissionItem_{{$permission->id}}" checked="checked">
                                            <label class="custom-control-label font-weight-bold" for="permissionItem_{{$permission->id}}">
                                                {{$permission->name}}
                                            </label>
                                        @else
                                            <input type="checkbox" class="custom-control-input" name="permissions[]" value="{{$permission->id}}" id="permissionItem_{{$permission->id}}">
                                            <label class="custom-control-label font-weight-bold" for="permissionItem_{{$permission->id}}">
                                                {{$permission->name}}
                                            </label>
                                        @endif
                                    </div>
                                @endforeach
                            </div>
                        </div>
                    </form>


                    <div class="col-lg-12">
                        @include('layouts.errors')
                    </div>
                </div>
            </div>
        </div>
    </div>

@endsection