@extends('layouts.project_ux', ['isLoggedIn' => $isLoggedIn])

@section('content')
    <div class="container-fluid bg-bluegrey-mid bsh-1" {{ Session::has('notification') ? 'data-notification' : '' }} data-notification-type='{{ Session::get('alert_type', 'info') }}' data-notification-message='{{ json_encode(Session::get('message')) }}'>
        <div class="container pt-5">
            <div class="alert alert-dismissible fade show" role="alert" id="alert-laudatio">
                <span class="alert-laudatio-message"></span>
                <button type="button" class="close" data-dismiss="alert" aria-label="Close">
                    <span aria-hidden="true">&times;</span>
                </button>
            </div>
            @if (session('error'))
                <div class="alert alert-danger alert-dismissible fade show" role="alert" id="alert-laudatio-danger">
                    <span class="alert-laudatio-message">
                        We are really sorry, but something unexpected happened:  <br />
                    <ul>
                        @foreach (session('error') as $errorid => $error)
                            <li>{{lcfirst($errorid)}} {{ $error[0] }}</li>
                        @endforeach
                    </ul>
                    </span>
                    <button type="button" class="close" data-dismiss="alert" aria-label="Close">
                        <span aria-hidden="true">&times;</span>
                    </button>
                </div>
            @endif
            <div class="row">
                <div class="col-2 pl-7 pr-7">
                    @if (isset($corpus->corpus_logo))
                        <img class="rounded-circle bg-white w-100"  src="/images/corpuslogos/{{$corpus_data['project_directorypath']}}_{{$corpus->directory_path}}_{{$corpus->corpus_logo}}" alt="corpus-logo">
                    @else
                        <img class="rounded-circle bg-white w-100"  src="/images/placeholder_circle.svg" alt="circle-image">
                    @endif
                </div>
                <div class="col">
                    <small class="text-14 text-grey">
                        {{$corpus_data['project_name']}}
                    </small>
                    <h3 class="h3 font-weight-bold text-grey">
                        @if(strpos($corpus_data['name'], "Untitled") > -1)
                            no name defined yet
                        @else
                            {{$corpus_data['name']}}
                            @if(is_numeric($corpus_data['version']))
                                Version {{$corpus_data['version']}}
                            @else
                                <div id="newWorkflowBadge" class="horizontalBadge bg-blueadmin text-12 font-weight-bold text-white rounded bsh-1"><span>NEW WORKING VERSION</span></div>
                            @endif
                        @endif
                    </h3>

                    <div class="mt-1">
                        <div class="corpusProp text-14 d-flex align-items-center align-self-start pr-1 my-1 flex-nowrap">
                            <i class="fas fa-user fa-lg mr-2"></i>
                            <span>
                                {{$corpus_data['corpus_admin']['user_name']}} ({{$corpus_data['corpus_admin']['role_name']}})
                            </span>
                        </div>
                    </div>
                    <div class="mt-4">
                        <div class="corpusProp text-14 d-flex align-items-center align-self-start pr-1 my-1 ">
                            @if(!is_numeric($corpus_data['version']))
                                <div class="text-grey corpusProp text-14 d-flex align-items-center align-self-start pr-1 my-1 flex-nowrap">
                                    <i class="fas fa-bell text-primary fa-2x font-weight-bold p-2 "></i>
                                    <em>You are now editing a new working version. You are seeing data of the currently published version. <br /> <strong>Upload a new Corpus header in order to publish a new version</strong></em>
                                </div>
                            @endif
                        </div>
                    </div>
                </div>
                @if($corpus_data['headerdata']['corpusheader'] == 1)
                <div class="col-2">
                    <div class="card text-white bg-transparent">
                        <h6 class="corpus-title h6 text-uppercase text-12 text-wine-trans">
                            Corpus
                        </h6>
                        <div class="card-body d-flex flex-column">
                            <a href="/browse/corpus/{{$corpus->elasticsearch_id}}/{{$corpus->elasticsearch_index}}" class="btn btn-primary font-weight-bold text-uppercase rounded small">
                                Preview
                            </a>
                            @if (Auth::user()->hasPermissionTo('Can create corpus','web') && $corpus_data['version'] != "working_period")
                                <button id="publishCorpusButton" class=" btn btn-primary font-weight-bold text-uppercase rounded small mt-3" data-toggle="modal"
                                        data-target="#publishCorpusModal">
                                    Publish
                                </button>
                            @endif
                        </div>
                    </div>
                </div>
                @endif
            </div>
            <div class="row mt-5">

                <nav class="navbar navbar-expand-sm navbar-light bg-transparent p-0 container" role="tablist">

                    <div class="navbar-nav nav row w-100 px-5">

                        <div class="nav-item maintablink col-2 text-center text-14 font-weight-bold active">
                            <a class=" nav-link maintablink text-dark text-uppercase " data-toggle="tab" href="#editCorpus" role="tab">
                                Upload / Edit
                                <div class="labelBadge badge mx-1 py-1 " style="margin-top: 4px">&nbsp;</div>
                            </a>
                        </div>

                        <div class="nav-item maintablink col-auto text-center text-14 font-weight-bold ">
                            <a class="nav-link maintablink text-dark text-uppercase " data-toggle="tab" href="#messageBoard" role="tab">Message Board
                                <div class="labelBadge badge bg-white border border-corpus-dark rounded mx-1 py-1 ">
                                    <i class="fa fa-comment-o fa-fw fa-edit align-text-top fa-lg text-wine"></i>
                                    <span class="text-primary text-14 font-weight-bold" id="messageboardCount">{{$corpus_data['messagecount']}}</span>
                                </div>
                            </a>
                        </div>

                        <div class="nav-item maintablink col-auto text-center text-14 font-weight-bold ">
                            <a class="nav-link maintablink text-dark text-uppercase" data-toggle="tab" href="#corpusCollaborators" role="tab">Collaborators
                                <div class="labelBadge badge bg-white border border-corpus-dark rounded mx-1 py-1 ">
                                    <i class="fa fa-user fa-fw fa-edit align-text-middle fa-lg text-wine"></i>
                                    <span class="text-primary text-14 font-weight-bold" id="collaboratorCount">{{count($corpus_data['user_roles'])}}</span>
                                </div>
                            </a>
                        </div>

                    </div>

                </nav>

            </div>
        </div>
    </div>


    <!-- Tab panes -->

    <div class="container-fluid tab-content content">

        <div role="tabpanel"  class="tab-pane active" id="editCorpus">
            @include('project.corpus.headerFiles')
            <input type="hidden" name="auth_user_name" id="auth_user_name" value="{{Auth::user()->name}}" />
            <input type="hidden" name="auth_user_id" id="auth_user_id" value="{{Auth::user()->id}}" />
            <input type="hidden" name="auth_user_email" id="auth_user_email" value="{{Auth::user()->email}}" />
        </div>
        <div role="tabpanel"  class="tab-pane fade in" id="messageBoard">
            @include('project.corpus.messageboard')
        </div>
        <div role="tabpanel"  class="tab-pane fade in" id="corpusCollaborators">
            <div class="container-fluid">
                <div id="collaboratorcontainer" class="container">
                    <div class="row">

                        <div class="col">
                            <div class="d-flex justify-content-between mt-7 mb-3">
                                <h3 class="h3 font-weight-normal">Corpus Collaborators</h3>
                                @if (Auth::user()->hasPermissionTo('Can assign users to corpus','web'))
                                    <button id="inviteUsersButton" class=" btn btn-primary font-weight-bold text-uppercase rounded" data-toggle="modal"
                                            data-target="#searchInviteUsersModal">
                                        Invite Collaborator
                                    </button>

                                @endif
                            </div>

                            <table class="documents-table message-table table table-bluegrey-dark  table-striped" id="collaboratortable">
                                <thead class="bg-bluegrey-mid">
                                <tr class="text-14 text-grey-light">
                                    <th scope="col">Collaborator</th>
                                    <th scope="col">Role</th>
                                    <th scope="col">Institute</th>
                                    @if (Auth::user()->hasPermissionTo('Can assign users to corpus','web'))
                                        <th scope="col">Delete</th>
                                    @else
                                        <th scope="col" colspan="2">&nbsp;</th>
                                    @endif
                                </tr>
                                </thead>
                                <tbody>
                                @foreach($corpus_data['user_roles'] as $user_id => $role_datas)
                                    @foreach($role_datas as $role_data)
                                        <tr>
                                            <td>
                                                @if ($corpus_data['authuser_role_mappings']['corpus_roles'][$corpus->id] == 3 || $corpus_data['authuser_role_mappings']['corpus_roles'][$corpus->id] == 2)
                                                    @if($role_data['role_id'] > 2 && $user_id != $user->id)
                                                    <div class="custom-control custom-checkbox">
                                                        <input type="checkbox" class="custom-control-input" id="collaboratorEditItem|{{$user_id}}|{{$role_data['role_id']}}|{{$role_data['user_name']}}">

                                                        <label class="custom-control-label font-weight-bold" for="collaboratorEditItem|{{$user_id}}|{{$role_data['role_id']}}|{{$role_data['user_name']}}">
                                                            <i class="fa fa-user fa-fw"></i>
                                                            {{$role_data['user_name']}}
                                                        </label>
                                                    </div>
                                                    @else
                                                        <label class="custom-control-label font-weight-bold" for="collaboratorEditItem|{{$user_id}}|{{$role_data['role_id']}}|{{$role_data['user_name']}}">
                                                            <i class="fa fa-user fa-fw"></i>
                                                            {{$role_data['user_name']}}
                                                        </label>
                                                    @endif
                                                @else
                                                    <label class="custom-control-label font-weight-bold" for="collaboratorEditItem|{{$user_id}}|{{$role_data['role_id']}}|{{$role_data['user_name']}}">
                                                        <i class="fa fa-user fa-fw"></i>
                                                        {{$role_data['user_name']}}
                                                    </label>
                                                @endif
                                            </td>

                                            @if ($corpus_data['authuser_role_mappings']['corpus_roles'][$corpus->id] == 3 || $corpus_data['authuser_role_mappings']['corpus_roles'][$corpus->id] == 2)
                                                <td>
                                                    @if($role_data['role_id'] > 2 && $user_id != $user->id)
                                                        {{ Form::select('role', $corpus_data['roles'], $role_data['role_id'],['class' => 'custom-select custom-select-sm font-weight-bold text-uppercase collaboratorselect','data-user-id' => $user_id]) }}
                                                    @else
                                                        <span class="font-weight-bold text-uppercase">{{$corpus_data['roles'][$role_data['role_id']]}}</span>
                                                    @endif
                                                </td>
                                            @else
                                                <td>
                                                    <span class="font-weight-bold text-uppercase">{{$corpus_data['roles'][$role_data['role_id']]}}</span>
                                                </td>
                                            @endif
                                            <td class="text-14 text-grey-light">{{$role_data['user_affiliation']}}</td>
                                            @if ($corpus_data['authuser_role_mappings']['corpus_roles'][$corpus->id] == 3 || $corpus_data['authuser_role_mappings']['corpus_roles'][$corpus->id] == 2)
                                                <td>
                                                    @if($role_data['role_id'] > 2 && $user_id != $user->id)
                                                    <a href="javascript:" id="collaboratorDeleteItem-{{$user_id}}-{{$role_data['role_id']}}-{{$role_data['user_name']}}">
                                                        <i class="fa fa-trash-o fa-fw fa-lg text-dark collaboratorDeleteTrashcan"></i>
                                                    </a>
                                                    @else
                                                        <a href="#"></a>
                                                    @endif
                                                </td>
                                            @else
                                                <td>
                                                    <a href="#"></a>
                                                </td>
                                            @endif
                                        </tr>
                                    @endforeach
                                @endforeach
                                </tbody>
                                <tfoot class="bg-bluegrey-mid">
                                <tr>
                                    <td colspan="2">
                                        @if (Auth::user()->hasPermissionTo('Can assign users to corpus','web') && $corpus_data['authuser_role_mappings']['corpus_roles'][$corpus->id] == 3 || $corpus_data['authuser_role_mappings']['corpus_roles'][$corpus->id] == 2)
                                            <div class="custom-control custom-checkbox">
                                                <input type="checkbox" class="custom-control-input" id="selectAll_collaboratorEdit">
                                                <label class="custom-control-label text-14" for="selectAll_collaboratorEdit">
                                                    Select all
                                                </label>
                                            </div>
                                        @else
                                            <div class="custom-control custom-checkbox">&nbsp;</div>
                                        @endif
                                    </td>
                                    <td colspan="2">
                                        @if (Auth::user()->hasPermissionTo('Can assign users to corpus','web') && $corpus_data['authuser_role_mappings']['corpus_roles'][$corpus->id] == 3 || $corpus_data['authuser_role_mappings']['corpus_roles'][$corpus->id] == 2)
                                            <button class="float-right disabled btn btn-outline-corpus-dark font-weight-bold text-uppercase btn-sm" id="deleteSelectedCollaboratorsButton">
                                                Delete Selected Collaborators
                                            </button>
                                        @else
                                            &nbsp;
                                        @endif

                                    </td>
                                </tr>
                                </tfoot>
                            </table>
                        </div>
                    </div>
                    @if (Auth::user()->hasPermissionTo('Can assign users to corpus','web') && count($corpus_data['invitedusers']) > 0 && $corpus_data['authuser_role_mappings']['corpus_roles'][$corpus->id] == 3 || $corpus_data['authuser_role_mappings']['corpus_roles'][$corpus->id] == 2)
                        <div class="row">
                            <div class="col">
                                <div class="d-flex justify-content-between mt-7 mb-3">
                                    <h3 class="h3 font-weight-normal">Pending Corpus Collaborator invites</h3>
                                </div>
                                <table class="documents-table message-table table table-bluegrey-dark  table-striped" id="pendinginvitetable">
                                    <thead class="bg-bluegrey-mid">
                                    <tr class="text-14 text-grey-light">
                                        <th scope="col">Collaborator</th>
                                        <th scope="col">Role</th>
                                        <th scope="col">Institute</th>
                                        <th scope="col">Delete</th>
                                    </tr>
                                    </thead>
                                    <tbody>
                                    @if (Auth::user()->hasPermissionTo('Can assign users to corpus','web') && $corpus_data['authuser_role_mappings']['corpus_roles'][$corpus->id] == 3 || $corpus_data['authuser_role_mappings']['corpus_roles'][$corpus->id] == 2)
                                        @foreach($corpus_data['invitedusers'] as $inviteduser)
                                            <tr id="inviteDeleteItem-{{$inviteduser['user_id']}}-{{$inviteduser['user_role_id']}}">
                                                <td>
                                                    <label class="custom-control-label font-weight-bold text-grey-light">
                                                        <i class="fa fa-user fa-fw"></i>
                                                        {{$inviteduser['user_name']}}
                                                    </label>
                                                </td>
                                                <td>
                                                    <span class="font-weight-bold text-grey-light">{{$inviteduser['user_role_name']}}</span>
                                                </td>
                                                <td class="text-14 text-grey-light">{{$inviteduser['user_affiliation']}}</td>
                                                <td>
                                                    <a href="javascript:" data-invite="inviteDeleteItem-{{$inviteduser['user_id']}}-{{$inviteduser['user_role_id']}}-{{$inviteduser['user_name']}}">
                                                        <i class="fa fa-trash-o fa-fw fa-lg text-dark inviteDeleteTrashcan"></i>
                                                    </a>
                                                </td>
                                            </tr>
                                        @endforeach
                                    @endif
                                    </tbody>
                                    <tfoot class="bg-bluegrey-mid">
                                        <tr><td colspan="4">&nbsp;</td></tr>
                                    </tfoot>
                                </table>
                        </div>
                    </div>
                    @endif
                </div>
            </div>
        </div>
    </div>
    <div class="modal fade" id="inviteUsersModal" tabindex="-1" role="dialog" aria-labelledby="inviteUsersModalTitle" aria-hidden="true">
        <div class="modal-dialog modal-dialog-centered modal-lg" role="document">
            <div class="modal-content">
                <div class="modal-header">
                    <div class="h3 modal-title mt-3 w-75" id="inviteUsersModalTitle">Choose the users you want to invite</div>
                    <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                        <span aria-hidden="true">&times;</span>
                    </button>
                </div>
                <div class="modal-body"></div>
                <div class="modal-footer">
                    <button class="btn btn-outline-corpus-dark font-weight-bold text-uppercase rounded px-5" data-dismiss="modal"
                            aria-label="Close">
                        Cancel
                    </button>
                    <button type="button" class="btn btn-primary font-weight-bold text-uppercase rounded px-5" data-dismiss="modal" data-toggle="modal" id="doInvite" data-idkey="">Invite</button>
                </div>
            </div>
        </div>
    </div>
    <div class="modal fade" id="searchInviteUsersModal" tabindex="-1" role="dialog" aria-labelledby="searchInviteUsersModalTitle" aria-hidden="true">
        <div class="modal-dialog modal-dialog-centered modal-lg" role="document">
            <div class="modal-content">
                <div class="modal-header">
                    <div class="h3 modal-title mt-3 w-75" id="searchInviteUsersModalTitle">Search users you want to invite by name</div>
                    <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                        <span aria-hidden="true">&times;</span>
                    </button>
                </div>
                <div class="modal-body" id="searchbar">
                    <div class="form-group">
                        <label class="text-14 text-dark-trans mb-1 pl-3" for="collaborator_user_name">
                            <i class="fa fa-user fa-fw"></i> User name: </label>
                        </label>
                        <input type="text" class="usertypeahead typeahead form-control" id="collaborator_user_name" name="collaborator_user_name" required placeholder="What's the name of the user?" autocomplete="off">
                    </div>
                </div>
                <div class="modal-footer">
                    <button class="btn btn-outline-corpus-dark font-weight-bold text-uppercase rounded px-5" data-dismiss="modal"
                            aria-label="Close">
                        Cancel
                    </button>
                    <button type="button" class="btn btn-primary font-weight-bold text-uppercase rounded px-5"  id="doInvite" data-idkey="">Invite</button>
                </div>
            </div>
        </div>
    </div>
    <div class="modal fade" id="formatModal" tabindex="-1" role="dialog" aria-labelledby="formatModalTitle" aria-hidden="true">
        <div class="modal-dialog modal-dialog-centered modal-lg" role="document">
            <div class="modal-content">
                <div class="modal-header">
                    <div class="h3 modal-title mt-3 w-75" id="formatModalTitle">Choose the format of the uploaded file</div>
                    <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                        <span aria-hidden="true">&times;</span>
                    </button>
                </div>
                <div class="modal-body">
                    <div class="alert alert-dismissible fade show" role="alert" id="alert-formatmodal">
                        <span class="alert-formatmodal-message"></span>
                        <button type="button" class="close" data-dismiss="alert" aria-label="Close">
                            <span aria-hidden="true">&times;</span>
                        </button>
                    </div>
                    {{ Form::label('formatselect', 'Format', null,['class' => 'text-14 text-dark-trans'])}} &nbsp;

                    {{ Form::select('formatselect', $corpus_data['formats'], null, ['class' => 'text-14 text-dark-trans custom-select custom-select-sm font-weight-bold text-uppercase formatselect','data-corpus-id' => $corpus->id,'placeholder' => 'Select a format']) }}
                    <br /><br />
                    <div class="panel-group" id="accordion" role="tablist" aria-multiselectable="true">

                        <div class="panel panel-default">
                            <div class="panel-heading" role="tab" id="headingOne">
                                <h4 class="panel-title">
                                    <a role="button" data-toggle="collapse" data-parent="#accordion" href="#collapseOne" aria-expanded="true" aria-controls="collapseOne">
                                        <i class="fas fa-angle-down text-primary font-weight-bold"></i>
                                        Or create a new format category
                                    </a>
                                </h4>
                            </div>
                            <div id="collapseOne" class="panel-collapse collapse" role="tabpanel" aria-labelledby="headingOne">
                                <div class="panel-body">
                                    <div class="form-group">
                                        <label class="text-14 text-dark-trans" for="format_name">
                                            <small>Format Title</small>
                                        </label>
                                        <input type="text" class="corpusProject-title-edit form-control" name="format_name" id="format_name" placeholder="What's the name of the format?" value="">
                                    </div>
                                    <div class="form-row mt-3">
                                        <div class="col">
                                            <button class="format-create btn btn-secondary rounded text-uppercase font-weight-bold w-15 pull-right">Create</button>
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>

                </div>
                <div class="modal-footer">
                    <button class="btn btn-outline-corpus-dark font-weight-bold text-uppercase rounded px-5" data-dismiss="modal" id="cancelFormatModal"
                            aria-label="Close">
                        Cancel
                    </button>
                    <button type="button" class="btn btn-primary font-weight-bold text-uppercase rounded px-5" data-dismiss="modal" data-toggle="modal" id="setFormat">Choose</button>
                </div>
            </div>
        </div>
    </div>
    <div class="form-group">
        <input type="hidden"  name="currentFormat" id="currentFormat" value="">
    </div>
    <div class="modal fade" id="publishCorpusModal" tabindex="-1" role="dialog" aria-labelledby="publishCorpusModalTitle"
         aria-hidden="true">
        <div class="modal-dialog " role="document">
            <div class="modal-content border-0 rounded-lg bsh-1">

                <div class="modal-body px-5">
                    <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                        <i class="fa fa-close" aria-hidden="true"></i>
                    </button>
                    <div id="publish-error-message" class="alert alert-danger" role="alert"></div>
                    <h3 class="h3 modal-title mt-3 w-75" id="publishCorpusModalTitle">
                    </h3>

                    <p class="mt-3 mb-1" id="publishCorpusModalSubtitle">
                    </p>

                    <div id="publishCorpusModalSubtitleContent"></div>

                </div>
                <div class="modal-footer bg-corpus-light px-4 rounded-lg-bt">
                    <button class="btn btn-outline-corpus-dark font-weight-bold text-uppercase rounded px-5" data-dismiss="modal"
                            aria-label="Close">
                        Cancel
                    </button>
                    <button class="btn btn-primary font-weight-bold text-uppercase rounded px-5" data-dismiss="modal"
                            data-toggle="modal" data-target="#publishSuccessCorpusModal" id="doPublish">
                        Publish
                    </button>
                </div>
            </div>
        </div>
    </div>
    <div class="modal fade" id="confirmDelete" tabindex="-1" role="dialog" aria-labelledby="confirmDeleteTitle" aria-hidden="true">
        <div class="modal-dialog modal-dialog-centered" role="document">
            <div class="modal-content">
                <div class="modal-header">
                    <div class="h3 modal-title mt-3 w-75" id="confirmDeleteTitle"></div>
                    <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                        <span aria-hidden="true">&times;</span>
                    </button>
                </div>
                <div class="modal-body"></div>
                <div class="modal-footer">
                    <button class="btn btn-outline-corpus-dark font-weight-bold text-uppercase rounded px-5" data-dismiss="modal"
                            aria-label="Close">
                        Cancel
                    </button>
                    <button type="button" class="btn btn-primary" data-dismiss="modal" data-toggle="modal" id="doDelete" data-idkey="" data-callback="">Delete</button>
                    <input type="hidden" data- id="postDeleteData" value="" />
                </div>
            </div>
        </div>
    </div>
    <div class="modal fade" id="pleaseWaitDialog" tabindex="-1" role="dialog" aria-labelledby="myModalLabel" aria-hidden="true">
        <div class="modal-dialog">
            <div class="modal-content">
                <div class="modal-header">
                    <h1>Processing...</h1>
                </div>
                <div class="modal-body">
                    <div class="progress">
                        <div id="dynamicprogressbar" class="progress-bar progress-bar-info progress-bar-striped" role="progressbar" aria-valuenow="0" aria-valuemin="0" aria-valuemax="100" style="width: 0%">
                            <span id="current-progress"></span>
                        </div>
                        <div class="processProgressBar"></div >
                    <div class="processPercent">0%</div >
                    <div class="processstatus"></div>
                    </div>
                    <div class="progress" style="display:none;">
                        <div class="progress-bar progress-bar-success progress-bar-striped
active" role="progressbar"
                             aria-valuemin="0" aria-valuemax="100" style="width:0%">
                            0%
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </div>
@endsection