@extends('layouts.project_ux', ['isLoggedIn' => $isLoggedIn])
@section('content')
    <div class="container-fluid">
        <div class="container">
            <div class="row mt-5">
                <div class="col-3 mt-2">
                    <nav class="sidebar text-14 nav flex-column border-top border-light mt-7" role="navigation">
                        <a class="font-weight-normal text-uppercase py-3 px-0 border-bottom border-light nav-link active">Corpus projects</a>
                        <a class="font-weight-normal text-uppercase py-3 px-0 border-bottom border-light nav-link" href="{{ route('corpusProject.invitations') }}">Invitations @if(count($invitations) > 0)<span class="badge badge-pill badge-light pull-right" >{{count($invitations)}}</span>@endif </a>
                        @if ($user->hasPermissionTo("Can create corpus project","web"))
                            <a class="font-weight-normal text-uppercase py-3 px-0 border-bottom border-light nav-link" href="{{ route('corpusProject.create') }}">Initiate Project</a>
                        @endif
                        @if ($user->hasPermissionTo('Administer the application','web'))
                            <a class="font-weight-normal text-uppercase py-3 px-0 border-bottom border-light nav-link" href="{{ route('admin.roles.index') }}">Administer Roles and Permissions</a>
                        @endif
                        @if ($user->isSuperUser())
                            <a class="font-weight-normal text-uppercase py-3 px-0 border-bottom border-light nav-link" href="{{ route('registerldapuser') }}">Create LDAPUser</a>
                        @endif
                    </nav>
                </div>

                <div class="col">
                    <div class="alert alert-dismissible fade show" role="alert" id="alert-laudatio">
                        <span class="alert-laudatio-message"></span>
                        <button type="button" class="close" data-dismiss="alert" aria-label="Close">
                            <span aria-hidden="true">&times;</span>
                        </button>
                    </div>
                    @if (Session::has('msg'))
                        <div class="alert alert-success alert-dismissible alert-success fade show" role="alert" id="alert-laudatio">
                            <span class="alert-laudatio-message">{{ Session::get('msg') }}</span>
                            <button type="button" class="close" data-dismiss="alert" aria-label="Close">
                                <span aria-hidden="true">&times;</span>
                            </button>
                        </div>
                    @endif
                    @if (session('error'))
                        <div class="alert alert-danger alert-dismissible fade show" role="alert" id="alert-laudatio-danger">
                                <span class="alert-laudatio-message">
                                     We are really sorry, but something unexpected happened: <br />
                                <ul>
                                    @foreach (session('error') as $errorid => $error)
                                        <li>{{lcfirst($errorid)}} : {{ $error[0] }}</li>
                                    @endforeach
                                </ul>
                                </span>
                            <button type="button" class="close" data-dismiss="alert" aria-label="Close">
                                <span aria-hidden="true">&times;</span>
                            </button>
                        </div>
                    @endif
                    <div class="d-flex justify-content-between  ">
                        <div class="h3 font-weight-normal">Corpus Projects ({{$corpusProjectCount}}) / Corpora ({{$corpusAccessCount}})</div>
                    </div>

                    <div class="mt-4">
                        @foreach($corpusProjects as $corpusProjectId => $corpusProject)
                            @if(isset($project_access_mapping[$corpusProjectId][$user->id]))
                                <div id="corpusProject_{{$corpusProjectId}}" class="corpusProject container bg-bluegrey-middark mt-1 mb-1 p-5">

                                    <div class="row corpusProject-save">
                                        <div class="col">
                                            <small class="text-14 text-grey">
                                                Corpus Project
                                            </small>

                                            <div class="h4 font-weight-bold corpusProject-title" id="corpusProject-title_{{$corpusProjectId}}">
                                                {{$corpusProject['name']}}
                                            </div>
                                            <div class="mt-2">
                                                @if (count($corpusProject['user_roles']) > 0)
                                                    @foreach($corpusProject['user_roles'] as $projectRole)
                                                        <div class="corpusProp text-14 d-flex align-items-center align-self-start pr-1 my-1 flex-nowrap">
                                                            <i class="fas fa-user fa-lg mr-2"></i>
                                                            <span>{{$projectRole['user_name']}} ({{$projectRole['role_name']}})</span>
                                                        </div>
                                                    @endforeach
                                                @endif
                                            </div>

                                            <p class="mt-3 text-14 mb-0 corpusProject-description" id="corpusProject-description_{{$corpusProjectId}}">
                                                {{$corpusProject['description']}}
                                            </p>

                                        </div>
                                        <div class="col-2 p-0 mr-2">
                                            @if (Auth::user()->hasPermissionTo('Can edit corpus project','web'))
                                                <button  class="corpusProject-startEdit btn btn-outline-corpus-dark font-weight-bold text-uppercase rounded mb-4 w-100">
                                                    Edit project
                                                </button>
                                            @endif
                                            @if (Auth::user()->hasPermissionTo('Can create corpus','web'))
                                                <a href="{{ route('corpus.create',['corpusproject' => $corpusProjectId]) }}" class="btn btn-outline-corpus-dark font-weight-bold text-uppercase rounded mb-4 w-100">
                                                    Add Corpus
                                                </a>
                                            @endif
                                        </div>
                                    </div>
                                    <div class="row">
                                        <div class="col">&nbsp;</div>
                                        <div class="col-2 p-0 mr-2">
                                            @if (Auth::user()->hasPermissionTo('Can edit corpus project','web'))
                                                <a  class="btn btn-outline-corpus-dark font-weight-bold text-uppercase rounded mb-4 w-100" href="/corpusprojects/assignusers/{{$corpusProjectId}}" role="button" aria-expanded="false">
                                                    Manage users
                                                </a>
                                            @endif
                                        </div>
                                    </div>

                                    <div class="row corpusProject-edit hidden">
                                        <form id="editCorpusProject_{{$corpusProjectId}}" method="POST" action="api/adminapi/updateCorpusProject/{{$corpusProjectId}}" class="w-100 updateform">
                                            <div class="form-group">
                                                <label class="text-14 text-dark-trans mb-1 pl-3" for="corpusproject_name_{{$corpusProjectId}}">
                                                    <small>Corpus project Title</small>
                                                </label>
                                                <input type="text" class="corpusProject-title-edit form-control" name="corpusproject_name_{{$corpusProjectId}}" id="corpusproject_name_{{$corpusProjectId}}" required
                                                       placeholder="What's the project title?" value="{{$corpusProject['name']}}">
                                            </div>
                                            <div class="form-group mt-3">
                                                <label class="text-14 text-dark-trans mb-1 pl-3" for="corpusproject_description">
                                                    <small>Corpus Project Description</small>
                                                </label>
                                                <textarea id="corpusproject_description_{{$corpusProjectId}}" name="corpusproject_description_{{$corpusProjectId}}" class="corpusProject-description-edit form-control py-3"
                                                          cols="30" rows="8" placeholder="here you can describe the purpose of your project. We recommend between 300 and max. 500 chars.">{{$corpusProject['description']}}</textarea>
                                            </div>
                                            <div class="form-row mt-3">
                                                <div class="col offset-7">
                                                    <button class="corpusProject-endEdit btn btn-outline-corpus-mid text-uppercase font-weight-bold rounded w-100">Cancel</button>
                                                </div>
                                                <div class="col">
                                                    <button  class="corpusProject-saveEdit btn btn-primary rounded text-uppercase font-weight-bold w-100">Save</button>
                                                </div>
                                            </div>
                                        </form>
                                    </div>
                                </div>
                                <!-- CORPO-->
                                @foreach($corpusProject['corpora'] as $corpus)
                                    @if (isset($corpus_access_mapping[$corpusProjectId."_".$corpus['id']]) && $corpus['workflow_status'] == 0)
                                        <div class="container bg-bluegrey-midlight mt-1 mb-1 p-5">
                                            <div class="row">
                                                <div class="col-2 pl-4">
                                                    @if (isset($corpus['corpus_logo']))
                                                        <img class="rounded-circle bg-white w-100"  src="/images/corpuslogos/{{$corpusProject['directory_path']}}_{{$corpus['corpus_directorypath']}}_{{$corpus['corpus_logo']}}" alt="corpus-logo">
                                                    @else
                                                        <img class="rounded-circle bg-white w-100"  src="/images/placeholder_circle.svg" alt="circle-image">
                                                    @endif
                                                </div>
                                                <div class="col">
                                                    <small class="text-14 text-grey">
                                                        Corpus
                                                    </small>
                                                    <div class="h4 font-weight-bold">
                                                        <a class="text-dark" href="{{route('corpus.edit',['corpus' => $corpus['id']])}}">
                                                            @if ($corpus['publication_version'] == "working_period")
                                                                {{$corpus['name']}}
                                                                <div id="newWorkflowBadge" class="horizontalBadge bg-blueadmin text-12 font-weight-bold text-white rounded bsh-1"><span>NEW WORKING VERSION</span></div>
                                                            @else
                                                                {{$corpus['name']}} Version {{$corpus['publication_version']}}
                                                            @endif
                                                        </a>
                                                    </div>
                                                    @if (count($corpus['user_roles']) > 0)
                                                        @foreach($corpus['user_roles'] as $corpusRole)
                                                            <div class="corpusProp text-14 d-flex align-items-center align-self-start pr-1 my-1 flex-nowrap">
                                                                <i class="fas fa-user fa-lg mr-2"></i>
                                                                <span>{{$corpusRole['user_name']}} ({{$corpusRole['role_name']}})</span>
                                                            </div>
                                                        @endforeach
                                                    @endif
                                                </div>
                                                <div class="col-2 p-0 mr-2">
                                                    <div class="dropdown">
                                                        <button class="btn btn-outline-corpus-dark dropdown-toggle font-weight-bold text-uppercase rounded mb-4 w-100 text-left"
                                                                type="button" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                                                            Edit
                                                        </button>
                                                        <div class="dropdown-menu" aria-labelledby="dropdownMenuButton">
                                                            <a class="dropdown-item text-14" href="{{ route('corpus.edit',['corpus' => $corpus['id']]) }}">Edit Corpus</a>
<<<<<<< HEAD
                                                            @if(null != $corpus['elasticsearch_id'])
                                                                <a class="dropdown-item text-14" href="{{route('browse.showHeaders.get', ['header' => 'corpus', 'id' => $corpus['elasticsearch_id'], 'index' => $corpus['elasticsearch_index']])}}">Preview Corpus</a>
                                                            @endif
                                                            @if (Auth::user()->isSuperUser() && count($role_mappings['corpus_project_roles']) > 0 && $corpus['workflow_status'] == 0)
=======
                                                            <a class="dropdown-item text-14" href="{{route('browse.showHeaders.get', ['header' => 'corpus', 'id' => $corpus['elasticsearch_id'], 'index' => $corpus['elasticsearch_index']])}}">Preview Corpus</a>
                                                        <!--a class="dropdown-item text-14" href="#" data-toggle="modal" data-target="#publishCorpusModal" data-corpusid="{{$corpus['id']}}" data-corpuspath="{{$corpus['corpuspath']}}" id="publishCorpusButton">Publish Corpus</a-->
                                                            @if (Auth::user()->can('Can delete Corpus') && count($role_mappings['corpus_project_roles']) > 0 && $corpus['workflow_status'] == 0)
>>>>>>> 67c340b91acfaded8bfdfa2b09bc7842e1ecad6a
                                                                <a class="dropdown-item text-14" href="#" data-toggle="modal" data-target="#deleteCorpusModal" data-authusername="{{$user->name}}" data-authuseremail="{{$user->email}}" data-corpusid="{{$corpus['id']}}" data-corpusname="{{$corpus['name']}}" data-corpuspath="{{$corpus['corpuspath']}}" data-corpusprojectid="{{$corpusProjectId}}"  id="checkDeleteCorpusButton">Delete Corpus</a>
                                                            @endif
                                                        </div>
                                                    </div>
                                                </div>
                                            </div>
                                        </div>
                                    @endif
                                @endforeach
                            @endif
                        @endforeach
                    </div>
                </div>
            </div>
        </div>
    </div>
    <div class="modal fade" id="deleteCorpusModal" tabindex="-1" role="dialog" aria-labelledby="deleteCorpusModalTitle"
         aria-hidden="true">
        <div class="modal-dialog " role="document">
            <div class="modal-content border-0 rounded-lg bsh-1">

                <div class="modal-body px-5">
                    <button type="button" class="close" data-dismiss="modal" aria-label="Close">
                        <i class="fa fa-close" aria-hidden="true"></i>
                    </button>
                    <h3 class="h3 modal-title mt-3" id="deleteCorpusModalTitle">
                    </h3>

                    <p class="mt-3 mb-1">
                        It includes:
                    </p>

                    <div id="corpusContent"></div>

                </div>
                <div class="modal-footer bg-corpus-light px-4 rounded-lg-bt">
                    <button class="btn btn-outline-corpus-dark font-weight-bold text-uppercase rounded px-5" data-dismiss="modal"
                            aria-label="Close">
                        Cancel
                    </button>
                    <button class="btn btn-primary font-weight-bold text-uppercase rounded px-5" id="deleteCorpusButton">
                        Delete
                    </button>
                </div>
            </div>
        </div>
    </div>
@endsection