<?php

use Illuminate\Http\Request;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

Route::group(array('prefix' => 'searchapi'), function() {
    Route::get('search/{index}/{field}/{searchterm}','ElasticController@search');
    Route::get('getcorpus','ElasticController@getCorpus');
    Route::get('getdocument','ElasticController@getDocument');
    Route::get('getannotation','ElasticController@getAnnotation');
    Route::get('getpublishedindexes','ElasticController@getPublishedIndexes');
    Route::post('listAllPublished','ElasticController@listAllPublished');
    Route::post('searchCorpus','ElasticController@searchCorpusIndex');
    Route::post('searchDateRange','ElasticController@searchCorpusIndex');
    Route::post('searchDocument','ElasticController@searchDocumentIndex');
    Route::post('searchDocumentWithParam','ElasticController@searchDocumentIndexWithParam');
    Route::post('getSearchTotal','ElasticController@getSearchTotal');
    Route::post('searchAnnotation','ElasticController@searchAnnotationIndex');
    Route::post('getCorpusTitlesByDocument','ElasticController@getCorpusTitlesByDocument');
    Route::post('getCorpusByDocument','ElasticController@getCorpusByDocument');
    Route::post('getDocumentsByCorpus','ElasticController@getDocumentsByCorpus');
    Route::post('getAnnotationsByCorpus','ElasticController@getAnnotationByCorpus');
    Route::post('getDocumentsByAnnotation','ElasticController@getDocumentsByAnnotation');
    Route::post('getCorporaByAnnotation','ElasticController@getCorporaByAnnotation');
    Route::post('getAnnotationsByDocument','ElasticController@getAnnotationsByDocument');
    Route::post('truncateIndex','ElasticController@truncateIndex');
});
Route::middleware(['cors'])->prefix('corpusapi')->group(function() {
    Route::get('corpusprojects','ApiController@getCorpusProjects');
    Route::get('corpusprojects/{id}','ApiController@getCorpusProject');
    Route::get('corpusprojects/{id}/corpora','ApiController@getCorporaByCorpusproject');

    Route::get('corpora','ApiController@getCorpora');
    Route::get('corpora/{id}','ApiController@getCorpus');
    Route::get('corpora/{id}/documents','ApiController@getDocumentsByCorpus');
    Route::get('corpora/{id}/annotations','ApiController@getAnnotationsByCorpus');

    Route::get('documents','ApiController@getDocuments');
    Route::get('documents/{id}','ApiController@getDocument');
    Route::get('documents/{id}/annotations','ApiController@getAnnotationsByDocument');

    Route::get('annotations','ApiController@getAnnotations');
    Route::get('annotations/{id}','ApiController@getAnnotation');
    Route::get('annotations/{id}/documents','ApiController@getDocumentsByAnnotation');
    Route::get('formats','ApiController@getAllFormats');
    Route::post('formats/create','ApiController@createFormat');
});

Route::middleware(['cors'])->prefix('elasticapi/v1')->group(function() {
    Route::get('corpora','ApiController@getElasticCorpora');
    Route::get('corpora/latest/count','ApiController@getCorporaCount');
    Route::get('corpora/latest/{from}/{size}','ApiController@getLatestElasticCorpora');
    Route::get('corpora/{id}/{index}','ApiController@getElasticCorpus');
    Route::get('corpora/{id}/{index}/documents','ApiController@getElasticDocumentsByCorpus');
    Route::get('corpora/{id}/{index}/annotations','ApiController@getElasticAnnotationsByCorpus');
    Route::post('corpora/latest/searchMain','ApiController@corpusSearchMain');
    Route::post('corpora/latest/searchMainFilter','ApiController@corpusSearchMainFilter');
    Route::post('corpora/latest/searchMain/count','ApiController@corpusSearchMainCount');
    Route::post('corpora/latest/searchMainFilter/count','ApiController@corpusSearchMainFilterCount');

    Route::get('documents','ApiController@getElasticDocuments');
    Route::get('documents/latest/count','ApiController@getDocumentsCount');
    Route::get('documents/latest/{from}/{size}','ApiController@getLatestElasticDocuments');
    Route::get('documents/{id}/{index}','ApiController@getElasticDocument');
    Route::get('documents/{id}/{index}/annotations','ApiController@getElasticAnnotationsByDocument');
    Route::post('documents/latest/searchMain','ApiController@documentSearchMain');
    Route::post('documents/latest/searchMain/count','ApiController@documentSearchMainCount');
    Route::post('documents/latest/searchMainFilter','ApiController@documentSearchMainFilter');
    Route::post('documents/latest/searchMainFilter/count','ApiController@documentSearchMainFilterCount');




    Route::get('annotations','ApiController@getElasticAnnotations');
    Route::get('annotations/latest/count','ApiController@getAnnotationsCount');
    Route::get('annotations/latest/{from}/{size}','ApiController@getLatestElasticAnnotations');
    Route::get('annotations/{id}/{index}','ApiController@getElasticAnnotation');
    Route::get('annotations/{id}/{index}/documents','ApiController@getElasticDocumentsByAnnotation');
    Route::post('annotations/latest/searchMain','ApiController@annotationSearchMain');
    Route::post('annotations/latest/searchMainFilter','ApiController@annotationSearchMainFilter');
    Route::post('annotations/latest/searchMain/count','ApiController@annotationSearchMainCount');
    Route::post('annotations/latest/searchMainFilter/count','ApiController@annotationSearchMainFilterCount');
    Route::post('suggestsearch','ApiController@searchMainSuggest');

});


Route::group(array('prefix' => 'dbapi'), function() {
    Route::post('getdatabaseid','ApiController@getDatabaseIdByFilenameAndCorpusId');
});



Route::group(array('prefix' => 'adminapi'), function() {
    Route::post('usersandrolesforcollaboration','CorpusController@usersAndRolesForAssignment');
    Route::post('acceptcorpusinvite','InviteController@storeRelationsByCorpus');
    Route::post('acceptprojectinvite','InviteController@storeRelationsByProject');
    Route::post('storeinvitesbycorpus','InviteController@storeInvitesByCorpus');
    Route::post('storeinvitesbyproject','InviteController@storeInvitesByProject');
    Route::post('deleteinvitesbycorpus','InviteController@deleteInvitesByCorpus');
    Route::post('deleteinvitesbyproject','InviteC   ontroller@deleteInvitesByProject');

    Route::post('updatecollaboratorbycorpus','InviteController@updateRelationsByCorpus');
    Route::post('deletecollaboratorsbycorpus','InviteController@deleteCollaboratorsByCorpus');
    Route::post('deletecollaboratorsbyproject','InviteController@deleteCollaboratorsByProject');
    Route::post('userrolesbyproject','CorpusProjectController@storeRelationsByProject');
    Route::post('userrolesbycorpus','CorpusController@storeRelationsByCorpus');
    Route::post('storeexternalenviromentbycorpus','CorpusController@storeExternalEnvironmentByCorpus');
    Route::post('getexternalenviromentsbycorpus','CorpusController@getExternalEnvironmentByCorpus');
    Route::post('deleteexternalenviromentsbycorpus','CorpusController@deleteExternalEnvironment');
    Route::post('deleterolesbycorpus','CorpusController@deleteRelationsByCorpus');
    Route::post('deleterolesbyproject','CorpusProjectController@deleteRelationsByProject');
    Route::post('deletemultiple','GitRepoController@deleteMultipleFiles');
    Route::post('createFormat','GitRepoController@createFormatFolder');
    Route::post('validateHeaders','GitRepoController@validateCorpus');
    Route::post('preparePublication','PublicationController@preparePublication');
    Route::post('checkCorpusContent','PublicationController@checkCorpusContent');
    Route::post('deleteCorpusContent','CorpusController@destroyCorpusContent');
    Route::post('deleteDocumentContent','DocumentController@destroyDocumentContent');
    Route::post('deleteAnnotationContent','AnnotationController@destroyAnnotationContent');
    Route::post('deleteFormatContent','ApiController@destroyFormatFileContent');
    Route::post('publishCorpus','PublicationController@publishCorpus');
    Route::post('destroyCorpus','CorpusController@destroy');
    Route::post('updateCorpusProject/{corpusProject}','CorpusProjectController@update');
    Route::post('postMessage','MessageBoardController@create');
    Route::post('assignMessage','MessageController@assignMessage');
    Route::post('completeMessage','MessageController@completeMessage');
    Route::post('deleteMessage','MessageController@destroyMessage');
    Route::post('flushFileCache','ApiController@flushFileCache');
    Route::post('userroles','RoleController@storeRelations');
    Route::post('removesuperuser','RoleController@removeSuperUserAssignment');
    Route::post('deleteroles','RoleController@destroy');
    Route::post('deletepermissions','PermissionController@destroy');
    Route::get('externalurls','ApiController@importAnnisLinks');
    Route::get('checkElasticSearchIds/{corpusId}','ApiController@checkElasticSearchIds');
    Route::get('fixElasticSearchIds/{corpusId}','ApiController@fixElasticSearchIds');
    Route::get('clearcache','ApiController@clearCache');

});

Route::group(array('prefix' => 'browseapi'), function() {
    Route::post('scrapeLicenseDeed','ScraperController@scrapeLicenseDeed');
});
